#------------------------------------------
# Основная часть Makefile сборки проектов
#   для Multex-ARM
#
#   © ООО «Сэт Код», 2023 (set-code.ru)
#
#------------------------------------------
# Возможные варианты вызова.
#
# make
# make help
#   Вывод памятки по целям сборки.
#------------------------------------------
MKF_VERSION = Multex-ARM Makefile v.1.06
MKF_DATE = 12.26.2023

#------------------------------------------
# Управление цветом вывода и escape-последовательностями
# Использование:
# echo"$(COL) Some Text $(NCOL)"

COL_ESCAPE = \033
COL_NONE = 0m
COL_BRIGHT_GREEN   = 92m
COL_BRIGHT_BLUE    = 94m
COL_BRIGHT_MAGENTA = 95m
COL_BRIGHT_CYAN    = 96m

COL = $(COL_ESCAPE)[$(COL_BRIGHT_CYAN)
NCOL = $(COL_ESCAPE)[$(COL_NONE)
CLS = clear

# Отключение раскраски для терминалов, не поддерживающих ANSI-последовательности.
ifdef DISABLE_ANSI_ESCAPES
	COL = 
	NCOL =
	CLS = 
endif 

#------------------------------------------
# Функции работы с переменными

# Проверка, что переменная есть и она не пустая
check_variable = $(if $(strip $($1)),,$(error A variable "$1" is not defined))
check_variable_hint = $(if $(strip $($1)),,$(error Переменная "$1" не задана. $2))

# Заполнение строки для подстановки параметра в строку компиляции
add_custom_def = -D$1
add_custom_def_value = -D$1=$($1)

#------------------------------------------
# Параметры сборки

# Имя проекта/библиотеки для использования в именах шаблонных функций.
# PROJ_NAME = 
$(call check_variable,PROJ_NAME)


# Адрес для V3s
# MX_TEXT = 0x41000000
# Адрес для A20 uSd
# MX_TEXT = 0x48000000

# Адрес начала ОЗУ.
# Одинаков для всех поддерживаемых процессоров.
ifndef DRAM_START
DRAM_START		= 0x40000000
endif

# Адрес начала сегмента .text (Запускаемая программ + константы).
# Зависит от используемого загрузчика
ifndef MX_TEXT
MX_TEXT			= 0x48000000
endif

# Адрес начала и конца сегмента .bss вычисляется линкером.
# Адрес начала HEAP-области вычисляется в memlib и располагается за областью .bss.

# Адрес конца ОЗУ (первый адрес за пределами памяти).
# Зависит от используемой платы.
# Lichee, Orange PI - 0x44000000
# Se-Som - 0x80000000
# Banana - 0x7FFFFF00
ifndef MEM_POOL_END
MEM_POOL_END	= 0x80000000
endif

# Количество потоков сборки
ifndef STREAM_COUNT
STREAM_COUNT	= 8
endif


#------------------------------------------
# Пути для файлов проекта

# Исходные коды - папка по умолчанию. Будет создана, если её нет.
ifndef SRC_PATH_DEFAULT
SRC_PATH_DEFAULT	= ./src
endif

# Результаты компиляции
ifndef OUT_PATH
OUT_PATH = ./out
endif

# # Карта памяти в системе - по умолчанию для подключаемых дисков uSD.
# ifndef MICRO_SD_DEV
# MICRO_SD_DEV = /dev/sda1
# endif

# Точка монтирования карты памяти 
ifndef MICRO_SD_PATH
MICRO_SD_PATH	= /mnt/multexTmp
endif


# Логи компиляции
ifndef LOG_PATH
LOG_PATH	= ./log
endif

# Полное имя для сохранения готового бинарника проекта / библиотеки
BIN_PATH	= $(OUT_PATH)/$(PROJ_NAME).bin
LIB_PATH	= $(OUT_PATH)/lib_$(PROJ_NAME).a



#------------------------------------------
# Пути MULTEX

# Папка с текущей версией Multex.
# MULTEX_PATH 	= ../../mainCernelBuild/multex_arm
$(call check_variable,MULTEX_PATH)

# Заголовочные файлы библиотек.
ifndef MULTEX_INC_DIR
MULTEX_INC_DIR 	= $(MULTEX_PATH)/include
endif

# Системные библиотеки.
ifndef MULTEX_LIB_DIR
MULTEX_LIB_DIR 	= $(MULTEX_PATH)/lib
endif

# Системные утилиты.
ifndef MULTEX_BIN_DIR
MULTEX_BIN_DIR 	= $(MULTEX_PATH)/bin
endif

# Особые файлы для компиляции проектов.
ifndef MULTEX_ALL_DIR
MULTEX_ALL_DIR 	= $(MULTEX_INC_DIR)/all
endif



# STARTUP_S_PATH 	= $(MULTEX_INC_DIR)/all/startup.s
# CSTART_C_PATH 	= $(MULTEX_INC_DIR)/all/cstart.c
# Подключение библиотечных функций к проекту.
# FUNREG_C_PATH 	= $(MULTEX_INC_DIR)/all/funreg.c
# Функции-заглушки для формирования статической таблицы символов.
# REPLACER_C_PATH = $(MULTEX_INC_DIR)/all/replacer.c
# Функции работы с таблицей символов после её создания.
# SYMTBLINFO_C_PATH = $(MULTEX_INC_DIR)/all/symtblinfo.c

# Директорий для копирования заголовочных файлов в Мультекс.
ifdef OUT_HEADERS_PATH
OUT_HEADERS_DIR = $(MULTEX_INC_DIR)/$(OUT_HEADERS_PATH)
else
OUT_HEADERS_DIR = $(MULTEX_INC_DIR)
endif


#------------------------------------------
# Локальные файлы для сборки

# Сбор всех путей к файлам исходников
SRC_PATHS += $(SRC_PATH_DEFAULT)
SRC_PATHS += $(MULTEX_EXAMPLES_DIR)

# Рекурсивный сбор файлов в директории проекта
rwildcard=$(foreach d,$(wildcard $(1:=/*)),$(call rwildcard,$d,$2) $(filter $(subst *,%,$2),$d))
# Функция разбирает список и возвращает для каждого элемента:
# - имя файла, если это файл, удовлетворяющий шаблону;
# - рекурсивный список файлов внутри директории, если это имя директория.
frwildcard=$(foreach d,$1,$(if $(strip $(filter $(subst *,%,$2),$d)),$d, $(call rwildcard,$d,$2) ))

# Перебор всех путей с исходниками - создание общего списка исходных файлов
SRC_FILES = $(call frwildcard,$(SRC_PATHS),*.c *.cpp *.s)

# Список всех объектных файлов проекта
Pack_Main 	   	:= $(patsubst %.c, %.o, $(SRC_FILES))
Pack_Main 	   	:= $(patsubst %.s, %.o, $(Pack_Main))
Pack_Main 	   	:= $(patsubst %.cpp, %.o, $(Pack_Main))
localCodePack =  $(Pack_Main)

# Основной набор исходников проекта.
USER_PACK = $(localCodePack)

#------------------------------------------
# Подключаемые библиотеки.

# Подключение системных библиотек
KERNEL_LIB = -l_armkernel -l_sunxi -l_m -l_a64

# Подключение библиотек пользователя
ifndef LIBRARIES
LIBRARIES	=
endif


ALL_LIB_PACK =  $(LIBRARIES) $(KERNEL_LIB)
ALL_LIB =  $(WL)--start-group $(ALL_LIB_PACK) $(WL)--end-group
SYM_PACK = startup.o cstart.o funreg.o

# Функция с именами вида libnameDetails будут прилинкованы из библиотек принудительно
DETAILS_FUNC_DIRECTIVES	:= $(patsubst -l_%, -u %Details, $(ALL_LIB_PACK))


#------------------------------------------
# Параметры сборки

# Папки с заголовочными файлами системных библиотек 
# и библиотек пользователя.
INCLUDES += $(MULTEX_INC_DIR)
# Набор папок с заголовочными файлами для с флагом -I
ALL_INCLUDES_PACK = $(foreach d,$(INCLUDES),-I$(d))

# Пользовательские флаги сборки
ifndef USR_CFLAGS
USR_CFLAGS = 
endif

# Флги оптимизации бинарных и объектных файлов
ifndef FLAG_OPTY
FLAG_OPTY = -O2
endif

# Набор флогов компиляции
# WARNINGS =  -Wall -Wswitch-enum -Wswitch-default -Wimplicit-fallthrough
WARNINGS  =  -Wall -Wextra -Wswitch-default -fanalyzer
# Отключение некоторых предупреждений статического анализатора кода.
WARNINGS +=  -Wno-analyzer-file-leak
WARNINGS +=  -Wno-unused-parameter
WARNINGS +=  -Wno-analyzer-malloc-leak

COMPILATION_CFLAGS += $(WARNINGS) -fno-reorder-functions -std=gnu11 
SYS_CFLAGS =-marm -fno-builtin -mfloat-abi=hard -mcpu=cortex-a8 -mfpu=neon 

# Дефайны распределения памяти - используются только для сборки финального бинарника.
MEMORY_DEFINES += $(call add_custom_def_value,DRAM_START)
MEMORY_DEFINES += $(call add_custom_def_value,MEM_POOL_END)

# Дефайны типа сборки (отладочная/релизная).
debug:   			BUILD_TYPE_DEFINES += $(call add_custom_def,DEBUG)
debug_clean:		BUILD_TYPE_DEFINES += $(call add_custom_def,DEBUG)
lib_debug_clean:	BUILD_TYPE_DEFINES += $(call add_custom_def,DEBUG)

release: 			BUILD_TYPE_DEFINES += $(call add_custom_def,RELEASE) $(call add_custom_def,NDEBUG)
release_clean:		BUILD_TYPE_DEFINES += $(call add_custom_def,RELEASE) $(call add_custom_def,NDEBUG)
lib:				BUILD_TYPE_DEFINES += $(call add_custom_def,RELEASE) $(call add_custom_def,NDEBUG)
lib_clean:			BUILD_TYPE_DEFINES += $(call add_custom_def,RELEASE) $(call add_custom_def,NDEBUG)


# Итоговый набор флагов для компиляции объектников.
BIN_CFLAGS  = $(USR_CFLAGS) $(FLAG_OPTY) $(ALL_INCLUDES_PACK) $(COMPILATION_CFLAGS) $(SYS_CFLAGS) $(MEMORY_DEFINES) $(BUILD_TYPE_DEFINES) -nostdinc -c

# Формирование итоговых параметров сборки для различных файлов.
CFLAGS 	= $(BIN_CFLAGS)
ASFLAGS = $(BIN_CFLAGS)

#------------------------------------------
# Определение команд компилятору

# Выбор компилятора
GNU_PREFIX = arm-linux-gnueabihf-

CC = $(GNU_PREFIX)gcc
CXX = $(GNU_PREFIX)g++

AR = $(GNU_PREFIX)ar
AS = $(GNU_PREFIX)gcc
LD = $(GNU_PREFIX)ld
NM = $(GNU_PREFIX)nm

# Особенности директив компилятора
ifeq ($(GNU_PREFIX), arm-linux-gnueabihf-)
WL =
else
WL = -Wl,
endif

# Дополнительные утилиты
MS = $(MULTEX_BIN_DIR)/makesym
KF = $(MULTEX_BIN_DIR)/kernelFiller

# Совместимость Windows-Linux
ifdef SystemRoot
 # Windows
 RM = del
 RMRF = del
 CP = copy
 CPR = copy
 NP = NUL
 MKDIR = mkdir
 else
 # Linux
 RM = rm -f
 RMRF = rm -rf
 CP = cp
 CPR = cp -r
 NP = /dev/null
 MKDIR = mkdir -p
endif


#------------------------------------------
# Пользовательские скрипты и запуски внешних программ

EXTERNAL_CALL_1 ?= 
EXTERNAL_CALL_2 ?= 
EXTERNAL_CALL_3 ?= 
EXTERNAL_CALL_4 ?= 
EXTERNAL_CALL_5 ?= 

#------------------------------------------
# Цели сборки

.PHONY: help
help: getting_started show_help finalize

# Отладочный вариант сборки.
.PHONY: debug
debug: getting_started prepare_project make_debug clean_temporary add_checksumm print_project_summary print_mxtext finalize

# Основной вариант сборки.
.PHONY: release
release: getting_started prepare_project make_release clean_temporary add_checksumm print_project_summary print_mxtext finalize

# Отладочный вариант сборки с очисткой объектников.
.PHONY: debug_clean
debug_clean: getting_started clean_obj prepare_project make_debug clean_temporary add_checksumm print_project_summary print_mxtext finalize

# Основной вариант сборки с очисткой объектников.
.PHONY: release_clean
release_clean: getting_started clean_obj prepare_project make_release clean_temporary add_checksumm print_project_summary print_mxtext finalize

# Удаление временных и объектных файлов
.PHONY: clean
clean: clean_obj clean_temporary finalize

.PHONY: prepare
prepare: prepare_project finalize

# Сборка одиночного .с файла. Aргумент - .c файл.
compile_c_file  = $(CC) -c $1 $(CFLAGS) -o $(subst .c,.o,$1)
%.o: %.c
	$(call compile_c_file, $(subst .o,.c,$@))

# Сборка одиночного .s файла. Aргумент - .s файл.
compile_s_file  = $(AS) -c -x assembler-with-cpp $1 $(ASFLAGS) -o $(subst .s,.o,$1)  
%.o: %.s
	$(call compile_s_file, $(subst .o,.s,$@))

# Сборка библиотеки
.PHONY: lib
lib: getting_started prepare_project library copy_lib clean_temporary print_library_summary finalize

# Сборка библиотеки с очисткой объектников.
.PHONY: lib_clean
lib_clean: getting_started clean_obj prepare_project library copy_lib clean_temporary print_library_summary finalize

# Отладочный вариант сборки библиотеки с очисткой объектников.
.PHONY: lib_debug_clean
lib_debug_clean: getting_started clean_obj prepare_project library copy_lib clean_temporary print_library_summary finalize

# Копирование библиотеки и заголовочников
.PHONY: copy
copy: getting_started copy_lib finalize

.PHONY: install
install: getting_started flash_mount flash_install flash_umount finalize

.PHONY: installer
installer: getting_started flash_mount flash_installer flash_umount finalize

.PHONY: update
update: getting_started flash_mount flash_update flash_umount finalize

.PHONY: ffmpeg
ffmpeg: getting_started do_ffmpeg finalize

.PHONY: call
call: getting_started do_call_scripts finalize


#------------------------------------------
# Вспомогательные переменные (информация о проекте)

get-current-time = $(shell date +"%Y-%m-%d %T")
current-time := $(get-current-time)

# Номер версии - обновляется перед каждой сборкой.
build-num-file := buildnum.txt
# Файл с экземалярами функций проекта пересобирается каждый раз
# при сборке проекта для поддержании актуальности информации.
info-filename := info

build-num-value 	:= $(shell cat ${build-num-file} 2> $(NP))
# git-branch 			:= $(shell git show -s --format=%D 2> $(NP))
git-branch 			:= $(shell git symbolic-ref --short HEAD 2> $(NP))
git-repo-name		:= $(shell basename `git rev-parse --show-toplevel 2> $(NP)` 2> $(NP))
git-date 			:= $(shell git show -s --format=%ci 2> $(NP))
git-committer-name 	:= $(shell git show -s --format=%cn 2> $(NP))
git-committer-mail 	:= $(shell git show -s --format=%ce 2> $(NP))


#------------------------------------------
# Сценарии сборки

# Вывод помощи 
show_help:
	@echo Возможные цели сборки:
	@echo "  $(COL)make help$(NCOL)       - Вывод справки."
	@echo "  $(COL)make debug$(NCOL)      - Сборка проекта с опцией $(COL)DEBUG$(NCOL)."
	@echo "  $(COL)make release$(NCOL)    - Сборка проекта с опцией $(COL)RELEASE$(NCOL)."
	@echo "  $(COL)make lib$(NCOL)        - Сборка библиотеки и копирование файлов."
	@echo "  $(COL)make copy$(NCOL)       - Копирование файлов библиотеки без сборки."
	@echo "  $(COL)make clean$(NCOL)      - Очистка временных и объектных файлов проекта."
	@echo "  $(COL)make prepare$(NCOL)    - Подкотовка директорий нового проекта."
	@echo "  $(COL)make filename.o$(NCOL) - Компиляция указанного объектного файла без опций $(COL)DEBUG$(NCOL)/$(COL)RELEASE$(NCOL)."
	@echo "  $(COL)make install$(NCOL)    - Копирование всех файлов из папки $(COL)$(OUT_PATH)$(NCOL) на диск $(COL)$(MICRO_SD_PATH)$(NCOL)."
	@echo "  $(COL)make installer$(NCOL)  - Копирование файлов проекта из папки $(COL)$(OUT_PATH)$(NCOL)"
	@echo "			на диск-установщик в папку $(COL)$(MICRO_SD_PATH)/EMMC_CP$(NCOL)."
	@echo "  $(COL)make update$(NCOL)     - Обновление исполняемого файла $(COL)$(PROJ_NAME).bin$(NCOL) на дисеке $(COL)$(MICRO_SD_PATH)$(NCOL)."
	@echo "  $(COL)make ffmpeg$(NCOL)     - Конвертировать все видео (в процессе...)."
	@echo "  $(COL)make call$(NCOL)       - Запуск пользовательских скриптов, определённых в $(COL)EXTERNAL_CALL_$(NCOL)."

# Копирование служебных файлов проекта для отладочной сборки
copy_temporary_files_debug: 
	@$(CP) $(MULTEX_ALL_DIR)/startup.s startup.s > $(NP)
	@$(CP) $(MULTEX_ALL_DIR)/cstart.c cstart.c > $(NP)
	@$(CP) $(MULTEX_ALL_DIR)/funreg.c funreg.c > $(NP)
	@$(CP) $(MULTEX_ALL_DIR)/replacer.c replacer.c > $(NP)
	@$(CP) $(MULTEX_ALL_DIR)/symtblinfo.c symtblinfo.c > $(NP)

# Копирование служебных файлов проекта для релизной сборки
copy_temporary_files_release: 
	@$(CP) $(MULTEX_ALL_DIR)/startup.s startup.s > $(NP)
	@$(CP) $(MULTEX_ALL_DIR)/cstart.c cstart.c > $(NP)
	@$(CP) $(MULTEX_ALL_DIR)/funreg.c funreg.c > $(NP)
	@$(CP) $(MULTEX_ALL_DIR)/symtbl.c symtbl.c > $(NP)

# Параллельная сборка не-служебных объектных файлов.
# Переменная BUILD_TYPE_DEFINES определяется через цель для первичного мейка и не расчитывается для вторичного, поэтому мы отправляем её туда вручную. 
build_obj:
	@$(MAKE) parallel_build -j$(STREAM_COUNT) BUILD_TYPE_DEFINES='$(BUILD_TYPE_DEFINES)'

# Цель для параллельной сборки
parallel_build: $(USER_PACK) 



# Сборка проекта в отладочном режиме с генерацией таблицы символовв
make_debug: copy_temporary_files_debug build_obj
	@echo Удаление предыдущей версии бинарного файла
	@$(RM) $(BIN_PATH)
	@echo Список файлов проекта:
	@echo $(USER_PACK)
	$(call compile_s_file, startup.s)
	$(call compile_c_file, cstart.c)
	$(call compile_c_file, funreg.c)
	$(call compile_c_file, replacer.c)
	$(call compile_c_file, symtblinfo.c)
	$(call compile_c_file, $(info-filename).c)
	@echo Создание таблицы символов
	$(LD) -L$(MULTEX_LIB_DIR) $(DETAILS_FUNC_DIRECTIVES) replacer.o $(SYM_PACK) $(USER_PACK) $(info-filename).o $(ALL_LIB) -o tmp.o
	@$(NM) tmp.o > tmp.sym
	@$(MS) tmp.sym
	@$(RM) tmp.o
	@$(RM) tmp.sym
	$(call compile_c_file, symtbl.c)	
	@echo Сборка проекта
	$(LD) -L$(MULTEX_LIB_DIR) -Ttext $(MX_TEXT) -entry _start $(SYM_PACK) $(USER_PACK) $(info-filename).o symtblinfo.o symtbl.o $(ALL_LIB) -o project_binary -Map ${LOG_PATH}/build.map
	@$(NM) project_binary > project_binary.sym
	@$(GNU_PREFIX)objcopy --gap-fil=0xff -O binary project_binary $(BIN_PATH)

# Сборка проекта без отладки
make_release: copy_temporary_files_release build_obj
	@echo Удаление предыдущей версии бинарного файла
	@$(RM) $(BIN_PATH)
	@echo Список файлов проекта:
	@echo $(USER_PACK)
	$(call compile_s_file, startup.s)
	$(call compile_c_file, cstart.c)
	$(call compile_c_file, funreg.c)
	$(call compile_c_file, symtbl.c)
	$(call compile_c_file, $(info-filename).c)
	@echo Сборка проекта	
	$(LD) -L$(MULTEX_LIB_DIR) -Ttext $(MX_TEXT) -entry _start $(SYM_PACK) $(USER_PACK) $(info-filename).o symtbl.o $(ALL_LIB) -o project_binary -Map ${LOG_PATH}/build.map
	@$(NM) project_binary > project_binary.sym
	@$(GNU_PREFIX)objcopy --gap-fil=0xff -O binary project_binary $(BIN_PATH)

library: build_obj
	@$(RM) $(LIB_PATH)
	$(call compile_c_file, $(info-filename).c)
	@echo Список файлов проекта:
	@echo $(USER_PACK)
	@$(AR) -q $(LIB_PATH) $(USER_PACK) $(info-filename).o

copy_lib:
	@echo Копирование библиотеки по адресу: $(MULTEX_LIB_DIR)
	@$(CP) $(LIB_PATH) $(MULTEX_LIB_DIR)
ifneq ($(strip $(OUT_HEADERS)),)
	@echo Копирование заголовочных файлов по адресу: $(OUT_HEADERS_DIR)
	@$(MKDIR) $(OUT_HEADERS_DIR)
	@$(CP) $(OUT_HEADERS) $(OUT_HEADERS_DIR)
endif

getting_started:
	@$(CLS)
	@echo "$(COL)$(MKF_VERSION)$(NCOL)"
	@echo --

add_checksumm:
	@echo Добавление контрольной суммы к бинарному файлу
	@$(KF) $(BIN_PATH) $(BIN_PATH)

clean_temporary:
	@echo Удаление временных файлов
	@$(RM) startup.s
	@$(RM) cstart.c
	@$(RM) funreg.c
	@$(RM) replacer.c
	@$(RM) symtbl.c
	@$(RM) symtblinfo.c
	@$(RM) $(info-filename).c
	@$(RM) *.o
	@$(RM) project_binary
	@$(RM) *.sym

clean_obj:
	@echo Удаление объектных файлов
	@$(RM) $(USER_PACK)

print_project_summary:
	@echo --
	@echo Бинарный файл создан: "$(COL)$(BIN_PATH)$(NCOL)"
	@echo Время создания: "$(COL)$(get-current-time)$(NCOL)"
	@echo Номер сборки: "$(COL)$(build-num-value)$(NCOL)"

print_library_summary:
	@echo --
	@echo Библиотека создана: "$(COL)$(LIB_PATH)$(NCOL)"
	@echo Время создания: "$(COL)$(get-current-time)$(NCOL)"
	@echo Номер сборки: "$(COL)$(build-num-value)$(NCOL)"


print_mxtext:
	@echo Адрес запуска: "$(COL)$(MX_TEXT)$(NCOL)"

# Подготовка окружения при первом запуске.
# Сбор информации о текущей сборке (создание info.h).
prepare_project:
	@echo Подготовка директорий проекта
	@${MKDIR} ${OUT_PATH}
	@${MKDIR} ${LOG_PATH}
	@${MKDIR} ${SRC_PATH_DEFAULT}
	@echo Обновление файла информации о проекте
	@$(eval build-num-value=$(shell echo $$(($(build-num-value)+1))))
	@printf $(build-num-value) > $(build-num-file)
	@echo "#ifndef _ENVERONMENT_FILE_" 								>  $(info-filename).h
	@echo "#define _ENVERONMENT_FILE_\n"							>> $(info-filename).h
	@echo "#define BUILD_ID         \""$(PROJ_NAME)"\""				>> $(info-filename).h
	@echo "#define BUILD_NUMBER      "$(build-num-value)			>> $(info-filename).h
	@echo "#define BUILD_TIME        \""$(current-time)"\""			>> $(info-filename).h
	@echo ""														>> $(info-filename).h
	@echo "#define GIT_REPO          \""$(git-repo-name)"\""		>> $(info-filename).h
	@echo "#define GIT_BRANCH        \""$(git-branch)"\""			>> $(info-filename).h
	@echo "#define GIT_COMMITTER     \""$(git-committer-name)"\""	>> $(info-filename).h
	@echo "#define GIT_MAIL          \""$(git-committer-mail)"\""	>> $(info-filename).h
	@echo "#define GIT_TIME          \""$(git-date)"\""				>> $(info-filename).h
	@echo "\n#endif"												>> $(info-filename).h
	@echo Создание файла подписи проекта
	@echo "#include \"config.h\"" 				>  $(info-filename).c
	@echo "#include \"info.h\"" 				>> $(info-filename).c
	@echo "#define T_INFO_NAME "$(PROJ_NAME) 	>> $(info-filename).c
	@echo "#include <all/templates.h>" 			>> $(info-filename).c

flash_mount:
	@echo "Подключение диска $(COL)$(MICRO_SD_PATH)$(NCOL)"
	@-sudo ${MKDIR} ${MICRO_SD_PATH}
	$(call check_variable_hint,DEVNAME,Используйте export DEVNAME=sdc для выбора диска C или любой другой диск по вашему выбору.)
	@-sudo mount /dev/$(DEVNAME)1 $(MICRO_SD_PATH)

flash_update:
	@echo "Копирование файла $(COL)$(BIN_PATH)$(NCOL)"
	@sudo $(CP) $(BIN_PATH) $(MICRO_SD_PATH)

flash_install:
	@echo "Копирование файлов проекта ..."
	@sudo $(CPR) $(OUT_PATH)/* $(MICRO_SD_PATH)

flash_installer:
	@sudo ${MKDIR} ${MICRO_SD_PATH}/EMMC_CP
	@echo "Копирование проекта на установочный диск ..."
	@sudo $(CPR) $(OUT_PATH)/* $(MICRO_SD_PATH)/EMMC_CP

flash_umount:
	@sudo sync
	@sudo umount $(MICRO_SD_PATH)
	@echo --
	@echo "Копирование завершено - $(COL)диск можно вынимать$(NCOL)"

do_ffmpeg:
	@echo Подготовка директории для сконвертированного видео
	@${MKDIR} ${OUT_PATH}/avi
	ffmpeg -y -i  "./video/countdwn.avi"  -hide_banner -vf scale=480:272 -c:v libxvid -b:v 8000k -bf 0 -force_key_frames expr:eq\(n,0\) -an "${OUT_PATH}/avi/countdwn.avi"
	@echo --
	@echo "Видео готово"

do_call_scripts:
	@echo "Запуск внешних программ и скриптов..."
	@$(EXTERNAL_CALL_1)
	@$(EXTERNAL_CALL_2)
	@$(EXTERNAL_CALL_3)
	@$(EXTERNAL_CALL_4)
	@$(EXTERNAL_CALL_5)
	@echo "Внешние программы и скрипты выполнены."

# Завершение сборки
finalize:
	@echo --
