/**
 *  @file templates.h
 *  @brief Набор шаблонных функций.
 */

/*
 *  @author Матяш А.В. (a.matiash@set-code.ru)
 *  @version 1.0
 *  @date 15.01.2023
 *  
 *  @copyright Copyright (c) "Сэт Код" 2023
 */

#ifndef _INFOTEMPLATE_H_
#define _INFOTEMPLATE_H_

#include <stdio.h>

/**
 *  @name Макросы общего назначения для построения шаблонов.
 *  @{
 */
#define TEMPCAT(X,Y) X##Y
#define TEMPLATENAME(X,Y) TEMPCAT(X,Y)
/// @}


/**
 *  @name Esc. последовательности при выводе на печать.
 *  @{
 */

// #define COL_ESCAPE          "\033"
#define COL_NONE            "\033[0m"
#define COL_RED             "\033[31m"
#define COL_GREEN           "\033[32m"
#define COL_BLUE            "\033[34m"
#define COL_BRIGHT_GREEN    "\033[92m"
#define COL_BRIGHT_BLUE     "\033[94m"
#define COL_BRIGHT_MAGENTA  "\033[95m"
#define COL_BRIGHT_CYAN     "\033[96m"


/// @}

// Отметки об отладочной сборке
#ifdef DEBUG
#   define COL_NAME COL_BRIGHT_BLUE
#   define BUILD_HINT " DEBUG"
#else
#   define COL_NAME COL_BRIGHT_CYAN
#   define BUILD_HINT ""
#endif

#endif //_INFOTEMPLATE_H_


#ifdef T_INFO_NAME

/**
 *  @name Функции создания описаний для библиотек и проектов
 *  @brief Экземпляры функций создаются для каждого модуля
 *  в системе. Используемые макросы должны быть определены
 *  перед включением экземпляров функций. Обычно это делается
 *  в файлах \b config.h и \b info.h собираемого проекта.
 *  Подключение каждого экземпляра функций описания модуля
 *  осуществляется с помощью следующих директив:
 *  @code
 *  #define T_INFO_NAME projectName // Общая часть имён функций данного модуля.
 *  #include <all/templates.h>      // Создание и подключение экземпляров функций.
 *  @endcode
 * 
 *  Для поддержания актуальности информации при каждой сборке
 *  рекомендуется принудительно пересобирать файл, содержащий
 *  экземпляры функций. Создание такого файла и принудительная
 *  пересборка реализована в \b Makefile, поставляемом вместе
 *  с ядром операционной системы.
 *  @{
 */

/**
 *  @brief Экземпляр функции короткого описания программного модуля.
 *  @details Экземпляр функции короткого описания вида
 *  \b projectNameBrief, уникальное имя \b projectName которого 
 *  задаётся перед подключением с помощью макроса \b T_INFO_NAME.
 */
__attribute__ ((used))
int TEMPLATENAME (T_INFO_NAME,Brief) () {
    printf ("%s%15s%s ver.%s%d.%02d.%04d%s %s%s\n",
            COL_NAME, BUILD_ID, COL_NONE,
            COL_GREEN, PROJECT_VERSION_NUMBER,
            PROJECT_VERSION_SUB_NUMBER,
            BUILD_NUMBER, COL_NONE,
            BUILD_TIME, BUILD_HINT);
    return OK;
}

/**
 *  @brief Экземпляр функции полного описания программного модуля.
 *  @details Экземпляр функции полного описания вида
 *  \b projectNameBrief, уникальное имя \b projectName которого 
 *  задаётся перед подключением с помощью макроса \b T_INFO_NAME.
 */
__attribute__ ((used))
int TEMPLATENAME (T_INFO_NAME,Details) () {
    TEMPLATENAME (T_INFO_NAME,Brief) ();
    printf ("  %s\n", PROJECT_BRIEF);
    printf ("\n");
    printf ("Git info:\n");
    printf ("  repo: %s\n", GIT_REPO);
    printf ("  branch: %s\n", GIT_BRANCH);
    printf ("  committer: %s (%s)\n",
            GIT_COMMITTER,
            GIT_MAIL);
    printf ("  commit time: %s\n", GIT_TIME);
    return OK;
}

/// @}

#undef T_INFO_NAME
#endif  // T_INFO_NAME
