/**
    @addtogroup groupCore
    @{
    @file arch.h
    @brief Описание аппаратной части текущего проекта.
    @details 

    <b>Подключение</b>: @verbatim #include <arch.h> @endverbatim
    
    @see Подробное описание в разделе \ref sectionArch.

    @}
 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 27.03.2023
*/

#ifndef _ARCH_H_
#define _ARCH_H_

#include "mapstr.h"
#include <arch/archdef.h>




/**
 *  @name Добавление параметров
 *  @details Дополнительные параметры конфигурации могут 
 *  быть добавлены в конце списка из программы пользователя.
 *  @{
 */

/**
 *  @brief Добавление строкового параметра.
 *  @details Добавление строки в конец списка параметров.
 *  
 *  @param key Уникальное имя параметра. Рекомендуется брать из 
 *  группы \ref ancArchKeys "макросов", описывающих поля данных.
 *  @param text Добавляемая строка.
 */
void archAddString (const char *key, const char *text);

/**
 *  @brief Добавление целочисленного параметра.
 *  @details Добавление целого числа в конец списка параметров.
 *  
 *  @param key Уникальное имя параметра. Рекомендуется брать из 
 *  группы \ref ancArchKeys "макросов", описывающих поля данных.
 *  @param value Добавляемое значение.
 *  @param description Описание параметра --- актуально для вывода
 *  списка параметров на печать.
 */
void archAddInt (const char *key, int value, const char *description);

/**
 *  @brief Добавление массива целых чисел.
 *  @details Добавление массива целых чисел в конец списка параметров.
 *  
 *  @param key Уникальное имя параметра. Рекомендуется брать из 
 *  группы \ref ancArchKeys "макросов", описывающих поля данных.
 *  @param values Указатель на массив значений.
 *  @param count Количество элементов массива.
 *  @param description Описание массива чисел --- актуально для вывода
 *  списка параметров на печать.
 */
void archAddIntArray (const char *key, int *values, int count, const char *description);
/// @}


/**
 *  @name Чтение параметров из списка
 *  @brief Параметры конфигурации используются при инициализации
 *  библиотек и могут быть использованы в программе пользователя.
 *  @{
 */

/**
 *  @brief Получить строковое значение из списка.
 *  @details Чтение строкового значения из списка по ключу.
 *  Если ключ не найден или найденное значение не является
 *  строкой -- параметр \b ok будет содержать значение \ref false.
 *  
 *  @param key Уникальное имя параметра. Рекомендуется брать из 
 *  группы \ref ancArchKeys "макросов", описывающих поля данных.
 *  @param ok Признак актуальности данных.
 *  @return Указатель на найденную строку если данные найдены
 *  и актуальны, иначе \ref NULL.
 */
const char *archGetString (const char *key, bool *ok);

/**
 *  @brief Получить номер \b GPIO из списка.
 *  @details Чтение номера \b GPIO из списка по ключу.
 *  Значение в списке должно быть строковым 
 *  и содержать имя порта и номер пина.
 *  Стока может быть записана без пробелов, как
 *  в документации на процессор (\b PG1), либо в стиле @osName 
 *  (<b>P_G + 1</b>).
 *  Если ключ не найден или найденное значение не является
 *  строкой -- параметр \b ok будет содержать значение \ref false.
 * 
 *  
 *  @param key Уникальное имя параметра. Рекомендуется брать из 
 *  группы \ref ancArchKeys "макросов", описывающих поля данных.
 *  @param ok Признак актуальности данных.
 *  @return Номер линии \b GPIO.
 */
unsigned int archGetGpio (const char *key, bool *ok);


/**
 *  @brief Получить целое значение из списка.
 *  @details Чтение целого числа из списка по ключу.
 *  Если ключ не найден или найденное значение не является
 *  числом -- параметр \b ok будет содержать значение \ref false.
 *  
 *  @param key Уникальное имя параметра. Рекомендуется брать из 
 *  группы \ref ancArchKeys "макросов", описывающих поля данных.
 *  @param ok Признак актуальности данных.
 *  @return Найденное число если данные найдены, иначе \ref NULL.
 */
int archGetInt (const char *key, bool *ok);

/**
 *  @brief Получить массив целых чисел из списка.
 *  @details Поиск массива целых чисел в списке по ключу.
 *  Если ключ не найден или найденное значение не является
 *  массивом чисел -- параметр \b count будет равен нулю.
 *  
 *  @param[in] key Уникальное имя параметра. Рекомендуется брать из 
 *  группы \ref ancArchKeys "макросов", описывающих поля данных.
 *  @param[out] count Количество элементов массива, либо ноль, 
 *  если данные не найдены.
 *  @return Указатель на найденный массив, либо \ref NULL.
 */
const int *archGetIntArray (const char *key, int *count);

/**
 *  @brief Функция сравнения строк.
 *  @details Функция не работает со списком и написана для удобства
 *  работы со строковыми параметрами. Найденное с помощью archGetString()
 *  значение можно сравнивать с зарезервированными строковыми константами.
 *  
 *  @param value Найденное значение.
 *  @param pattern Шаблон -- строковая константа.
 *  @return Результат сравнения -- \ref true при полном совпадении,
 *  иначе \ref false.
 */
bool archCheckString (const char *value, const char *pattern);
/// @}


/**
 *  @name Элементы драйверов устройств
 *  @brief Функции настройки регистров конфигурации используются при
 *  написании драйверов устройств.
 *  @{
 */


/**
 *  @brief Описание аппаратных линий вывода.
 *  @details Описание аппаратных линий вывода.
 */
typedef struct {
    bool enabled;       ///< @brief Использование линии драйвером. @details Могут использоваться не все пины из доступного набора.
    bool available;     ///< @brief Аппаратное наличие линии. @details Не все наборы содержат полный комплект линий.
    unsigned int pin;   ///< @brief Используемая линия. @details Номер порта + номер бита.
    unsigned int mux;   ///< @brief Значение мультиплексора. @details Значение мультиплексора для подключения к аппаратному модулю.
} tDrvGpio;


/**
 *  @brief Описание группы бит регистра.
 *  @details Описание группы бит регистра, отвечающих 
 *  за настройку работы модуля. В зависимости от процессора 
 *  одни и те же настройки могут быть включены в состав различных регистров.
 */
typedef struct {
    unsigned int addr;  ///< @details Адрес регистра (абсолютный). 0 -- настройка отсутствует.
    unsigned int n;     ///< @details Номер младшего бита в регистре.
    unsigned int mask;  ///< @details Маска (не сдвинутая -- содержится в младших битах).
} tDrvBitGroup;


/**
 *  @brief Описание одного бита регистра.
 *  @details Описание одного бита регистра, 
 *  отвечающего за настройку работы модуля.
 *  В зависимости от процессора одни и те же настройки
 *  могут быть включены в состав различных регистров.
 */
typedef struct {
    unsigned int addr;  ///< @details Адрес регистра (абсолютный). 0 -- настройка отсутствует.
    unsigned int n;     ///< @details Номер бита в регистре.
} tDrvBit;


/**
 *  @brief Задать параметры линии ввода / вывода.
 *  @details Функция используется в инициализации конфигурации драйверов
 *  и заполняет параметры структуры описания
 *  линии порта ввода / вывода. Изначально все линии не активны.
 *  Активировать нужные линии следует при инициализации драйвера.
 *  
 *  @param gpio Заполняемое описание аппаратной части линии.
 *  @param pin Номер порта + номер бита, например, \b P_B+1.
 *  @param mux Значение мультиплексора для подключения к аппаратному модулю.
 */
void drvInitGpio (tDrvGpio *gpio, unsigned int pin, unsigned int mux);

/**
 *  @brief Подключить линию ввода / вывода.
 *  @details Функция используется для подключения линий ввода / вывода
 *  при инициализации драйвера устройства. Для успешного подключения
 *  линия должна быть доступна для инициализируемого устройства и активирована
 *  драйвером. При несоблюдении этих условий линия подключена не будет.
 *  Линии в драйверах подключаются наборами и игнорирование неактивированных линий 
 *  является нормальной практикой.
 *  
 *  @param gpio Описание линии ввода / вывода.
 */
void drvSetGpio (const tDrvGpio *gpio);

/**
 *  @brief Задать описание группы бит.
 *  @details Функция используется в инициализации конфигурации драйверов
 *  и заполняет параметры структуры описания
 *  группы бит регистра.
 *  
 *  @param group Указатель на описание группы бит.
 *  @param addr Адрес регистра -- абсолютное значение.
 *  @param n Номер младшего бита группы в регистре.
 *  @param mask Маска группы бит.
 */
void drvInitBitGroup (tDrvBitGroup *group, unsigned int addr, unsigned int n, unsigned int mask);

/**
 *  @brief Установить группу бит в регистре.
 *  @details Функция записывает значение в группу бит в регистре конфигурации,
 *  не затрагивая остальные биты.
 *  
 *  @param group Указатель на описание изменяемой группы бит.
 *  @param value Записываемое значение.
 */
void drvSetBitGroup (const tDrvBitGroup *group, unsigned int value);

/**
 *  @brief Задать описание бита регистра.
 *  @details Функция используется в инициализации конфигурации драйверов
 *  и заполняет параметры структуры описания
 *  бита регистра.
 *  
 *  @param bit Указатель на описание бита регистра.
 *  @param addr Адрес регистра -- абсолютное значение.
 *  @param n Номер бита в регистре.
 */
void drvInitBit (tDrvBit *bit, unsigned int addr, unsigned int n);

/**
 *  @brief Установить бит в регистре.
 *  @details Функция записывает значение в указанный бит регистра конфигурации,
 *  не затрагивая остальные биты.
 *  
 *  @param bit Указатель на описание изменяемого бита.
 *  @param value Записываемое значение.
 */
void drvSetBit (const tDrvBit *bit, unsigned int value);

/**
 *  @brief Получить значение группы бит в регистре.
 *  @details Функция возвращает значение указанной группы бит регистра конфигурации.
 *  
 *  @param group Указатель на описание запрашиваемой группы бит.
 *  @return Прочитанное значение.
 */
unsigned int drvGetBitGroup (const tDrvBitGroup *group);

/**
 *  @brief Получить значение бита в регистре.
 *  @details Функция возвращает значение указанного бита регистра конфигурации.
 *  
 *  @param bit Указатель на описание запрашиваемого бита.
 *  @return Прочитанное значение.
 */
unsigned int drvGetBit (const tDrvBit *bit);

// Записать бит в регистр и дождаться его автоматического снятия в течение delay милисекунд.

/**
 *  @brief Запись бита и проверка автоматического снятия.
 *  @details Записать бит в регистр конфигурации и дождаться его автоматического снятия
 *  в течение заданного времени. Сразу после записи бита выдерживается короткая пауза
 *  для контроля быстрых процессов. Если бит не снимается за короткое время
 *  функция ожидает снятия в течение \b delay тиков системного таймера.
 *  
 *  @param bit Указатель на описание бита.
 *  @param delay Количество миллисекунд. Допускается установка параметра \ref NO_WAIT. 
 *  @return \ref true если выставленный бит был автоматически снят в течение заданного времени,
 *  иначе -- \ref false.
 */
bool drvResetBit (const tDrvBit *bit, unsigned int delay);


/// @}

/**
 *  @name Дополнительно
 *  @{
 */

/**
 *  @brief Освобождение выделенной памяти.
 *  @details Память может быть освобождена после
 *  инициализации всех библиотек.
 */
void archFree ();

/**
 *  @brief Печать таблицы параметров.
 *  @details Список параметров выводится в консоль
 *  в режиме \b DEBUG после инициализации операционной
 *  системы. Можно вызвать дополнительно в программе пользователя.
 *  
 */
void archPrint ();

/// @}



///@privatesection

/**
 *  @brief Создание списка.
 *  @details Создание списка архитектуры 
 *  выполняется перед запуском ядра ОС в cstart.c.
 *  
 */
void archCreate ();


/**
 *  @brief Заполнение списка параметров.
 *  @details Заполнение списка параметров
 *  из всех известных источников (файлы конфигурации на всех дисках).
 *  
 */
void archMainInit ();


#endif //_ARCH_H_
