/**
    @file blkcache.h
    @brief Кэширование записи / чтения блоков данных.
 */

#ifndef BLKCACHE_H_
#define BLKCACHE_H_

/*
    RTOS MULTEX32
    Author: S.Sokolov
    04.2011
*/

#ifdef __cplusplus
extern "C" {
#endif

/**
 *  Тип указателя на процедуру чтения/записи драйвера.
 */
typedef int (*blk_cache_proc)(void *hDrv, int start, int num, char *buf);

/**
 *  Структура данных организации КЭШ-памяти.
 */
struct blk_cache {
	   int cacheOff;
	   void *hDrv;              ///< @details Указатель на драйвер.
	   blk_cache_proc read;     ///< @details Процедура чтения драйвера.
	   blk_cache_proc write;    ///< @details Процедура записи драйвера.
       int CacheSize;           ///< @details Размер КЭШ-памяти в блоках.
	   int BlkSize;             ///< @details Размер блока памяти.
	   int CacheIdx;            ///< @details Номер первого блока в КЭШ.
	   char *Cache;             ///< @details КЭШ-память.
	   char *Cache_na;          ///< @details Невыровненная на границу страницы (для free).
	   char *BlkStat;           ///< @details Карта состояния КЭШ.
	   };

/**
 *  @brief Создать КЭШ заданного размера.
 *  @details Функция создания КЭШ заданного размера.
 *  
 *  @param CacheSize Заданный размер КЭШ-памяти в блоках.
 *  @param BlkSize Размер блока памяти.
 *  @return Указатель на созданный КЭШ. 
 */
struct blk_cache *blk_cache_create(int CacheSize, int BlkSize);

/**
 *  @brief Задать процедуру загрузки / выгрузки.
 *  @details Функция задаёт процедуры загрузки / выгрузки КЭШ.
 *  
 *  @param sc Указатель на КЭШ.
 *  @param drv Указатель на драйвер.
 *  @param read Указатель на процедуру чтения драйвера.
 *  @param write Указатель на процедуру записи драйвера.
 */
void blk_cache_callback(struct blk_cache *sc, void *drv,
                        blk_cache_proc read, blk_cache_proc write);


/**
 *  @brief Удалить КЭШ.
 *  @details Функция удаления КЭШ.
 *  
 *  @param sc Указатель на КЭШ.
 */
void blk_cache_free(struct blk_cache *sc);
						
/**
 *  @brief Выгрузить КЭШ.
 *  @details Выгрузить КЭШ.
 *  
 *  @param sc Указатель на КЭШ.
 *  @return \ref OK при успехе.
 *  @return \ref ERROR при неудаче.
 */
int blk_cache_flush(struct blk_cache *sc);

/**
 *  @brief Загрузить КЭШ.
 *  @details Функция загрузки КЭШ.
 *  
 *  @param sc Указатель на КЭШ.
 *  @param startBlk Номер начального сектора.
 *  @return \ref OK при успехе.
 *  @return \ref ERROR при неудаче.
 */
int blk_cache_load(struct blk_cache *sc, int startBlk);


/**
 *  @brief Считать сектора через КЭШ.
 *  @details Функция считывает через КЭШ \b nBlock секторов 
 *  начиная с сектора \b startBlk в буфер \b pBuf.
 *  
 *  @param sc Указатель на КЭШ.
 *  @param startBlk Номер начального сектора.
 *  @param nBlock Количество считываемых секторов.
 *  @param pBuf Указатель на массив-приёмник данных.
 *  @return \ref OK при успехе.
 *  @return \ref ERROR при неудаче.
 */
int blk_cache_read(struct blk_cache *sc, int startBlk, int nBlock, char *pBuf);

/**
 *  @brief Записать сектора через КЭШ.
 *  @details Функция записывает через КЭШ \b nBlock секторов
 *  начиная с сектора \b startBlk из буфера \b pBuf.
 *  
 *  @param sc Указатель на КЭШ.
 *  @param startBlk Номер начального сектора.
 *  @param nBlock Количество считываемых секторов.
 *  @param pBuf Указатель на массив-источник данных.
 *  @return \ref OK при успехе.
 *  @return \ref ERROR при неудаче.
 */
int blk_cache_write(struct blk_cache *sc, int startBlk, int nBlock, char *pBuf);

#ifdef __cplusplus
}
#endif
#endif // ifndef BLKCACHE_H_
