/**
 *  @addtogroup groupStdTypes
 *  @{
 *  @file ctype.h
 *  @brief Классификация и преобразование отдельных символов.
 *  @details
 *  @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.4</a>.
 *  @}
 */

/*
 * @date Created on: unknown.
 * @author Sergey Sokolov
 * @author Dubovich Timofey
 */

#ifndef __CTYPE_H
#define __CTYPE_H


#ifdef __cplusplus
extern "C" {
#endif

//-----------------------------------------------

/**
 *  @name Стандартные функции
 *  @{
 */
/**
 * Проверить, является ли символ буквой или цифрой.
 * @param c Символ для проверки.
 * @return 0, если символ не является буквой или цифрой, не-0 иначе.
 */
int isalnum(int c);
/**
 * Проверить, является ли символ буквой.
 * @param c Символ для проверки.
 * @return 0, если символ не является буквой, не-0 иначе.
 */
int isalpha(int c);
/**
 * Проверить, является ли символ пробелом или табуляцией.
 * @param c Символ для проверки.
 * @return 0, если символ не является пробелом или табуляцией, не-0 иначе.
 */
int isblank(int c);
/**
 * Проверить, является ли символ управляющим символом.
 * @param c Символ для проверки.
 * @return 0, если символ не является управляющим символом, не-0 иначе.
 */
int iscntrl(int c);
/**
 * Проверить, является ли символ десятичной цифрой.
 * @param c Символ для проверки.
 * @return 0, если символ не является десятичной цифрой, не-0 иначе.
 */
int isdigit(int c);
/**
 * Проверить, является ли символ печатаемым символом, отличным от пробела.
 * @param c Символ для проверки.
 * @return 0, если символ не является печатаемым символом, отличным от пробела, не-0 иначе.
 */
int isgraph(int c);
/**
 * Проверить, является ли символ символом нижнего регистра.
 * @param c Символ для проверки.
 * @return 0, если символ не является символом нижнего регистра, не-0 иначе.
 */
int islower(int c);
/**
 * Проверить, является ли символ печатаемым символом.
 * @param c Символ для проверки.
 * @return 0, если символ не является печатаемым символом, не-0 иначе.
 */
int isprint(int c);
/**
 * Проверить, является ли символ знаком препинания.
 * @param c Символ для проверки.
 * @return 0, если символ не является знаком препинания, не-0 иначе.
 */
int ispunct(int c);
/**
 * Проверить, является ли символ пробельным символом.
 * @param c Символ для проверки.
 * @return 0, если символ не является пробельным символом, не-0 иначе.
 * @details Пробельными символами являются: ' ', '\\f', '\\n', '\\r', '\\t', '\\v'.
 */
int isspace(int c);
/**
 * Проверить, является ли символ символом верхнего регистра.
 * @param c Символ для проверки.
 * @return 0, если символ не является символом верхнего регистра, не-0 иначе.
 */
int isupper(int c);
/**
 * Проверить, является ли символ шестнадцатеричной цифрой.
 * @param c Символ для проверки.
 * @return 0, если символ не является шестнадцатеричной цифрой, не-0 иначе.
 */
int isxdigit(int c);

/**
 * Преобразовать символ в нижний регистр.
 * @param c Символ для преобразования.
 * @return Преобразованый символ или исходный символ, если преобразование невозможно.
 */
int tolower(int c);
/**
 * Преобразовать символ в верхний регистр.
 * @param c Символ для преобразования.
 * @return Преобразованый символ или исходный символ, если преобразование невозможно.
 */
int toupper(int c);
/// @}

//-----------------------------------------------

/**
 *  @name Расширения BSD и SVID
 *  @{
 */
/**
 * Проверить, является ли символ ascii-символом.
 * @param c Символ для проверки.
 * @return 0, если символ не является ascii-символом, не-0 иначе.
 */
int isascii(int c);

/**
 * Преобразовать символ в в символ ascii, округлив его до 7 младших битов.
 * @param c Символ для преобразования.
 * @return Преобразованый символ.
 */
int toascii(int c);


/**
 * Псевдоним для toupper из SVID.
 * @details Псевдоним для toupper из SVID.
 */
#define _toupper(c) toupper(c)
/**
 * Псевдоним для tolower из SVID.
 * @details Псевдоним для tolower из SVID.
 */
#define _tolower(c) tolower(c)
/// @}

//-----------------------------------------------

/**
 *  @name Макросы для обратной совместимости
 *  @{
 */
#define _IS_SP  1           ///< @details is space
#define _IS_DIG 2           ///< @details is digit indicator
#define _IS_UPP 4           ///< @details is upper case
#define _IS_LOW 8           ///< @details is lower case
#define _IS_HEX 16          ///< @details [0..9] or [A-F] or [a-f]
#define _IS_CTL 32          ///< @details Control
#define _IS_PUN 64          ///< @details punctuation
#define _IS_BLN 128         ///< @details blank
/// @}


#ifdef __cplusplus
}
#endif

#endif /* __CTYPE_H */
