/**
 * @addtogroup groupCore
 * @{
 * @file filesyst.h
 * @brief Дополнительные функции для работы с файловой системой.
 * @see Общее описание системы ввода / вывода см. в главе \ref sectionIoLib.
 * @}
 */


#ifndef FILESYST_H
#define FILESYST_H

#include "multex.h"
#include "stdio.h"



/**
 * Установить устройство в качестве рабочего.
 * @param devName Имя устройства (например, имя тома).
 * @return Установленное рабочее устройство.
 */
const char *setWorkDevice(const char *devName);

/**
 * Распечатать файл в stdout в текстовом режиме.
 * @param filePath Путь к файлу.
 * @return \ref OK, если файл был распечатан, \ref ERROR иначе.
 * @note Бинарные файлы будут распечатаны некорректно.
 */
STATUS type(const char *filePath); // Вывод файла на экран

/**
 * Получить размер файла.
 * @param filePath Путь к файлу.
 * @return Размер файла >=0 или отрицательное значение при ошибке.
 */
int getFileSize(const char *filePath);

/**
 * Загрузить файл в динамическую память.
 * @param filePath Путь к файлу.
 * @return Выделенный блок памяти с содержимым файла или \ref NULL при ошибке.
 * @note По истечении надобности возвращенную память следует освободить при помощи функции \ref free().
 */
void* loadFile(const char *filePath);

/**
 * Загрузить файл в динамическую память и вернуть его размер.
 * @param filePath Путь к файлу.
 * @param pSizeVar Указатель на переменную, в которую будет записан размер загруженного файла.
 * @return Выделенный блок памяти с содержимым файла или \ref NULL при ошибке.
 * @note По истечении надобности возвращенную память следует освободить при помощи функции \ref free().
 */
void* loadFileSz(const char *filePath, int *pSizeVar);

/**
 * Распечатать содержимое каталога в stdout.
 * @param dirPath Путь к каталогу.
 * @param usePageMode Использовать ли страничный режим.
 * @return \ref OK при успехе, \ref ERROR иначе.
 * @details При использовании страничного режима будет печататься по 24 файла, после чего ожидаться символ в stdin, 
 * для продолжения печати. В не-страничном режиме будут печататься всё содержимое сразу.
 */
STATUS dir(const char *dirPath, bool usePageMode);

/**
 * Удалить файл(ы) по маске.
 * @details Удаляемые файлы будут перечислены в stdout.
 * @param fileMask Имя файла или маска файла.
 * @return \ref OK, если удаление прошло успешно или файлы не были обнаружены, \ref ERROR иначе.
 */
int del(const char *fileMask);

/**
 * Скопировать файл(ы) по маске.
 * @param srcMask Маска копируемых файлов.
 * @param dstMask Маска файлов-копий.
 * @param overwriteDst Нужно ли перезаписывать файлы, если они существует.
 * @return \ref OK, если при копировании не произошло ошибок (отсутствие файлов для копирования не считается ошибкой), 
 * \ref ERROR иначе.
 */
STATUS copy(const char *srcMask, const char *dstMask, bool overwriteDst);

/**
 * Скопировать файл.
 * @param src Путь к копируемому файлу.
 * @param dst Путь к копии.
 * @param overwriteDst Нужно ли перезаписывать файл с именем dst, если он существует.
 * @return \ref OK при успехе, \ref ERROR иначе.
 */
STATUS copyFile(const char *src, const char *dst, bool overwriteDst);

/**
 * Перенести файл(ы) по маске.
 * @param srcMask Маска переносимых файлов.
 * @param dstMask Маска перенесенных файлов.
 * @param overwriteDst Нужно ли перезаписывать файлы, если они существует.
 * @return \ref OK, если при перемещении не произошло ошибок (отсутствие файлов для перемещения не считается ошибкой), 
 * \ref ERROR иначе.
 */
STATUS move(const char *srcMask, const char *dstMask, bool overwriteDst);

/**
 * Скопировать содержимое каталога src в каталог dst.
 * @details Функция будет печатать в stdout созданные каталоги и скопированные файлы. При конфликтах файлы будут заменены.
 * @param src Путь к каталогу, из которого будет производится копирование.
 * @param dst Путь к каталогу, в который должно производится копирование. Каталог не должен существовать.
 * @return \ref OK, если при копировании не произошло ошибок (отсутствие файлов для копирования не считается ошибкой), 
 * \ref ERROR иначе.
 */
STATUS dircopy(const char *src, const char *dst);

/**
 * Скопировать содержимое каталога src в каталог dst.
 * @details Функция не будет ничего печатать в stdout. При конфликтах файлы будут заменены.
 * @param src Путь к каталогу, из которого будет производится копирование.
 * @param dst Путь к каталогу, в который должно производится копирование.
 * @return \ref OK, если при копировании не произошло ошибок (отсутствие файлов для копирования не считается ошибкой), 
 * \ref ERROR иначе.
 */
STATUS silentDirCopy (const char *src, const char *dst);

/**
 * Скопировать содержимое каталога src в каталог dst с задержкой в ходе копирования.
 * @details Функция не будет ничего печатать в stdout. При конфликтах файлы будут заменены.
 * @param src Путь к каталогу, из которого будет производится копирование.
 * @param dst Путь к каталогу, в который должно производится копирование.
 * @param delay Задержка в ходе копирования, -1 для отключения задержки, 
 * 0 для принудительного переключения задач в ходе копирования, 
 * положительное число для задержки (чем выше, тем выше задержка итогового копирования).
 * @return \ref OK, если при копировании не произошло ошибок (отсутствие файлов для копирования не считается ошибкой), 
 * \ref ERROR иначе.
 */
STATUS dirCopy_Delay (const char *src, const char *dst, int delay);

/**
 * Удалить содержимое каталога.
 * @details Не удаляет сам каталог.
 * @param dir Путь к каталогу.
 * @return \ref OK при успехе, \ref ERROR при ошибке.
 */
STATUS removeContentFromDir(const char *dir);

/**
 * Проверить, пуст ли каталог (есть ли там файлы и/или каталоги).
 * @param dir Путь к каталогу
 * @return \ref OK, если каталог пуст, \ref ERROR, если не пуст.
 */
STATUS dirIsEmpty(const char *dir);

/**
 * Найти все файлы соответствующие маске в каталоге и его подкаталогах.
 * @details Для директории со следующей структурой:
@verbatim
dir
 |   iniFile1.ini
 |
 +---folder1
 |   +---folder1_1
 |   |       iniFile1_1_1.ini
 |   +---folder1_2
 |           pngFile1_2_1.png
 +---folder2
 |       iniFile2_1.ini
 |       pngFile2_1.png
 +---folder3
 +---folder4
         iniFile4_1.ini
@endverbatim
 *  Вызов функции 
 *  @code findFilesInDirAndSubdirs("*.ini", dirPath, 1, &buf); @endcode
 *  вернёт массив длинной 3 со следующим содержимым:
 *  @verbatim ["folder2/iniFile2_1.ini", "folder4/iniFile4_1.ini", "iniFile1.ini"] @endverbatim
 * 
 * @note Полученный массив необходимо будет освободить: сначала каждый элемент, затем сам массив.
 * 
 * @param fileMask Маска файла.
 * @param searchPath Путь, откуда нужно начинать поиск.
 * @param maxNestingDepth Максимальная глубина вложенности, -1 для поиска без ограничений.
 * @param[out] arrayLength Длинна возвращаемого массива.
 * @return Массив с путями найденных файлов относительно пути для поиска или \ref NULL, 
 * если произошла ошибка или файлы не были обнаружены.
 */
char** findFilesInDirAndSubdirs(const char* fileMask, const char* searchPath, unsigned int maxNestingDepth, int* arrayLength);



#endif
