/**
 * @addtogroup groupCore
 * @{
 * @file fnames.h
 * @brief Функционал для работы с именами файлов.
 * @}
 */
/*
 * @date 2003.
 * @author Sergey V.Sokolov
 * @author Dubovich Timofey
 */
#ifndef FNAMES_H_
#define FNAMES_H_

#include "iolib.h"


#ifdef __cplusplus
extern "C" {
#endif

/**
 * Снабдить имя файла полным путем к нему.
 * @param fileName Имя файла или путь к нему.
 * @param fullPathBuf Буфер под итоговый полный путь.
 * @return \ref OK при успехе, \ref ERROR иначе.
 * @details Если путь к файлу изначально не полный, то он будет дополнен рабочим устройством и/или рабочим каталогом.
 */
STATUS expandFileName(const char *fileName, char *fullPathBuf);

/**
 * Извлечь из пути к файлу имя устройства.
 * @param filePath Путь к файлу.
 * @param driveBuf Буфер под имя устройства.
 * @details Выделяет часть до символа двоеточия из имени файла.
 * Если имя устройства не удалось извлечь, то в буфер будет записано устройство по-умолчанию.
 */
void extractFileDrive(const char *filePath, char *driveBuf);

/**
 * Извлечь из пути к/имени файла полный путь.
 * @param fileName Путь к файлу или имя файла.
 * @param pathBuf Буфер под путь.
 * @details Если путь к файлу изначально не полный, то он будет дополнен рабочим устройством и/или рабочим каталогом.
 */
void extractFilePath(const char *fileName, char *pathBuf);

/**
 * Извлечь из пути к файлу его каталог и присоединить его к корневому каталогу.
 * @param filePath Путь к файлу.
 * @param rootPath Корневой каталог.
 * @param dstBuffer Итоговый буфер.
 * @details Примеры вызова: extractFileDevPath("dir/file.txt", "C:/root/", buf), extractFileDevPath("/home/dir/file.txt", "none", buf).
 * Результат в buf: "C:/root/dir/" и "/home/dir/" соответственно.
 */
void extractFileDevPath(const char *filePath, const char *rootPath, char *dstBuffer);

/**
 * Извлечь имя устройства из пути.
 * @param path Путь.
 * @param deviceBuf Буфер поди имя устройства.
 * @return Смещенный указатель, указывающий на часть буфера path, с которой начинается часть пути без имени устройства.
 */
const char* extractDevice(const char *path, char *deviceBuf);


/**
 * Получить короткое имя файла (имя + расширение) из пути к нему.
 * @param path Путь к файлу.
 * @return Смещенный указатель, указывающий на часть буфера path, с которой начинается короткое имя файла.
 */
const char *getFileName(const char *path);

/**
 * Получить короткое имя файла (имя + расширение) из пути к нему для не-константного указателя.
 * @param path Путь к файлу.
 * @return Смещенный указатель, указывающий на часть буфера path, с которой начинается короткое имя файла.
 * @details Аналогично функции getFileName, но возвращает не-константный указатель.
 */
char *getFileNameNonConst(char *path);

/**
 * Считать имя первого каталога из пути.
 * @param path Путь.
 * @param dirBuf Буфер под каталог.
 * @return Смещенный указатель, указывающий на часть буфера path, с которой начинается следующий каталог,
 * после записанного в буфер.
 * @details Если в пути нет каталогов, то будет возвращено значение переменной path.
 */
const char* extractNextDir(const char *path, char *dirBuf);

/**
 * Предобработка имени файла/устройства.
 * @param path Путь к файлу.
 * @param devBuf Буфер, в который будет записано устройство.
 * @param dh Указатель на указатель на буфер, в который будет записан общий заголовок устройства.
 * @return Смещенный указатель, указывающий на часть буфера path,
 * с которой начинается часть пути без имени устройства или \ref NULL, если устройство не было найдено.
 */
const char *fileNamePreprocess(const char *path, char *devBuf, DEV_HDR **dh);

/**
 * Установить устройство в качестве рабочего.
 * @param path Имя устройства (например, имя тома).
 * @return Установленное рабочее устройство.
 */
const char *setWorkDevice(const char *path);

/**
 * Получить имя текущего рабочего устройства.
 * @return Имя текущего рабочего устройства.
 */
const char *getWorkDevice(void);

/**
 * Сравнить маску поиска и имя файла.
 * @param mask Маска поиска.
 * @param name Имя файла.
 * @return OK, если имя соответствует маске, ERROR иначе.
 */
STATUS compareNames(const char *mask, const char *name);

/**
 * Создать имя файла на основе имени файла и маски пути.
 * @param fileName Имя файла.
 * @param pathMask Маска пути.
 * @param dstNameBuffer Буфер под итоговое полное имя.
 * @details Функция берет маску пути и, с учетом маски, присоединяет к ней имя файла, 
 * создав в итоге полное имя (например, "text.txt" + "./dir/?*.*" = "С:/currentDir/dir/text.txt").
 */
void buildFileName(const char *fileName, const char *pathMask, char *dstNameBuffer);

/**
 * Получить рабочий каталог устройства.
 * @param dev Имя устройства.
 * @return Рабочий каталог устройства или NULL, 
 * при ошибке или если устройство не может иметь рабочего каталога.
 */
const char *getWorkDirectory(const char *dev);

#ifdef __cplusplus
}
#endif
#endif // ifndef FNAMES_H_
