/**
    @addtogroup groupDrivers
    @{
    @file i2c.h
    @brief Интерфейс I2C.
    @details Настройка аппаратного модуля \b I2C.

    <b>Подключение</b>: @verbatim #include <i2c.h> @endverbatim
    
    @see Общее описание работы с интерфейсом \b I2C в разделе \ref sectionI2c.
    @}

    @page pageDrivers

    @section sectionI2c I2C
    @see Описание методов работы с интерфейсом \b I2C в файле i2c.h.

    Интерфейс \b I2C --- это широко распространенный последовательный синхронный
    полудуплексный двухпроводный внутрисхемный интерфейс, используемый для
    управления многими устройствами, например, \b CMOS видеокамерами, или
    часами реального времени \b RTC. 
    
    В составе встроенных аппаратных модулей \b ARM процессоров \em Allwinner
    может содержаться до пяти контроллеров последовательных шин \b I2C. В ОС
    @osName для подключения драйвера одного из аппаратных интерфейсов следует
    вызвать функцию инициализации i2cInit() с указанием тактовой частоты
    сигнала \b CLK. Как правило, устройства на шине \b I2C работают на
    частотах 100 или 400 кГц и в этих случах для выбора частоты шины можно
    воспользоваться значениями из группы \ref ancI2cFreqDefines "макросов".
    На самом деле при инициализации можно использовать и другие частоты
    работы шины. В этом случае частоту следует указать в виде числа в герцах.
    Далее можно читать и писать в подключенные по данному интерфейсу
    устройства по адресу с помощью функций i2cRead() и i2cWrite(). Адрес
    устройства должен быть указан в документации на само устройство. 

    Ниже приведён пример инициализации драйвера интерфейса I2C0 и запись
    массива данных в устройство по адресу 0x10.
    
    @code
    char data[5];
    i2cInit (I2C_0, I2C_GPIO_DEFAULT, I2C_CLK_400_kHz);
    // ...
    i2cWrite (I2C_0, 0x10, data, sizeof (data));
    @endcode  

 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 29.05.2023
*/


#ifndef _I2C_H
#define _I2C_H

#include <multex.h>

/**
 *  @anchor ancI2cDefines
 *  @name Макросы выбора каналов I2C
 *  @{
 */

#define I2C_0   0 ///< @details Индекс для \b I2C0.
#define I2C_1   1 ///< @details Индекс для \b I2C1.
#define I2C_2   2 ///< @details Индекс для \b I2C2.
#define I2C_3   3 ///< @details Индекс для \b I2C3.
#define I2C_4   4 ///< @details Индекс для \b I2C4.

/// @}

/**
 *  @anchor ancI2cSetDefines
 *  @name Макросы выбора набора выходных линий I2C
 *  @{
 */

#define I2C_GPIO_DEFAULT    0 ///< @details Выбор основного набора линий модуля I2C. В большинстве случаев это единственный набор линий.
#define I2C_GPIO_ADDITIONAL 1 ///< @details Выбор дополнительного набора линий модуля I2C.

/// @}


/**
 *  @anchor ancI2cFreqDefines
 *  @name Макросы выбора частоты сигнала CLK модуля I2C
 *  @{
 */

#define I2C_CLK_100_kHz     100000 ///< @details Частота сигнала CLK модуля I2C 100 кГц.
#define I2C_CLK_400_kHz     400000 ///< @details Частота сигнала CLK модуля I2C 400 кГц.

#define I2C_CLK_NORMAL      I2C_CLK_100_kHz ///< @details Нормальная частота сигнала CLK (значение по умолчанию).
#define I2C_CLK_FAST        I2C_CLK_400_kHz ///< @details Высокая частота сигнала CLK.

/// @}


/**
 *  @brief Инициализация канала \b I2C.
 *  @details Настройка и запуск выбранного канала \b I2C.
 *  
 *  @param n Номер канала \b I2C. Для определения номера канала
 *  рекомендуется использовать один из \ref ancI2cDefines "макросов".
 *  @param set Набор линий ввода/вывода, подключаемых к аппаратному модулю
 *  \b I2C. Рекомендуется выбирать один из наборов, описанных в группе
 *  \ref ancI2cSetDefines "макросов". Название выбранного порта и номера
 *  линий, подключенных к модулю можно увидеть в отладочном выводе функции
 *  в консоли. Какие именно выводы используются на плате, следует
 *  выяснить, изучив схему этой платы.
 *  @param clk Частота работы шины \b I2C в герцах. Рекомендуется 
 *  выбирать одну из частот, определённых в группе \ref ancI2cFreqDefines
 *  "макросов", либо задать значение частоты самостоятельно.
 *  Реальная частота работы будет равна либо меньше заданной, если
 *  точное значение частоты выставить не возможно.
 *  @return \ref true если инициализация прошла успешна,
 *  модуль запущен, иначе \ref false.
 */
bool i2cInit (unsigned int n, unsigned int set, unsigned int clk);

/**
 *  @brief Отправка данных по шине \b I2C в режиме \b ведущего устройства.
 *  @details Копирование данных во внутренний буфер драйвера и запуск отправки
 *  буфера в режиме ведущего устройства \b MASTER. Функция не дожидается окончания отправки.
 *  Перед началом отправки нового массива данных функция ожидает окончания предыдущей транзакции.
 *  
 *  @param n Номер канала \b I2C. Для определения номера канала
 *  рекомендуется использовать один из \ref ancI2cDefines "макросов".
 *  @param addr Адрес устройства на шине данных \b I2C.
 *  @param data Передаваемые данные.
 *  @param len Длина передаваемых данных (количество байт).
 *  @return \ref OK Если пакет отправлен без ошибок, иначе \ref ERROR.
 */
STATUS i2cWrite (
    unsigned int n,
    unsigned int addr,
    const void *data,
    unsigned int len);

/**
 *  @brief Запись в регистр адресуемого устройства на шине \b I2C в режиме \b ведущего устройства.
 *  @details Копирование данных во внутренний буфер драйвера и запуск отправки
 *  буфера в режиме ведущего устройства \b MASTER. 
 *  Функция является надстройкой над i2cWrite(). Номер регистра передаётся как первый байт(ы)
 *  данных.
 *  
 *  @param n Номер канала \b I2C. Для определения номера канала
 *  рекомендуется использовать один из \ref ancI2cDefines "макросов".
 *  @param devAddr Адрес устройства на шине данных \b I2C.
 *  @param regAddr Номер регистра, в который записываются данные.
 *  @param regLen Длина адреса регистра.
 *  @param data Передаваемые данные.
 *  @param dataLen Длина передаваемых данных (количество байт) без учёта адреса регистра.
 *  @return \ref OK Если пакет отправлен без ошибок, иначе \ref ERROR.
 */
STATUS i2cRegisterWrite (
    unsigned int n, 
    unsigned int devAddr, 
    unsigned int regAddr, 
    unsigned int regLen, 
    const void *data,
    unsigned int dataLen);

/**
 *  @brief Чтение данных по шине \b I2C в режиме \b ведущего устройства.
 *  @details Чтение данных в буфер \b data в режиме ведущего устройства \b MASTER.
 *  Функция дожидается окончания приёма данных.
 *  Перед началом чтения нового массива данных функция ожидает окончания предыдущей транзакции.
 *  
 *  @param n Номер канала \b I2C. Для определения номера канала
 *  рекомендуется использовать один из \ref ancI2cDefines "макросов".
 *  @param addr Адрес устройства на шине данных \b I2C.
 *  @param data Получаемые данные.
 *  @param len Длина получаемых данных (количество байт).
 *  @return STATUS 
 */
STATUS i2cRead (
    unsigned int n,
    unsigned int addr,
    void *data,
    unsigned int len);


/**
 *  @brief Чтение регистра адресуемого устройства на шине \b I2C в режиме \b ведущего устройства.
 *  @details Копирование адреса во внутренний буфер драйвера и запуск обмена
 *  с ведомым в режиме ведущего устройства \b MASTER. 
 *  Функция является надстройкой над i2cWrite() с изменением направления передачи данных после отправки
 *  адреса регистра ведомому устройству. Номер регистра передаётся как первый байт(ы) данных.
 *  
 *  @param n Номер канала \b I2C. Для определения номера канала
 *  рекомендуется использовать один из \ref ancI2cDefines "макросов".
 *  @param devAddr Адрес устройства на шине данных \b I2C.
 *  @param regAddr Номер регистра, в который записываются данные.
 *  @param regLen Длина адреса регистра.
 *  @param data Получаемые данные.
 *  @param dataLen Длина принимаемых данных (количество байт) без учёта адреса регистра.
 *  @return \ref OK Если транзакция прошла без ошибок, иначе \ref ERROR.
 *  @return STATUS 
 */
STATUS i2cRegisterRead (
    unsigned int n,
    unsigned int devAddr,
    unsigned int regAddr,
    unsigned int regLen,
    void *data,
    unsigned int dataLen);



/// @privatesection

/**
 *  @brief Инициализация модуля.
 *  @details Считывание информации об используемом
 *  процессоре. Функция используется системой --- пользователю
 *  вызывать не нужно.
 *  
 *  @return STATUS 
 */
STATUS i2cArchInit ();



// ==================================================================
// DEPRECATED SECTION


struct i2c_client {
    int bus_nr;
    int addr;
};

static inline struct i2c_client *i2c_to_client (int bus, int addr) {
    static struct i2c_client cl;
    cl.bus_nr = bus;
    cl.addr = addr;
    return &cl;
}


/**
 *  @brief Инициализация подсистемы.
 *  @details Инициализация подсистемы \b I2C.
 *  @deprecated Используйте i2cInit();
 */
int __attribute__((deprecated)) i2c_init ();

/**
 *  @brief Установка частоты шины.
 *  @details Установка частоты шины.
 *  @deprecated Используйте i2cInit();
 */
int __attribute__((deprecated)) i2c_freq (int bus, int frq);

// Проверка наличия устройства
/**
 *  @brief Проверка наличия устройства.
 *  @details Проверка наличия устройства.
 *  @deprecated Устаревший драйвер будет удалён.
 */
int __attribute__((deprecated)) i2c_probe (int bus, u8 addr);

// Чтение байта из регистра, адресуемого одним байтом reg
/**
 *  @brief Чтение байта из регистра, адресуемого одним байтом reg.
 *  @details Чтение байта из регистра, адресуемого одним байтом reg.
 *  @deprecated Устаревший драйвер будет удалён.
 */
int __attribute__((deprecated)) i2c_reg_read (int bus, int addr, int reg);

// Запись байта val в регистр, адресуемый одним байтом reg
/**
 *  @brief Запись байта val в регистр, адресуемый одним байтом reg.
 *  @details Запись байта val в регистр, адресуемый одним байтом reg.
 *  @deprecated Используйте i2cRegisterWrite();
 */
int __attribute__((deprecated)) i2c_reg_write (int bus, int addr, int reg, int val);

// Обмен сообщением с клиентом
/**
 *  @brief Запись в режиме мастер.
 *  @details Запись в режиме мастер.
 *  @deprecated Используйте i2cWrite();
 */
int __attribute__((deprecated)) i2c_master_send (struct i2c_client *cl, void *buffer, int length);

/**
 *  @brief Чтение в режиме мастер.
 *  @details Чтение в режиме мастер.
 *  @deprecated Устаревший драйвер будет удалён.
 */
int __attribute__((deprecated)) i2c_master_recv (struct i2c_client *cl, void *buffer, int length);

/**
 *  Обмен сообщением с клиентом по его внутреннему адресу переменной длины.
 *  Шина и адрес клиента на шине - в структуре cl.
 *  @param addr Адрес памяти клиента, с которой производится обмен.
 *  @param alen Размер в байтах адреса памяти клиента.
 *  @param buffer Указатель на данные.
 *  @param length Количество передаваемых/получаемых по адресу байт.
 *  @deprecated Используйте i2cRegisterWrite();
 */
int __attribute__((deprecated)) i2c_write (struct i2c_client *cl, uint addr, int alen, void *buffer, int length);

/**
 *  @brief Чтение в режиме мастер.
 *  @details Чтение в режиме мастер.
 *  @deprecated Устаревший драйвер будет удалён.
 */
int __attribute__((deprecated)) i2c_read (struct i2c_client *cl, uint addr, int alen, void *buffer, int length);

#endif
