/**
    @addtogroup groupCore 
    @{
    @file intlib.h
    @brief Методы управления прерываниями.
    @details Файл содержит объявления методов управления прерываниями.
    @see Общее описание работы с прерываниями в разделе \ref sectionIntLib.
    @}
 */

#ifndef INTLIB_H_
#define INTLIB_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <multex.h>

/**
 *  @anchor ancIrqGroupPriority
 *  @name Макросы распределения приоритетов прерываний
 *  @brief Подробнее о приоритетах прерываний см. \ref subsectionIntTaskPriority.
 *  @{
 */

#define INTERRUPT_GROUP_TIME_CRITICAL   0   ///< @details Группа прерываний, для которых критично время выполнения. 
                                            ///< @note Не рекомендуется помещать в эту группу более, чем одно прерывание в проекте.
#define INTERRUPT_GROUP_SYSTEM          1   ///< @details Группа системных прерываний. Обычно в этой группе одно прерывание --- системный таймер.
#define INTERRUPT_GROUP_MAJOR           2   ///< @details Основная группа прерываний. Исходно все прерывания находятся в этой группе.
#define INTERRUPT_GROUP_MINOR           3   ///< @details Группа для прерываний с большим временем выполнения обработчика.

#define INTERRUPT_PRIORITY_HIGHEST      0   ///< @details Наивысший приоритет прерываний.
#define INTERRUPT_PRIORITY_BASE         3   ///< @brief Базовый приоритет прерываний. 
                                            ///  @details Для лучшей читаемости кода рекомендуется задавать приоритет, 
                                            ///  как смещение относительно базового. Например, INTERRUPT_PRIORITY_BASE + 1.
#define INTERRUPT_PRIORITY_LOWEST       7   ///< @details Самый низкий приоритет прерываний.
/// @}


/**
 *  @brief Процедурный тип обработчиков прерываний.
 *  @details Процедурный тип обработчиков прерываний.
 */
typedef  int (*usr_int_proc)(int);

/**
 *  @brief Инициализация прерываний.
 *  @details Процедура инициализации таблицы дескрипторов прерываний.
 */
int initIntLib(void);

/**
 *  @brief Подключение обработчика прерывания с указанием приоритета.
 *  @details Функция подключает процедуру пользователя
 *  в качестве обработчика аппаратного прерывания.
 *  Подключаемая процедура должна иметь формат:
 *  @code
 *  int routine(int parameter);
 *  @endcode
 * 
 *  @see Подробнее о приоритетах прерываний в разделе \ref subsectionIntTaskPriority. 
 *  
 *  @param vector Номер аппаратного прерывания,
 *  к которому требуется подключить обработчик.
 *  @param group Группа, к которой относится прерывание. Рекомендуется выбирать значение
 *  из макросов соответствующей \ref ancIrqGroupPriority "группы".
 *  @param priority Приоритет прерывания внутри группы.  Рекомендуется выбирать значение
 *  из макросов соответствующей \ref ancIrqGroupPriority "группы".
 *  @param routine Процедура, которую требуется подключить
 *  в качестве обработчика.
 *  @param parameter Параметр, который будет передан обработчику.
 *  @return \ref OK при успешном выполнении.
 *  @return \ref ERROR при неверном значении вектора прерывания.
 */
int interruptConnect (
    int vector, 
    int group, int priority, 
    usr_int_proc routine, int parameter);

/**
 *  @brief Подключение обработчика прерывания (устаревший вариант).
 *  @deprecated Функция устарела, поскольку не поддерживает
 *  приоритеты прерываний. Подключаемый прерывания будут
 *  иметь значения приоритета по умолчанию. В новых проектах
 *  следует использовать функцию interruptConnect().
 */
int intConnect(int vector, usr_int_proc routine, int parameter) __attribute__ ((deprecated("This function is deprecated and is using default interrupt priorities, use interruptConnect instead")));

/**
 *  @brief Просмотр подключенных прерываний.
 *  @details Вывод в консоль таблицы подключенных прерываний.
 */
void irq ();

/// @privatesection
int intUnconnect (
    int vector,
    usr_int_proc routine);


// int setIntPriority(int vector, int priority);


/**
 *  @deprecated Не поддерживает приоритеты прерываний.
 */
int __attribute__((deprecated)) enable_irq(int vector);

int run (intptr_t addr);
int disable_irq(int vector);
// unsigned int program_status();

/**
 *  @brief Запрет прерываний.
 *  @details Запрет прерываний при исполнении последующего кода.
 *  
 *  @param sender Имя функции --- инициатора входа в секцию кода с запретом прерываний.
 *  @return \ref true Прерывания были разрешены, при выходе их следует снова разрешить.
 *  @return \ref false Прерывания уже были запрещены, при выходе ничего делать не нужно.
 */
bool disable_interrupts (const char *sender);

/**
 *  @brief Проверка запрета прерываний.
 *  @details Проверка может использоваться для последующего открытия
 *  прерываний с помощью enable_interrupts();
 *  
 *  @return \ref true Прерывания разрешены.
 *  @return \ref false Прерывания запрещены.
 */
bool check_interrupts ();

/**
 *  @brief Разрешение прерываний.
 *  @details Разрешение прерываний --- следует выполнять, если
 *  на момент вызова disable_interrupts() прерывания были разрешены.
 *  
 *  @param sender Имя функции --- инициатора входа в секцию кода с запретом прерываний.
 */
void enable_interrupts (const char *sender);


/**
 *  @brief Вход в секцию кода с запретом прерываний.
 *  @details Запрет прерываний при исполнении последующего кода.
 *  Функция вызывает disable_interrupts().
 *  
 *  @param sender Имя функции --- инициатора входа в секцию кода с запретом прерываний.
 *  @return \ref true Прерывания были разрешены, при выходе их следует снова разрешить.
 *  @return \ref false Прерывания уже были запрещены, при выходе ничего делать не нужно.
 */
bool critical_enter (const char *sender);

/**
 *  @brief Выход из секции кода, где были запрещены прерывания.
 *  @details Разрешение прерываний --- следует выполнять, если
 *  на момент вызова critical_enter() прерывания были разрешены.
 *  Функция вызывает enable_interrupts().
 *  
 *  @param sender Имя функции --- инициатора входа в секцию кода с запретом прерываний.
 */
void critical_exit(const char *sender);


#define IRQ_NONE    (0)
#define IRQ_HANDLED (1)
typedef int interrupt_t;
	
#ifdef __cplusplus
}
#endif
#endif // ifndef INTLIB_H_
