/**
    @addtogroup groupCore 
    @{
    @file iolib.h
    @brief Работа с базовой системой ввода / вывода.
    @details Файл содержит описания методов базовой системы ввода / вывода.
    @see Общее описание системы ввода / вывода см. в главе \ref sectionIoLib.
    @}

 */

#ifndef _IOLIB_H
#define _IOLIB_H

/*
   Author: Sergey V. Sokolov
*/

#include "multex.h"
#include "tasklib.h"
#include "datetime.h"
#include "stdio.h"

typedef void DCB; ///< Блок управления устройства (специфичен для у-ва).

/**
 *  @brief Общий заголовок устройства.
 *  @details Структура общего заголовка устройства.
 *  
 */
typedef struct device_header
        {
         char *devName;     ///< @details Символьное имя устройства.
         int drvNumber;     ///< @details Номер драйвера в таблице.
         DCB *devDCB;       ///< @details Данные, зависящие от у-ва.
         struct device_header *next; ///< @details Следующее у-во в цепочке.
         char *devType;     ///< @details Строка, определяющая тип устройства. 
                            /// Служит для поиска однотипных устройств. 
                            /// Состав строки для разных типов устройств может отличаться.
                            /// Если тип не используется -- указатель равен \ref NULL.
        } DEV_HDR;

/**
 *  @brief Список устройств.
 *  @details Список устройств.
 *  
 */
extern DEV_HDR *sysDeviceList;


/**
 *  @brief Блок управления файла.
 *  @details Структура блока управления файла.
 */
typedef struct file_fcb
        {
         char *fileName;
         unsigned int fileSize;
         unsigned int position;
         int eof;      ///< @details Признак конца файла.
         char *blkBuf; ///< @details Для блочных устройств - буфер блока.
         int startBlk; ///< @details Стартовый блок файла.
         int catIndex; ///< @details Порядковый номер файла в каталоге.
         int blkNum;   ///< @details Номер блока, содержащегося в буфере.
         int flushed;  ///< @details Блок сохранен на диске.
         DEV_HDR *devHdr;   ///< @details Указатель на заголовок устройства.
         void *paramBlk; ///< @details Указатель на блок параметров для файла.
         int mode;
        }FCB;
		
/**
 *  @brief Указатель на блок управления файла.
 */
typedef FCB *P_FCB;


/**
 *  @brief Блок информации для поиска файлов в каталоге.
 *  @details Структура данных блока поиска файлов в каталоге.
 *  
 */
typedef struct ffblk
        {
         int ff_idx;                  ///< Индекс файла в каталоге.
         char ff_dname[13];           ///< Имя устройства.
         int ff_attrib;               ///< Атрибуты файла.
         int ff_fsize;                ///< Размер файла.
		 char ff_name[13];            ///< Имя файла .
		 char ff_mask[13];            ///< Маска имени файла.
		 DT_COMPACT ff_date_time;     ///< Время создания файла .
		 char *ff_path;               ///< Полный путь в каталог.
		 void *ff_directory;          ///< Указатель на каталог.
		 char ff_lname[128];          ///< Длинное имя файла.
        }FFBLK;
        


/**
 *  @brief Блок информации для функции \b seek.
 *  @details Структура данных блока информации функции \b seek.
 */
typedef struct seekblk
        {
         int seektype;
         int shift;
        }SEEKBLK;          

        

/**
 *  @name Функциональные типы базовых функций
 *  @brief Функциональные типы для 7 базовых функций ввода / вывода.
 *  @{
 */
typedef FCB *(*iosDevCreateProc)(DEV_HDR *iosDev, const char *filename, int mode);
typedef STATUS (*iosDevRemoveProc)(DEV_HDR *iosDev, const char *filename);
typedef FCB *(*iosDevOpenProc)(DEV_HDR *iosDev, const char *filename, int mode);
typedef STATUS (*iosDevCloseProc)(FCB *fp);
typedef int (*iosDevRdProc)(FCB *fp, char *buffer, int nBytes);
typedef int (*iosDevWrProc)(FCB *fp, const char *buffer, int nBytes);
typedef int (*iosDevIoctlProc)(FCB *fp, int requestCode, int arg);
/// @}


        
/**
 *  @name Прототипы внутренних функций IOS
 *  @{
 */

/**
 *  @brief Регистрация драйвера в системе.
 *  @details Функция регистрирует в системе драйвер устройства под заданным именем.
 *  
 *  @param iosDev Указатель на блок управления устройством.
 *  @param name Имя, под которым устройство будет доступно
 *  @param iosDrvNum Номер, под которым зарегистрировано устройство (результат возврата \ref iosDrvInstall).
 *  @return \ref OK при успешном завершении.
 *  @return \ref ERROR при неудаче.
 */
STATUS iosDevAdd (DCB *iosDev, const char *name, int iosDrvNum);

/**
 *  @brief Регистрация в системе драйвера типизированного устройства.
 *  @details Функция регистрирует в системе драйвер устройства 
 *  под заданным именем с указанием типа устройства.
 *  
 *  @param iosDev Указатель на блок управления устройством.
 *  @param name Имя, под которым устройство будет доступно
 *  @param iosDrvNum Номер, под которым зарегистрировано устройство (результат возврата \ref iosDrvInstall).
 *  @param devType Строка типа используется для поиска однотипных устройств. 
 *  Состав строки для разных типов устройств может отличаться.
 *  Для устройств без указания типа можно использовать упрощённую версию функции iosDevAdd().
 *  @return \ref OK при успешном завершении.
 *  @return \ref ERROR при неудаче.
 */
STATUS iosDevTypedAdd (DCB *iosDev, const char *name, int iosDrvNum, const char *devType);

/**
 *  @brief Удаление устройства по блоку управления.
 *  @details Функция удаляет из системы заданное устройство по его блоку управления.
 *  
 *  @param iosDev Блок управления удаляемым устройством.
 *  @return \ref OK при успешном завершении.
 *  @return \ref ERROR при неудаче.
 */
STATUS iosDevRemove (DCB *iosDev);

/**
 *  @brief Удаление устройства по имени.
 *  @details Функция удаляет из системы заданное устройство по его имени.
 *  
 *  @param devName Символьное имя устройства.
 *  @return \ref OK при успешном завершении.
 *  @return \ref ERROR при неудаче.
 */
STATUS removeDevice (const char *devName);

/**
 *  @brief Инсталляция драйвера в системе.
 *  @details Функция инсталлирует драйвер устройства, или файловую систему.
 *  Драйвер должен иметь семь функции:
 *  - Функцию создания файла на устройстве \ref FCB *createProc(\ref DEV_HDR *iosDev, char *name, int mode).
 *  - Функцию удаления файла на устройстве \ref STATUS removeProc(\ref DEV_HDR *iosDev, char *name).
 *  - Функцию открытия файла \ref FCB *openProc(\ref DEV_HDR *iosDev, char *name, int mode).
 *  - Функцию закрытия файла \ref STATUS closeProc(\ref FCB *fp).
 *  - Функцию чтения данных int readProc(\ref FCB *fp, char *buf, int count).
 *  - Функцию записи данных int writeProc(\ref FCB *fp, char *buf, int count).
 *  - Функцию обработки специальных запросов int ioctlProc(\ref FCB *fp, int requestCode, int arg).
 * 
 *  Не все функции обязательны, так, например, для устройства типа консоли имя файла указывается пустым,
 *  а функции создания и удаления вовсе отсутствуют.
 *  Если вместо указателя на функцию указан \b NULL, это означает, что функция не поддерживается устройством.
 *  
 *  @param createProc Функция вида \ref FCB *createProc(\ref DEV_HDR *iosDev, char *name, int mode).
 *  @param removeProc Функция вида \ref STATUS removeProc(\ref DEV_HDR *iosDev, char *name).
 *  @param openProc Функция вида \ref FCB *openProc(\ref DEV_HDR *iosDev, char *name, int mode).
 *  @param closeProc Функция вида \ref STATUS closeProc(\ref FCB *fp).
 *  @param readProc Функция вида int readProc(\ref FCB *fp, char *buf, int count).
 *  @param writeProc Функция вида int writeProc(\ref FCB *fp, char *buf, int count).
 *  @param ioctlProc Функция вида int ioctlProc(\ref FCB *fp, int requestCode, int arg).
 *  @return Целое число — номер, под которым зарегистрирован в системе драйвер этого устройства.
 */
int iosDrvInstall (iosDevCreateProc createProc, iosDevRemoveProc removeProc, iosDevOpenProc openProc, iosDevCloseProc closeProc, iosDevRdProc readProc, iosDevWrProc writeProc, iosDevIoctlProc ioctlProc);

/// @}



/**
 *  @name БЛОЧНЫЕ УСТРОЙСТВА
 *  @brief Устройства прямого доступа.
 *  @{
 */
typedef void DEVICE;        


/**
 *  Стандартный маркер структуры \ref BLK_DEV.
 */
#define BD_STD_SIGNATURE 0x4535FAEB

/*
 *  @brief Структура указателей на функции блочного устройства.
 */
typedef struct blk_dev
        {
         UINT   bd_signature;
         STATUS (*bd_blkRd)
                (struct blk_dev *pDev, int startBlk, int numBlks, char *pBuf);
         STATUS (*bd_blkWrt)
                (struct blk_dev *pDev, int startBlk, int numBlks, char *pBuf);
         STATUS (*bd_ioctlProc)
                (struct blk_dev *pDev, int funcCode, int arg);
         STATUS (*bd_resetProc)
                (struct blk_dev *pDev);
         STATUS (*bd_statusProc)
                (struct blk_dev *pDev);
                int bd_volume;   // 0 - A , 0x80 - C.
                int bd_startBlk; // Начальный блок устройства.
                int bd_blkTotal; // Число блоков на устройстве.
                int bd_bytesPerBlk; // Размер блока в байтах.
                void *bd_rootCat;   // Буфер корневого каталога.
                BOOL bd_catSaved;   // Каталог сохранен на устройстве.
         DEVICE *volConfig; // Указатель на параметры устройства (определяется
                            // физикой устройства).
         void   *fsData;    // Указатель на данные файловой системы.
        } BLK_DEV;            
 

/**
 *  @brief Монтирование раздела (тома) с указанием типа и номера устройства.
 *  @details Функция привязки
 *  \ref subsectionIoLibBlockDev "блочного устройства" к файловой системе
 *  с указанием типа устройства, номера физического диска и номера
 *  логического раздела (тома).
 *  
 *  @param fsDrvNum Дескриптор файловой системы (например, \em DosDriver).
 *  @param devName Имя устройства, например \b С: или \b D:
 *  (для файловой системы \b MSDOS всегда состоит из одной буквы и стоящего за ней двоеточия).
 *  Для получения следующего свободного имени устройства следует использовать iosDriveLabelGet().
 *  @param devType Строка типа устройства. Тип следует выбирать из зарезервированных значений:
 *  - \b mmc --- диск MMC, eMMC, NAND, uSD;
 *  - \b sata --- диск SATA;
 *  - \b usb --- USB flash диск.
 *  @param devNumber Номер физического устройства на плате/процессора (см. схему платы).
 *  Например, номер \b MMC диска на процессорном модуле \b SE-SOM -- \b 0. Дополнительная
 *  карта памяти на демонстрационной плате \b SE-DB-254 имеет физический номер \b 2.
 *  @param volume Если на устройстве имеются разделы (как на жестком диске),
 *  то нужно задать значение \b 0х80. В противном случае (как на дискете)
 *  требуется задать значение \b 0.
 *  @param partition Номер раздела (\b 0).
 *  @param driver Указатель на структуру — драйвер блочного устройства.
 *  @return \ref OK при удачном монтировании, иначе \ref ERROR.
 */
STATUS mountTypedVolume (int fsDrvNum, const char *devName, const char *devType, int devNumber, int volume, int partition, BLK_DEV *driver);


/**
 *  @brief Монтирование тома.
 *  @details Функция привязки
 *  \ref subsectionIoLibBlockDev "блочного устройства" к файловой системе.
 *  
 *  @deprecated Функция устарела и не позволяет использовать механизм поиска дисков по номеру физического устройства. 
 *  Для подключения дисков следует использовать mountTypedVolume().
 *  @param fsDrvNum Дескриптор файловой системы (например, \em DosDriver).
 *  @param devName Имя устройства, например \b С: или \b D:
 *  (для файловой системы \b MSDOS всегда состоит из одной буквы и стоящего за ней двоеточия).
 *  Для получения следующего свободного имени устройства следует использовать iosDriveLabelGet().
 *  @param volume Если на устройстве имеются разделы (как на жестком диске),
 *  то нужно задать значение \b 0х80. В противном случае (как на дискете)
 *  требуется задать значение \b 0.
 *  @param partition Номер раздела (\b 0).
 *  @param driver Указатель на структуру — драйвер блочного устройства.
 *  @return \ref OK.
 *  @return \ref ERROR при неудаче монтирования.
 */
STATUS mountVolume(int fsDrvNum, const char *devName, int volume, int partition, BLK_DEV *driver) __attribute__ ((deprecated("Функция устарела, для подключения дисков используйте mountTypedVolume().")));

/**
 *  @brief Получить свободное имя устройства.
 *  @details Функция возвращает свободное имя диска 
 *  для подключения нового устройства с помощью mountVolume().
 *  
 *  @return Указатель на строку, содержащую имя вида <b>C:</b>.
 */
const char *iosDriveLabelGet ();

/**
 *  @brief Поиск имени раздела (тома) в списке подключенных устройств.
 *  @details Функция позволяет найти имя уже подключенного 
 *  к файловой системе \ref subsectionIoLibBlockDev "блочного устройства"
 *  по типу устройства, номеру физического диска и номеру раздела.
 *  
 *  @param devType @param devType Строка типа устройства. Зарезервированные строки 
 *  для обозначения различных устройств перечислены в описании функции mountTypedVolume().
 *  @param devNumber Номер физического устройства на плате/процессора (см. схему платы).
 *  @param partition Номер раздела (\b 0).
 *  @return Указатель на имя раздела диска (например \b С: или \b D: ) 
 *  если указанный раздел найден, иначе \ref NULL.
 */
const char *iosTypedVolumeLabelFind (const char *devType, int devNumber, int partition);

/// @}


// Функциональные типы для блочных устройств
/// @private
typedef STATUS (*bd_blkRdProc)(BLK_DEV *pDev, int startBlk, int numBlks, char *pBuf);
/// @private
typedef STATUS (*bd_blkWrtProc)(BLK_DEV *pDev, int startBlk, int numBlks, char *pBuf);
/// @private
typedef STATUS (*bd_ioctlProc)(BLK_DEV *pDev, int funcCode, int arg);
/// @private
typedef STATUS (*bd_resetProc)(BLK_DEV *pDev);
/// @private
typedef STATUS (*bd_statusProc)(BLK_DEV *pDev);




/**
 *  @name БАЗОВЫЙ ВВОД/ВЫВОД
 *  @brief Флаги базовой системы ввода / вывода.
 *  @anchor ancIoLibFlags 
 *  @{
 */


#define O_BINARY  8         //! @details Работа с файлом должна вестись в бинарном режиме.
#define O_RDONLY  1         //! @details Файл только для чтения.
#define O_WRONLY  2         //! @details Файл только для записи.
#define O_RDWR    0         //! @details Файл для чтения/записи.
#define O_CAT     4         //! @details Флаг является каталогом.
#define O_FIXED   0x10      //! @details Фиксированный размер файла.
#define O_CREAT   0x200     //! @details Файл следует создать при его отсутствии.
#define O_TRUNC   0x400     //! @details Если файл существует, то его следует урезать до нуля.

// Аргументы IOCTL
#define FIODISKFORMAT 100
#define FIORESETDRV   104
#define FIOGETLNAME   105

#define FIOUNMOUNT    107

#define FIOFILESIZE   27
#define FIOEOF        28
#define FIOSEEK       29
#define FIOFINDFIRST  30
#define FIOFINDNEXT   31
#define FIOTELL       32
#define FIOFLUSH      33 
#define FIORENAME     34
#define FIOFREESIZE   35
#define FIOCHKDSK     36
#define FIOCD         37
#define FIOWRKDIR     38
#define FIOUPD        39 
#define FIOGETLABEL   40
#define FIOSETDT      41
#define FIOGETDT      42
#define FIOMKD        43
#define FIORMD        44


// Таймауты
#define FIOSETTO     200
#define FIOGETTO     201


// Атрибуты файлов (для findFirst и findNext)
#ifndef FA_RO
    #define FA_RO     0x01  //! @details Файл/каталог только для чтения.
    #define FA_HIDE   0x02  //! @details Скрытый файл/каталог.
    #define FA_SYSTEM 0x04  //! @details Системный файл/каталог.
    #define FA_LABEL  0x08  //! @details Том.
    #define FA_CAT    0x10  //! @details Каталог.
    #define FA_ARCH   0x20  //! @details Архив.
#endif
/// @}



/**
 *  @name Процедуры работы с устройствами / файлами
 *  @{
 */

/**
 *  @brief Создать файл по имени.
 *  @details Функция создает файл по его имени.
 *  
 *  @param path Имя создаваемого файла.
 *  @param flags Флаги для создания, такие как \ref O_RDONLY, \ref O_WRONLY, \ref O_RDWR, \ref O_CREAT.
 *  @see Полный перечень флагов см. в разделе \ref ancIoLibFlags "Флаги базовой системы ввода / вывода".
 *  @return Дескриптор создаваемого файла, небольшое целое число. При неудаче вернет отрицательное значение.
 */
int creat(const char *path, int flags);

/**
 *  @brief Создать пустой файл.
 *  @details Функция создаёт дескриптор для пустого файла на устройстве.
 *  
 *  @param devn Идентификатор устройства.
 *  @param flags Флаги для создания, такие как \ref O_RDONLY, \ref O_WRONLY, \ref O_RDWR, \ref O_CREAT.
 *  @see Полный перечень флагов см. в разделе \ref ancIoLibFlags "Флаги базовой системы ввода / вывода".
 *  @return Дескриптор создаваемого файла, небольшое целое число. При неудаче вернет отрицательное значение.
 */
int creat_empty(const char *devn, int flags);

/**
 *  @brief Открыть файл или устройство.
 *  @details Функция открывает файл или устройство по его имени.
 *  
 *  @param path Имя открываемого файла или устройства
 *  @param flags Флаги для открытия, такие как \ref O_RDONLY, \ref O_WRONLY, \ref O_RDWR, \ref O_CREAT.
 *  @see Полный перечень флагов см. в разделе \ref ancIoLibFlags "Флаги базовой системы ввода / вывода".
 *  @return Дескриптор открытого файла, небольшое целое число. При неудаче вернет отрицательное значение.
 */
int open(const char *path, int flags);

/**
 *  @brief Закрыть файл или устройство.
 *  @details Функция закрывает ранее открытый файл или устройство по дескриптору.
 *  
 *  @param fd Дескриптор открытого файла.
 */
int close(int fd);

/**
 *  @brief Удалить файл по имени.
 *  @details Функция удаляет файл по его имени.
 *  
 *  @param path Имя удаляемого файла.
 *  @return \ref OK.
 *  @return \ref ERROR при неудаче.
 */
STATUS remove(const char *path);

/**
 *  @brief Прочитать буфер из файла.
 *  @details Функция читает в буфер из файла заданное кол-во байт.
 *  
 *  @param fd Дескриптор файла.
 *  @param buffer Указатель на буфер.
 *  @param maxBytes Заданное количество байт.
 *  @return Количество фактически прочитанных байт.
 */
int read(int fd, void *buffer, int maxBytes);

/**
 *  @brief Записать буфер в файл.
 *  @details Функция записывает из буфера в файл заданное кол-во байт.
 *  
 *  @param fd Дескриптор файла.
 *  @param buffer Указатель на буфер.
 *  @param maxBytes Заданное количество байт.
 *  @return Количество фактически записанных байт.
 */
int write(int fd, const void *buffer, int maxBytes);

/**
 *  @brief \b ioctl-запрос к файлу.
 *  @details Функция выполняет \b ioctl-запрос к файлу
 *  
 *  @param fd Дескриптор файла.
 *  @param function Код запроса.
 *  @param arg Аргумент запроса.
 *  @return Определяется типом запроса. Если запрос не поддерживается драйвером, то возвращается -1.
 */
int ioctl(int fd, int function, int arg);
/// @}

/**
 *  @name Стандартные надстройки над ioctl
 *  @{
 */

/**
 *  @brief Переместить указатель чтения / записи.
 *  @details Функция перемещает указатель чтения / записи в файле.
 *  
 *  @param fd дескриптор файла
 *  @param shift смещение указателя чтения/записи в файле
 *  @param seektype Позиция указателя, относительно которой будет выполняться смещение.
 *  Такая позиция задаётся одной из следующих констант, определённых в stdio.h:
 *  - \ref SEEK_SET — Начало файла.
 *  - \ref SEEK_CUR — Текущее положение файла.
 *  - \ref SEEK_END – Конец файла.
 *  @return \ref OK.
 *  @return \ref ERROR при неудаче.
 */
int seek(int fd, int shift, int seektype);

/**
 *  @brief Переместить указатель чтения / записи (алиас \ref seek).
 *  @details Функция перемещает указатель чтения / записи в файле.
 *
 *  @param fd дескриптор файла
 *  @param shift смещение указателя чтения/записи в файле
 *  @param seektype Позиция указателя, относительно которой будет выполняться смещение.
 *  Такая позиция задаётся одной из следующих констант, определённых в stdio.h:
 *  - \ref SEEK_SET — Начало файла.
 *  - \ref SEEK_CUR — Текущее положение файла.
 *  - \ref SEEK_END – Конец файла.
 *  @return \ref OK.
 *  @return \ref ERROR при неудаче.
 */
int lseek(int fd, int shift, int seektype);

/**
 *  @brief Положение указателя в файле.
 *  @details Функция возвращает текущее значение положения указателя в файле.
 *  
 *  @param fd Дескриптор файла.
 *  @return Функция возвращает текущую позицию.
 */
int tell(int fd);

/**
 *  @brief Сброс всех изменений файла.
 *  @details Функция сбрасывает все изменения для файла в памяти в файл на устройстве.
 *  
 *  @param fd Дескриптор файла.
 *  @return При успешном завершении функция возвращает 0 и ненулевое значение при неудаче.
 */
int flush(int fd);
/// @}

/**
 *  @name Процедуры работы с каталогами
 *  @{
 */

/**
 *  @brief Поиск первого файла в каталоге.
 *  @details Функция отыскивает первый файл в каталоге, удовлетворяющий заданным условиям.
 *  
 *  @param pathMask Маска для поиска нужных файлов.
 *  @param ff Указатель на вспомогательную структуру для поиска.
 *  Функция заносит в эту структуру данные для последующих поисковых запросов.
 *  @return Если файл, удовлетворяющий условиям поиска, найден, то возвращает \b 0,
 *  в другом случае возвращает ненулевое значение.
 */
STATUS findFirst(const char *pathMask, FFBLK *ff);

/**
 *  @brief Поиск следующего файла в каталоге.
 *  @details Функция отыскивает следующий файл в каталоге, удовлетворяющий заданным условиям.
 *  
 *  @param ff Указатель на вспомогательную структуру для поиска, заполненную функцией findFirst().
 *  @return Если очередной файл, удовлетворяющий условиям поиска, найден, то возвращает \b 0,
 *  в другом случае возвращает ненулевое значение.
 */
STATUS findNext(FFBLK *ff);


/**
 *  @brief Получить двухмерный массив названий файлов.
 *  @details Функция получает двухмерный массив названий файлов (не системных, не скрытых, не папок)
 *  из указанной папки. Массив аллокируется из кучи, по окончанию использования его необходимо освободить!
 *  Максимальное количество файлов - до 1000, если файлов больше, то они не будут выведены. *
 *  @param dirName Имя каталога для поиска.
 *  @param[out] filesAmount Указатель на выходную переменную. При успешной отработке функции в переменную будет записан размер итогового массива.
 *  @return Двухмерный массив строк размером, соответствующим значению переменной filesAmount, или NULL при ошибке или отсутствии файлов.
 *  @note Для освобождения памяти следует по-очери освободить каждую строку в массиве, после чего освободить сам массив.
 */
char** getFilesInDir(const char* dirName, int* filesAmount);

/**
 * Отформатировать устройство, если это возможно.
 * @param devName Идентификатор устройства.
 * @return \ref OK при успехе, \ref ERROR при ошибке или если форматирование не возможно для устройства.
 */
STATUS formatVolume(const char *devName);

/**
 * Получить метку тома для устройства, если она имеется.
 * @param devName Идентификатор устройства.
 * @param buf Буфер под метку размером минимум 12 байт.
 * @return \ref OK при успехе, \ref ERROR при ошибке или если устройство не может иметь идентификатор тома.
 */
STATUS getVolumeLabel(const char *devName, char buf[static 12]);

/**
 * Получить размер свободного места на устройстве, если это имеет смысл.
 * @param devName Идентификатор устройства.
 * @return Свободное место в байтах в виде целого >= 0 при успехе, -1 при ошибке или если "свободное место" не имеет смысла для устройства.
 */
int freeSpace(const char *devName);

/**
 * Получить размер файла.
 * @param fileName Путь к файлу.
 * @return Размер файла или -1 в случае ошибки.
 */
int fileSize(const char *fileName);

/**
 * Проверить наличие файла на диске.
 * @param fileName Путь к файлу.
 * @return \ref true, если файл существует, \ref false, если нет.
 */
bool fileExists(const char *fileName);

/**
 * Проверить наличие каталога на диске.
 * @param dirName Путь к каталогу.
 * @return \ref true, если каталог существует, \ref false, если нет.
 */
bool dirExists(const char *dirName);

/**
 * Получить полный путь до файла по его дескриптору.
 * @param fd Дескриптор файла.
 * @return Выделенная в памяти строка с полным путем к файлу или \ref NULL при ошибке.
 */
char *getFullFileName(int fd);

/// @}


// Сервисные функции ввода-вывода.

/**
 * @brief Вывести в stdout список подключенных устройств.
 * @details Функция выводит список всех подключенных устройств в stdout.
 * @return Общее кол-во подключенных устройств.
 */
size_t iosDevShow(void);

/**
 *  @brief Вывести в stdout список открытых файлов.
 *  @details Функция выводит список всех открытых файлов в stdout.
 */
void iosFdShow(void);

/**
 *  @brief Перенаправление глобального (стандартного) ввода/вывода.
 *  @details Функция перенаправляет весь стандартный ввод/вывод
 *  на указанное устройство. Не меняет потоки ввода-вывода для задач с явно указанными потоками.
 *  
 *  @param ioe Поток: 0 = \b stdin, 1 = \b stdout, 2 = \b stderr.
 *  @param fd Дескриптор устройства, куда требуется перенаправить поток.
 *  @return \ref OK при успешном выполнении.
 *  @return \ref ERROR при ошибке в параметрах.
 */
STATUS ioGlobalStdSet(int ioe, int fd);

/**
 *  @brief Перенаправление ввода/вывода для отдельной задачи.
 *  @details Функция перенаправляет ввод/вывод для отдельной задачи.
 *  
 *  @param task Задача, ввод/вывод которой требуется перенаправить или NULL, для перенаправления ввода/вывода текущей задачи.
 *  @param ioe Поток: 0 = \b stdin, 1 = \b stdout, 2 = \b stderr.
 *  @param fd Дескриптор устройства, куда требуется перенаправить поток.
 *  @return \ref OK при успешном выполнении.
 *  @return \ref ERROR при ошибке в параметрах.
 */
STATUS ioTaskStdSet(TASK_ID task, int ioe, int fd); // Для отдельной задачи

/**
 * Получить заголовок с блоком управления устройством по его имени.
 * @param devName Идентификатор устройства.
 * @return Указатель на структуру типа \ref DEV_HDR или \ref NULL при ошибке.
 */
DEV_HDR *findDev(const char *devName);

/**
 * @private
 * @brief Получить указатель на блок управления устройством по его имени.
 * @param devName Идентификатор устройства.
 * @return Указатель на структуру типа \ref BLK_DEV или \ref NULL при ошибке.
 */
BLK_DEV *ioFindBlkDevice(const char *devName);

/**
 * @private
 * @brief Вывести в stdout информацию об устройстве.
 * @param devName Идентификатор устройства.
 * @return \ref OK при успешном выводе, \ref ERROR иначе.
 */
STATUS printPartInfo(const char *devName);

/**
 * Освободить структуру блока управления файла.
 * @param fcb Указатель на структуру блока управления файла.
 */
void freeFCB(FCB *fcb);

/**
 * Получить структуру блока управления файла по его дескриптору.
 * @param fd Дескриптор файла.
 * @return Указатель на структуру блока управления файла или NULL при ошибке.
 */
FCB *findFCB(int fd);

/**
 * Найти открытый дескриптор по имени файла/устройства.
 * @param name Имя файла/устройства.
 * @return Дескриптор в виде положительного числа, либо -1, если дескриптор не был найден.
 */
int findFd(const char *name);

/**
 * Изменить рабочий каталог.
 * @param newWorkDir Путь к новому рабочему каталогу.
 * @return \ref OK при успехе, \ref ERROR при ошибке.
 */
STATUS cd(const char *newWorkDir);

/**
 * Изменить рабочий каталог на его родительский каталог.
 * @details Аналог команды 'cd ../'.
 * @return \ref OK при успехе, \ref ERROR при ошибке.
 */
STATUS upd(void);

/**
 * Получить текущий рабочий каталог.
 * @param pwd Буфер, в который будет записан текущий диск:каталог.
 * @return \ref OK при успехе, \ref ERROR при ошибке.
 */
STATUS getWorkDir(char *pwd);

/**
 * Получить текущий рабочий каталог (безопасный аналог getWorkDir).
 * @param pwd Буфер размера bufSize, в который будет записан текущий диск:каталог.
 * @param bufSize Размер буфера.
 * @return \ref OK при успехе, \ref ERROR при ошибке.
 */
STATUS getWorkDir_s(char *pwd, size_t bufSize);

/**
 * Удалить пустой каталог.
 * @details Не-пустой или read-only каталог удалить не получится.
 * @param name Путь к каталогу.
 * @return \ref OK при успехе, \ref ERROR при ошибке.
 */
STATUS rmdir(const char *name);

/**
 * Создать каталог.
 * @details Если файл/каталог с таким названием уже существует, то функция вернет ошибку.
 * @param name Путь к каталогу.
 * @return \ref OK при успехе, \ref ERROR при ошибке.
 */
STATUS mkdir(const char *name);


/**
 * Провести проверку устройства и вывести результаты проверки в stdout.
 * @details Фактически проверка будет корректно проводится только для подключенных файловых томов.
 * @param devName Идентификатор устройства.
 * @return \ref OK при успехе, \ref ERROR при ошибке.
 */
STATUS chkDsk(const char *devName);

/**
 *  @brief Сброс контроллера \b IDE.
 *  @details Сброс контроллера \b IDE.
 *  
 *  @param devName Идентификатор устройства.
 *  @return \ref OK при успехе, \ref ERROR при ошибке.
 */
int reset(const char *devName);

/**
 *  @brief Разгрузка съемного носителя.
 *  @details Разгрузка съемного носителя.
 *  
 *  @param devName Идентификатор устройства.
 *  @return \ref OK при успехе, \ref ERROR при ошибке.
 */
int unloadVolume(const char *devName);


/**
 *  @brief \b ioctl-запрос к файлу \ref FIOGETLNAME.
 *  @details \b ioctl-запрос к файлу с кодом запроса \ref FIOGETLNAME без аргументов.
 *  
 *  @param fd Дескриптор файла.
 *  @return Определяется типом запроса. Если запрос не поддерживается драйвером, то возвращается -1.
 */
int getLongName(int fd);


/**
 * @private
 * @brief Проинициализировать систему ввода-вывода.
 * @param drivers Максимальное кол-во возможных драйверов ввода-вывода.
 * @param files Максимальное кол-во возможных файловых дескрипторов.
 * @return \ref OK при успехе, \ref ERROR при ошибке.
 * @warning Никогда не должна явно вызываться пользователем.
 */
STATUS initIoLib(int drivers, int files);

#endif
