/**
    @addtogroup groupStdTypes
    @{
    @file list.h
    @brief Не-потокобезопасный двухсвязный список.
    @details Работа с списками.
    @}
 */
/*
 * Авторы: Ларюшин Александр Андреевич
 * 	   364961@mail.ru
 *         Дубович Тимофей Михайлович
 *	   t.dubovich@setel.pro
*/

#ifndef _ELEM_LIST_H_
#define _ELEM_LIST_H_


#include <multex.h>

/**
 * Узел двухсвязного списка.
 */
typedef struct listNode
{
    void *pData;			//!< Указатель на данные в узле.
    struct listNode *pPrev;	//!< Указатель на предыдущий узел.
    struct listNode *pNext; //!< Указатель на следующий узел.
} sListNode;

/**
 * Двухсвязный список.
 */
typedef struct
{
    sListNode *pFirst;  //!< Указатель на первый узел в списке.
    sListNode *pLast;   //!< Указатель на последний узел в списке.
    size_t NumOfNodes;  //!< Кол-во узлов в списке.
} sList;

//Все нижеперечисленные функции НЕ потокобезопасны

//! @brief Макрос для инициализации списка в сегменте данных. Инициализация производится в виде sList* MyListPtr = INIT_STATIC_LIST(); Освобождение такого списка недопустимо!
#define INIT_STATIC_LIST() (&(sList){.pFirst = 0, .pLast = 0, .NumOfNodes = 0})

//! @brief Макрос для получения поля структуры, хранимой в списке.
#define NODE_FIELD(node, ptrType, field) (( (ptrType) ( node->pData ) )->field)


/**
 * @brief Создать новый двухсвязный список.
 * @return Указатель на sList или NULL при ошибке.
 */
sList* list_CreateNewList(void);

/**
 * @brief Создать новый узел списка и скопировать в него данные.
 * @details Данные из аргумента будут скопированы, а значение pData в узле будет указывать на эту копию.
 * @param pData Указатель на данные.
 * @param size Размер данных.
 * @return Указатель на sListNode или NULL при ошибке.
 * @note При освобождении списка нужно будет предварительно вручную освободить копии данных.
 */
sListNode* list_CreateNewNode(const void* pData, size_t size);

/**
 * @brief Создать новый узел списка без копирования данных.
 * @details Данные из аргумента НЕ будут скопированы, а значение pData в узле будет иметь значение аргумента.
 * @param pData Указатель на данные или NULL.
 * @param size Размер данных.
 * @return Указатель на sListNode или NULL при ошибке.
 */
sListNode* list_CreateNewNodeWithoutDataCopy(void* pData, size_t size);

/**
 * @brief Вставить узел в конец списка.
 * @param pList Указатель на список.
 * @param pNewNode Указатель на вставляемый узел.
 * @return OK при успехе, ERROR иначе.
 */
STATUS list_AddToBack (sList* pList, sListNode* pNewNode);

/**
 * @brief Вставить узел в начало списка.
 * @param pList Указатель на список.
 * @param pNewNode Указатель на вставляемый узел.
 * @return OK при успехе, ERROR иначе.
 */
STATUS list_AddToHead (sList* pList, sListNode* pNewNode);

/**
 * @brief Удалить узел из начала списка.
 * @param pList Указатель на список.
 * @return OK при успехе, ERROR иначе.
 * @details И узел, и данные в нем будут освобождены.
 */
STATUS list_RemoveFromHead (sList* pList);

/**
 * @brief Удалить узел из конца списка.
 * @param pList Указатель на список.
 * @return OK при успехе, ERROR иначе.
 * @details И узел, и данные в нем будут освобождены.
 */
STATUS list_RemoveFromBack (sList* pList);

/**
 * @brief Удалить узел по его индексу.
 * @param pList Указатель на список.
 * @param index Индекс удаляемого узла.
 * @return OK при успехе, ERROR иначе.
 * @details И узел, и данные в нем будут освобождены.
 */
STATUS list_RemoveByIndex (sList* pList, size_t index);

/**
 * @brief Удалить узел по его указателю.
 * @param pList Указатель на список.
 * @param pTargetNode Указатель на удаляемый узел.
 * @return OK при успехе, ERROR иначе.
 * @details И узел, и данные в нем будут освобождены.
 */
STATUS list_RemoveByNodePointer (sList* pList, sListNode* pTargetNode);

/**
 * @brief Вырезать узел из начала списка.
 * @param pList Указатель на список.
 * @return OK при успехе, ERROR иначе.
 * @details Узел и данные в нем не будут освобождены.
 */
STATUS list_CutFromHead (sList* pList);

/**
 * @brief Вырезать узел из конца списка.
 * @param pList Указатель на список.
 * @return OK при успехе, ERROR иначе.
 * @details Узел и данные в нем не будут освобождены.
 */
STATUS list_CutFromBack (sList* pList);

/**
 * @brief Вырезать узел по его индексу.
 * @param pList Указатель на список.
 * @param index Индекс вырезаемого узла.
 * @return OK при успехе, ERROR иначе.
 * @details Узел и данные в нем не будут освобождены.
 */
STATUS list_CutByIndex (sList* pList, size_t index);

/**
 * @brief Вырезать узел по его указателю.
 * @param pList Указатель на список.
 * @param pTargetNode Указатель на вырезаемый узел.
 * @return OK при успехе, ERROR иначе.
 * @details Узел и данные в нем не будут освобождены.
 */
STATUS list_CutByNodePointer (sList* pList, sListNode* pTargetNode);

/**
 * @brief Вызвать функцию для каждого элемента в списке.
 * @param pList Указатель на список.
 * @param pFunc Указатель на функцию, которая будет вызывана для каждого узла в списке. В качестве аргумента в функцию будет отправлен указатель на данные в узле.
 */
void list_DoForeach(sList *pList, void (*pFunc)(void *pData));

/**
 * @brief Освободить все ресурсы занимаемые списком.
 * @param pList Указатель на список.
 * @details Не освобождает ресурсы, занимаемые хранящимися в списке данными.
 */
void list_FreeNodesAndList(sList *pList);

/**
 * @brief Освободить все ресурсы занимаемые списком и хранящимися в нем данными.
 * @param pList Указатель на список.
 * @details Работает идентично list_FreeNodesAndList, но дополнительно вызывает free() для поля pData в каждом узле списка.
 */
void list_FreeAll(sList *pList);




//----------------------------------------------
//   Те же интерфейсы, просто названия другие
//----------------------------------------------

/**
 * @privatesection
 * @{
 */

#include <assert.h>

typedef struct _elemNode
{
    void *nodeVal;                  //!< Указатель на данные в узле.
    struct _elemNode *pPrev;        //!< Указатель на предыдущий узел.
    struct _elemNode *pNext;        //!< Указатель на следующий узел.
} elemNode;
static_assert(sizeof(sListNode) == sizeof(elemNode));

typedef struct
{
    elemNode *pFirst;   //!< Указатель на первый узел в списке.
    elemNode *pLast;    //!< Указатель на последний узел в списке.
    size_t numOfNodes;         //!< Кол-во узлов в списке.
} elemList;
static_assert(sizeof(sList) == sizeof(elemList));


elemList* createNewElemList();
elemNode* createNewElemItemNode (void* pData, int size); //Данные по указателю будут скопированы в узел
elemNode* createNewElemItemNodeNoDataCopy (void* pData, int size);

void addItemNodeToBack (elemList* pDstSeq, elemNode* pNewNode);
void addItemNodeToHead (elemList* pDstSeq, elemNode* pNewNode);

//После удаления элемента списка или всего списка необходимо также удалить сами данные
STATUS removeItemNodeFromHead (elemList* pDstSeq);
STATUS removeItemNodeFromBack (elemList* pDstSeq);
STATUS removeItemNodeByIndex (elemList* pDstSeq, int index);
STATUS removeItemNodeByNodePtr (elemList* pDstSeq, elemNode* pNode);

void elemListDoFuncForeachNode(elemList* pDstSeq, void (*func_ptr)(void *pData)); //Вызывает функцию func_ptr для nodeVal для каждого узла в списке

void freeElemListAndAllNodes(elemList* pDstSeq);

/// @}

#endif
