/**
    @addtogroup groupCore
    @{
    @file memlib.h
    @brief Управление диспетчером памяти.
    @details Файл содержит объявления методов управления диспетчером памяти.
    @see Общее описание см. в главе \ref sectionMemLib.
    @}

    @page pageCore

    @section sectionMemLib Диспетчер памяти
    @see Описание методов управления памятью см. в файле memlib.h.

    Ядро @osName включает комплекс процедур, обеспечивающих работу с динамически
    выделяемыми блоками памяти (\b Heap-memory). Этот комплекс, называемый
    диспетчером динамической памяти, позволяет задачам ядра и пользовательским
    задачам выделять для собственных нужд блоки памяти требуемого размера и
    освобождать их по мере надобности. Функции диспетчера памяти используются
    ядром @osName при создании задач, семафоров и очередей.

    Особенностью реализации диспетчера памяти в @osName является принцип
    выделения блоков памяти от старших адресов к младшим. При этом наибольший
    свободный сегмент памяти всегда оказывается первым в цепочке. Это очень
    важно для повышения быстродействия системы. Диспетчер обеспечивает защиту
    от сегментации памяти – при высвобождении блоков производится слияние
    смежных свободных блоков.

    Для обеспечения защиты от множественного доступа к диспетчеру памяти
    принято наиболее простое и эффективное решение:

    - Так как диспетчер памяти не может быть защищен семафорами
    взаимоисключений (сама работа с семафорами нуждается в функциях диспетчера
    памяти), то на время цикла выделения/высвобождения блока памяти процессор
    закрывается, что гарантирует непрерываемость операции.

 */

#ifndef _MEMLIB
#define _MEMLIB

#ifdef __cplusplus
extern "C" {
#endif

#include "multex.h"

/**
 * @name Функции из stdlib.
 * @{
 */

/**
 * Выделить блок памяти для объекта.
 * @param size Размер запрашиваемого блока. Размер может быть
 * равен нулю, при этом будет выделен минимально возможный блок памяти.
 * @return Указатель на блок памяти или NULL, в случае ошибки.
 */
void *malloc (size_t size);

/**
 *  @brief Выделить подписанный блок памяти для объекта.
 *  @details Выделение памяти с помощью malloc() с добавлением
 *  подписи владельца блока памяти. В дальнейшем подписи блоков можно 
 *  посмотреть с помощью memReport().
 *  
 *  @param size Размер запрашиваемого блока.
 *  @param owner Владелец выделяемого блока -- строка не более 239 символов.
 *  @return Указатель на блок памяти или NULL, в случае ошибки.
 */
void *mallocForOwner (size_t size, const char *owner);

/**
 * Выделить блок памяти для массива и инициализировать его нулями.
 * @param nmemb Количество элементов в массиве.
 * @param size Размер одного элемента.
 * @return Указатель на блок памяти или NULL, в случае ошибки.
 */
void *calloc (size_t nmemb, size_t size);

/**
 * Выделить новый блок памяти заданного размера вместо старого, сохранив при этом данные из старого блока.
 * @param ptr Указатель на старый блок памяти.
 * @param size Размер нового блока памяти.
 * @return Указатель на блок памяти или NULL, в случае ошибки.
 */
void *realloc (void *ptr, size_t size);

/**
 * Освободить блок памяти.
 * @param ptr Указатель на блок памяти для освобождения.
 * @warning Функция проверяет корректность заголовка блока памяти
 * и освобождает память только в случае его правильности.
 * @return \ref OK при удачном освобождении памяти, иначе \ref ERROR.
 */
STATUS free (void *ptr);

/**
 *  @brief Освободить блок памяти (с подписью).
 *  @details Функция используется для отладки.
 *  В случае неудачного освобождение памяти выводит сообщение
 *  об ошибке с указанным именем.
 *
 *  @param p Указатель на блок памяти для освобождения.
 *  @param name Подпись, выводимая при ошибке.
 *  @return \ref OK при удачном освобождении памяти, иначе \ref ERROR.
 */
STATUS mfree (void *p, const char *name);

/**
 *  @brief Вывод на печать заголовка блока памяти.
 *  @details Функция выводит в консоль информацию, записанную
 *  в заголовке выделенного с помощью malloc() блока памяти.
 *  Заголовок располагается перед каждым выделяемым блоком памяти
 *  и содержит системную информацию. Функция может пригодиться для отладки
 *  процесса управления памятью в проекте.
 *
 *  @param ptr Указатель на начало блока памяти.
 */
void minfo (const void *ptr);

#if __STDC_VERSION__ >= 201112L  // C11 or higher
// /**
//  * Выделить блок памяти для объекта с заданным выравниванием в памяти.
//  * @param alignment Выравнивание запрашиваемого блока.
//  * @param size Размер запрашиваемого блока.
//  * @return Указатель на блок памяти или NULL, в случае ошибки.
//  * @warning РЕАЛИЗАЦИЯ ФУНКЦИИ ОТСУТСТВУЕТ.
//  */
// void* aligned_alloc(size_t alignment, size_t size);
#endif

/// @}

/**
 *  @name Макросы обеспечения совместимости
 *  @{
 */
#define __free free
#define kmalloc(n, z) malloc (n)
#define kfree(x) free (x)
/// @}

/**
 *  @brief Получить размер свободной памяти.
 *  @details Функция возвращает общий размер свободной динамической памяти в байтах.
 *  @return Размер памяти в байтах.
 */
size_t memAvail (void);

/**
 *  @brief Получить отчёт о занятых блоках памяти.
 *  @details Функция возвращает работает аналогично memAvail(),
 *  кроме того выводит отчёт о всех выделенных с помощью mallocForOwner()
 *  блоках памяти в консоль.
 *  
 *  @param allBlocks Показывать все блоки, иначе будут показаны только
 *  подписанные блоки (владелец которых определён).
 *  @return Размер памяти в байтах.
 */
size_t memReport (bool allBlocks);

/**
 *  @brief Получить отчёт о занятых блоках с маскированием.
 *  @details Функция возвращает работает аналогично memAvail(),
 *  кроме того выводит отчёт о всех выделенных с помощью mallocForOwner()
 *  блоках памяти в консоль кроме блоков, подпись которых содержит
 *  подстрок, содержащуюся в маске. Например для маскирования
 *  блоков памяти, с которыми в данный момент работает декодер h.264
 *  следует указать маску:
 *  
 *  @code
 *  memReportMask ("vid_,h264_");
 *  @endcode
 * 
 *  Функцию можно вызывать из консоли, например:
 * 
 *  @verbatim
 *  memReportMask "vid_,h264_"
 *  @endverbatim
 *  
 *  @param mask Строка содержащая строки, разделённые запятыми.
 *  Блоки памяти, содержащие в подписи указанные в маске куски,
 *  не будут выведены в отчёте. Достаточно указывать любую часть
 *  маскируемой подписи.
 *  @return Размер памяти в байтах. 
 */
size_t memReportMask (char *mask);

/**
 *  @brief Получить размер максимального непрерывного сегмента памяти.
 *  @details Функция возвращает размер самого большого свободного сегмента динамической памяти в байтах.
 *  @return Размер сегмента в байтах.
 */
size_t maxAvail (void);

/**
 *  @brief Инициализация диспетчера памяти.
 *  @details Функция инициализирует диспетчер памяти.
 *  @warning Эта функция вызывается ядром @osName внутри вызова \b kernelInit.
 *  Пользователь не должен вызывать эту функцию из своих задач,
 *  так как это приведет к краху системы.
 *
 *  @param begAddr Start of heap memory pool.
 *  @param endAddr End of heap memory pool.
 */
void initMemLib (size_t begAddr, size_t endAddr);

/// @privatesection

/* Клонирование блока памяти */
void *memdup (void *ptr);

/* Получение размера выделенного блока памяти */
size_t memsize (void *ptr);

/* To trasse of heap memory pool */
void setFirstMBInfo (void);

/* Heap memory cluster information structure */
typedef struct
{
    UINT start;
    UINT size;
    UINT flags;
} heap_blk_info;

UINT getNextMBInfo (heap_blk_info *info);

// Additional debug's functions (added by Laryushin) 122 >>
void *dbgMalloc (size_t nBytes, char *pName, int lineNum);
void dbgMemPrintWholeDbgMem (int unitsInLine);
void dbgMemPrintDiffMem (bool silentMode);
void dbgMemCalcBlocksAmmount (bool isEmpty);
void dbgMemSetNewMaxPossibleMemsAmmount (int newSize);

#ifdef __cplusplus
}
#endif

#endif
