/**
    @addtogroup groupMultimedia
    @{
    @file a20graph.h
    @brief Работа с графической подсистемой.
    @details Библиотека аппаратной поддержки <b>2D</b>-графики для процессора \b Allwinner \b A20.

    <b>Подключение</b>: @verbatim #include <multimedia/a20graph.h> @endverbatim
    \ref sectionMakefile "Makefile": @verbatim LIBRARIES += -l_a20graph -l_png -l_z @endverbatim

    @see Общее описание работы с графической подсистемой в главе \ref sectionA20graph.
    @}
 */

#ifndef A20GRAPH_H_
#define A20GRAPH_H_
/*
Header file: VideoModes.h
*/

#ifdef __cplusplus
extern "C" {
#endif
#include <multex.h>  //Типы навроде __u32
#include <multimedia/colorspace.h>
#include <multimedia/g_display.h>

/**
 *  @details Выбор режима работы графического адаптера (HDMI/TV-Out).
 */
typedef enum {
    MODE_640x480 = 0,  ///< Режим работы адаптера: HDMI 640x480
    MODE_800x480,      ///< Режим работы адаптера: HDMI 800x480
    MODE_800x600,      ///< Режим работы адаптера: HDMI 800x600
    MODE_1024x768,     ///< Режим работы адаптера: HDMI 1024x768
    MODE_1280x720,     ///< Режим работы адаптера: HDMI 1280x720
    MODE_1280x768,     ///< Режим работы адаптера: HDMI 1280x768
    MODE_1280x800,     ///< Режим работы адаптера: HDMI 1280x800
    MODE_1368x768,     ///< Режим работы адаптера: HDMI 1368x768
    MODE_1280x1024,    ///< Режим работы адаптера: HDMI 1280x1024
    MODE_1920x1080,    ///< Режим работы адаптера: HDMI 1920x1080
    MODE_TV            ///< Режим работы адаптера: TV-Out
} VideoModes;

/**
 *  @details Выбор режима работы графического адаптера (LVDS).
 */
typedef enum {
    LVDS_1920x1080 = 0,  ///< Режим работы адаптера: LVDS 1920x1080
    LVDS_1920x360,       ///< Режим работы адаптера: LVDS 1920x360
    LVDS_1024x768,       ///< Режим работы адаптера: LVDS 1024x768
    LVDS_800x600,        ///< Режим работы адаптера: LVDS 800x600
    LVDS_800x480,        ///< Режим работы адаптера: LVDS 800x480
    LVDS_1280x800,       ///< Режим работы адаптера: LVDS 1280x800
    LVDS_158x1920,       ///< Режим работы адаптера: LVDS 158x1920
    LVDS_1920x165,       ///< Режим работы адаптера: LVDS 1920x165
    LVDS_1400x1050,      ///< Режим работы адаптера: LVDS 1400x1050
} lvds_param_t;

/**
 *  @brief Структура инициализации графического адаптера.
 */
struct Display {
    int VideoMode;             ///< Видео режим из набора \ref lvds_param_t.
    char *modeline;            ///< Строка-описатель (Xfree86 style modeline).
    struct fb_videomode mode;  ///< Настройки растра.
    int BPP;                   ///< Bit Per Pixel.
    int Width;                 ///< Ширина экрана в пикселях.
    int Height;                ///< Высота экрана в пикселях.
    int DSize;                 ///< Размер видимой области в байтах.
    void *Screen;              ///< Указатель на отображаемую область.
    void *Constr;              ///< Указатель на конструируемую область.
    int LFB;                   ///< Адрес начала видеопамяти.
    int interface;             ///< Интерфейс: \b HDMI, \b LCD, \b LVDS.
};

extern struct Display Display;  ///< Глобальная переменная описатель экрана.

/**
 *  @name Инициализация и управление графическим адаптером
 *  @brief Выбор режима инициализации графического адаптера.
 *  Функции управления и опроса состояния.
 *  @{
 */

/**
 *  @brief Инициализация адаптера \b HDMI / TV-Out.
 *  @details Функция инициализирует адаптер в режиме \b HDMI или \b TV-Out.
 *
 *  @param MODE Режим работы и разрешение выбирается из списка \ref VideoModes.
 *  @param BPP Количество бит на точку. Следует указать 32 или 16.
 *  @return \ref OK При успешном завершении.
 *  @return \ref ERROR При неудаче.
 */
int setVideoMode (int MODE, int BPP);

/**
 *  @brief Инициализация адаптера \b LVDS.
 *  @details Функция инициализирует видеоадаптер в режиме \b LVDS.
 *  По умолчанию \b LVDS будет инициализирован в режиме \b NS с разрешением \b FULL \b HD.
 *  Для инициализации в режиме \b JEDA нужно вызвать функцию set_lvds_mode() с параметром \b 1.
 *  Если требуется установить другое разрешение, то перед инициализацией
 *  следует вызвать функцию set_lvds_param().
 *  @return \ref OK При успешном завершении.
 *  @return \ref ERROR При неудаче.
 */
int initLvdsDisplay ();  // Full HD @72Hz LVDS

/**
 *  @brief Выбор разрешения \b LVDS.
 *  @details Для выбора разрешения \b LVDS отличного от разрешения по умолчанию (\b FULL \b HD)
 *  нужно вызвать функцию перед инициализацией initLvdsDisplay().
 *
 *  @param lp Разрешение экрана \b LVDS выбирается из списка \ref lvds_param_t.
 */
void set_lvds_param (lvds_param_t lp);

/**
 *  @brief Смена режима \b LVDS.
 *  @details Для выбора режима \b LVDS отличного от режима по умолчанию (\b NS)
 *  нужно вызвать данную функцию с параметром \b 1 перед инициализацией initLvdsDisplay().
 *  @param mode
 */
void set_lvds_mode (int mode);


/**
 *  @brief Ожидание обратного хода луча.
 *  @details Функция ожидает начала обратного хода луча
 *  на мониторе для избежания появления строба. Является
 *  синонимом функции sunxi_wait_retrace().
 *
 *  @return Всегда 0.
 */
int waitVerticalRetrace ();

//// ОПРОСЫ СОСТОЯНИЯ ГРАФИЧЕСКОЙ ПОДСИСТЕМЫ

/**
 *  @brief Опрос видимой ширины экрана.
 *  @return Видимую ширину экрана в пикселях.
 */
signed long getScreenWidth (void);

/**
 *  @brief Опрос реальной длины строки в памяти.
 *  @return Длину строки в памяти в байтах.
 */
signed long getScreenPitch (void);

/**
 *  @brief Опрос количества строк на экране.
 *  @return Количество строк выводимых на экран. 
 */
signed long getScreenHeight (void);

/**
 *  @brief Опрос количества бит на точку.
 *  @return Количество бит на точку в выдранном формате цвета.
 */
signed long getScreenBitsPerPixel (void);

/**
 *  @brief Получить указатель на видимый экран.
 *  @return Указатель на видимый экран.
 */
void *getScreenBuffer (void);

/**
 *  @brief Получить указатель на конструируемый экран.
 *  @return Указатель на конструируемый экран.
 */
void *getConstrBuffer (void);

/**
 *  @brief Получить указатель на видимый 2-ой экран.
 *  @return Указатель на видимый 2-ой экран.
 */
void *getScreen2Buffer (void);

/**
 *  @brief Получить указатель на конструируемый 2-ой экран.
 *  @return Указатель на конструируемый 2-ой экран.
 */
void *getConstr2Buffer (void);

/**
 *  @brief Получить указатель на начало видеопамяти.
 *  @return Указатель на начало видеопамяти.
 */
void *getLFB (void);

/// @}



/**************************************************************************

  2D - Графика

***************************************************************************/

/** Форматы смешивания цвета. */
typedef enum {
    G2D_FMT_ARGB_AYUV8888 = (0x0),  ///< @deprecated То же, что  \ref G2D_FMT_ARGB8888 (не рекомендуется использовать в новых проектах).
    G2D_FMT_BGRA_VUYA8888 = (0x1),  ///< @deprecated То же, что  \ref G2D_FMT_BGRA8888 (не рекомендуется использовать в новых проектах).
    G2D_FMT_ABGR_AVUY8888 = (0x2),  ///< @deprecated То же, что  \ref G2D_FMT_ABGR8888 (не рекомендуется использовать в новых проектах).
    G2D_FMT_RGBA_YUVA8888 = (0x3),  ///< @deprecated То же, что  \ref G2D_FMT_RGBA8888 (не рекомендуется использовать в новых проектах).

    G2D_FMT_ARGB8888 = (0x0),  ///< Формат цвета с альфа-каналом \b ARGB 8-8-8-8.
    G2D_FMT_BGRA8888 = (0x1),  ///< Формат цвета с альфа-каналом \b BGRA 8-8-8-8.
    G2D_FMT_ABGR8888 = (0x2),  ///< Формат цвета с альфа-каналом \b ABGR 8-8-8-8.
    G2D_FMT_RGBA8888 = (0x3),  ///< Формат цвета с альфа-каналом \b RGBA 8-8-8-8.

    G2D_FMT_XRGB8888 = (0x4),
    G2D_FMT_BGRX8888 = (0x5),
    G2D_FMT_XBGR8888 = (0x6),
    G2D_FMT_RGBX8888 = (0x7),

    G2D_FMT_ARGB4444 = (0x8),
    G2D_FMT_ABGR4444 = (0x9),
    G2D_FMT_RGBA4444 = (0xA),
    G2D_FMT_BGRA4444 = (0xB),

    G2D_FMT_ARGB1555 = (0xC),
    G2D_FMT_ABGR1555 = (0xD),
    G2D_FMT_RGBA5551 = (0xE),
    G2D_FMT_BGRA5551 = (0xF),

    G2D_FMT_RGB565 = (0x10),
    G2D_FMT_BGR565 = (0x11),

    G2D_FMT_IYUV422 = (0x12),

    G2D_FMT_8BPP_MONO = (0x13),
    G2D_FMT_4BPP_MONO = (0x14),
    G2D_FMT_2BPP_MONO = (0x15),
    G2D_FMT_1BPP_MONO = (0x16),

    G2D_FMT_PYUV422UVC = (0x17),
    G2D_FMT_PYUV420UVC = (0x18),
    G2D_FMT_PYUV411UVC = (0x19),

    /* just for output format */
    G2D_FMT_PYUV422 = (0x1A),
    G2D_FMT_PYUV420 = (0x1B),
    G2D_FMT_PYUV411 = (0x1C),

    /* just for input format */
    G2D_FMT_8BPP_PALETTE = (0x1D),
    G2D_FMT_4BPP_PALETTE = (0x1E),
    G2D_FMT_2BPP_PALETTE = (0x1F),
    G2D_FMT_1BPP_PALETTE = (0x20),

} g2d_data_fmt;

typedef enum {
    G2D_SEQ_NORMAL = 0x0,

    /* for interleaved yuv422 */
    G2D_SEQ_VYUY = 0x1,
    G2D_SEQ_YVYU = 0x2,

    /* for uv_combined yuv420 */
    G2D_SEQ_VUVU = 0x3,

    /* for 16bpp rgb */
    G2D_SEQ_P10 = 0x4,
    G2D_SEQ_P01 = 0x5,

    /* planar format or 8bpp rgb */
    G2D_SEQ_P3210 = 0x6,
    G2D_SEQ_P0123 = 0x7,

    /* for 4bpp rgb */
    G2D_SEQ_P76543210 = 0x8, /* 7,6,5,4,3,2,1,0 */
    G2D_SEQ_P67452301 = 0x9, /* 6,7,4,5,2,3,0,1 */
    G2D_SEQ_P10325476 = 0xA, /* 1,0,3,2,5,4,7,6 */
    G2D_SEQ_P01234567 = 0xB, /* 0,1,2,3,4,5,6,7 */

    /* for 2bpp rgb */
    G2D_SEQ_2BPP_BIG_BIG = 0xC,       /* 15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0 */
    G2D_SEQ_2BPP_BIG_LITTER = 0xD,    /* 12,13,14,15,8,9,10,11,4,5,6,7,0,1,2,3 */
    G2D_SEQ_2BPP_LITTER_BIG = 0xE,    /* 3,2,1,0,7,6,5,4,11,10,9,8,15,14,13,12 */
    G2D_SEQ_2BPP_LITTER_LITTER = 0xF, /* 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15 */

    /* for 1bpp rgb */
    G2D_SEQ_1BPP_BIG_BIG = 0x10,       /* 31,30,29,28,27,26,25,24,23,22,21,20,19,18,17,16,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0 */
    G2D_SEQ_1BPP_BIG_LITTER = 0x11,    /* 24,25,26,27,28,29,30,31,16,17,18,19,20,21,22,23,8,9,10,11,12,13,14,15,0,1,2,3,4,5,6,7 */
    G2D_SEQ_1BPP_LITTER_BIG = 0x12,    /* 7,6,5,4,3,2,1,0,15,14,13,12,11,10,9,8,23,22,21,20,19,18,17,16,31,30,29,28,27,26,25,24 */
    G2D_SEQ_1BPP_LITTER_LITTER = 0x13, /* 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31 */
} g2d_pixel_seq;

/**
 * Правила использования альфа-канала при заливке полигонов.
 * Значения параметра \b alpha: \em 0-255 соответствуют значению
 * прозрачности от 0.0 до 1.0.
 * @note Перемножение двух параметров прозрачности так же дают
 * значение от 0.0 до 1.0.
 */
typedef enum {
    G2D_FIL_NONE = 0x00000000,         ///< Заливка без учёта прозрачности.
    G2D_FIL_PIXEL_ALPHA = 0x00000001,  ///< Заливка с прозрачностью, указанной в качестве \b A формата цвета.
    G2D_FIL_PLANE_ALPHA = 0x00000002,  ///< Заливка с прозрачностью, указанной как аргумент функции \b alpha.
    G2D_FIL_MULTI_ALPHA = 0x00000004,  ///< При заливке используется прозрачность, полученная как результат перемножения
                                       ///  значений \ref G2D_FIL_PIXEL_ALPHA и \ref G2D_FIL_PLANE_ALPHA.
} g2d_fillrect_flags;

/**
 *  Правила (опции) копирования областей графики.
 *  Для альфа-канала значения параметра \b alpha: \em 0-255
 *  соответствуют значению прозрачности от 0.0 до 1.0.
 *  @note Перемножение двух параметров прозрачности так же дают
 *  значение от 0.0 до 1.0.
 */
typedef enum {
    /** Простое копирование. */
    G2D_BLT_NONE            = 0x00000000,
    /** Копирование с использованием альфа-канала.
        Используется параметр прозрачности,
        указанной в качестве \b A формата цвета.
    */
    G2D_BLT_PIXEL_ALPHA     = 0x00000001,
    /** Копирование с использованием альфа-канала.
        Используется параметр прозрачности,
        указанной как аргумент функции \b alpha.
    */
    G2D_BLT_PLANE_ALPHA     = 0x00000002,
    /** Копирование с использованием альфа-канала.
        Используется параметр прозрачности,
        полученным перемножением
        \ref G2D_BLT_PIXEL_ALPHA и \ref G2D_BLT_PLANE_ALPHA.
    */
    G2D_BLT_MULTI_ALPHA     = 0x00000004,
    /** Копирование с отбрасыванием ключевого цвета
        исходной поверхности.
    */
    G2D_BLT_SRC_COLORKEY    = 0x00000008,
    /** Копирование с отбрасыванием ключевого цвета
        результирующей поверхности.
        @see \ref subsubsectionGraphSurfaceBugColorKeyDst
    */
    G2D_BLT_DST_COLORKEY    = 0x00000010,
    /** Копирование с отражением по горизонтали. */
    G2D_BLT_FLIP_HORIZONTAL = 0x00000020,
    /** Копирование с отражением по вертикали. */
    G2D_BLT_FLIP_VERTICAL   = 0x00000040,
    /** Копирование с поворотом на 90° по направлению против часовой стрелки. 
        @see \ref subsubsectionGraphSurfaceBugRotate
    */
    G2D_BLT_ROTATE90        = 0x00000080,
    /** Копирование с поворотом на 180° по направлению против часовой стрелки. */
    G2D_BLT_ROTATE180       = 0x00000100,
    /** Копирование с поворотом на 270° по направлению против часовой стрелки.
        @see \ref subsubsectionGraphSurfaceBugRotate
    */
    G2D_BLT_ROTATE270       = 0x00000200,
    /** Копирование с отражением по диагонали 45°.
        @see \ref subsubsectionGraphSurfaceBugRotate
    */
    G2D_BLT_MIRROR45        = 0x00000400,
    /** Копирование с отражением по диагонали 135°.
        @see \ref subsubsectionGraphSurfaceBugRotate
    */
    G2D_BLT_MIRROR135       = 0x00000800,
} g2d_blt_flags;

/** Flip rectangle struct. */
typedef struct {
    __s32 x;  ///< @details Left top point coordinate x.
    __s32 y;  ///< @details Left top point coordinate y.
    __u32 w;  ///< @details Rectangle width.
    __u32 h;  ///< @details Rectangle height.
} g2d_rect;

/** image struct */
typedef struct {
    __u32 addr[3];            ///< @details Base addr of image frame buffer in byte.
    __u32 w;                  ///< @details Width of image frame buffer in pixel.
    __u32 h;                  ///< @details Height of image frame buffer in pixel.
    g2d_data_fmt format;      ///< @details Pixel format of image frame buffer.
    g2d_pixel_seq pixel_seq;  ///< @details Pixel sequence of image frame buffer.
} g2d_image;

typedef struct {
    g2d_fillrect_flags flag;
    g2d_image dst_image;
    g2d_rect dst_rect;

    __u32 color;  ///< @details Fill color.
    __u32 alpha;  ///< @details Plane alpha value.

} g2d_fillrect;

typedef struct {
    g2d_blt_flags flag;
    g2d_image src_image;
    g2d_rect src_rect;

    g2d_image dst_image;
    __s32 dst_x;  ///< @details Left top point coordinate x of dst rect.
    __s32 dst_y;  ///< @details Left top point coordinate y of dst rect.
    __u32 color;  ///< @details Colorkey color.
    __u32 alpha;  ///< @details Plane alpha value.

} g2d_blt;

typedef struct {
    g2d_blt_flags flag;
    g2d_image src_image;
    g2d_rect src_rect;

    g2d_image dst_image;
    g2d_rect dst_rect;

    __u32 color;  ///< @details Colorkey color.
    __u32 alpha;  ///< @details Plane alpha value.

} g2d_stretchblt;

typedef g2d_image surface_t;

typedef surface_t *HDC;

/**
 *  @anchor ancA20GraphSurfaces
 *  @name Работа с поверхностями
 *  @{
 */

/**
 *  @brief Инициализация \b 2D акселератора.
 *  @details Функция инициализирует аппаратный \b 2D
 *  акселератор для работы с поверхностями.
 *
 *  @return Всегда 0.
 */
int init_2D_engine ();

/**
 *  @brief Создать новую поверхность.
 *  @details Функция создает новую поверхность с указанными параметрами.
 *
 *  @param W Ширина в пикселях.
 *  @param H Высота в пикселях.
 *  @param F Формат цвета поверхности из перечисления \ref g2d_data_fmt.
 *  @return Указатель на созданную поверхность \ref surface_t.
 */
surface_t *newSurface (int W, int H, g2d_data_fmt F);


/**
 *  @brief Освободить массив данных поверхности.
 *  @details Функция освобождает память, выделенную под массив данных цвета
 *  при создании поверхности. Это нужно для подключения к поверхности массива
 *  памяти, полученных из других источников, например, из аппаратного декодера.
 *  Если в последующем нужно удалить саму поверхность (без освобождения памяти
 *  массива данных) следует делать просто через free().
 *  @warning После освобождения памяти следует сразу же подключить к поверхности
 *  новый массив данных, иначе попытка вывода поверхности на экран приведёт
 *  к ошибке.
 *  
 *  @param ps Указатель на поверхность. 
 */
void deleteSurfaceDataArray (surface_t *ps);

/**
 *  @brief Удалить поверхность.
 *  @details Функция удаляет из памяти указанную поверхность
 *  и все связанные с ней ресурсы, созданные с помощью newSurface().
 *  @warning Для удаления поверхностей с заранее освобожденным
 *  массивом данных (см. deleteSurfaceDataArray())
 *  следует воспользоваться обычным способом освобождения памяти free().
 * 
 *
 *  @param ps Указатель на поверхность.
 */
void deleteSurface (surface_t *ps);


/**
 *  @brief Смена поверхностей.
 *  @details Функция меняет местами видимую (\ref SCREEN) и
 *  конструируемую (\ref CONSTR) поверхности.
 *
 *  @return Всегда 0.
 */
int FlipScreenAndConstr ();

/**
 *  @brief Загрузить поверхность из \b BMP.
 *  @details Функция загружает поверхность данными из \b BMP -- файла.
 *  Поддерживаемые форматы файла: <b>RGB 24-бита, RGBX и RGBA 32-бита</b>.
 *  Форматы 16-бит и ниже не поддерживаются 
 *  @param fileName Имя файла на диске.
 *  @return Указатель на созданную поверхность \ref surface_t.
 */
surface_t *loadBMPSurface (char *fileName);

/**
 *  @brief Загрузить поверхность из \b JPG.
 *  @details Функция загружает поверхность данными из \b JPG - Файла.
 *  Поддерживаемый формат файла: <b>YCbCr 24-бита в baseline кодировке</b>.
 *  <b>Ограничения декодера:</b>
 *  - Изображение декодируется некорректно, если одна из его сторон нечётной длины.
 *  - Максимальный размер изображения 3840x2160.
 *  - Не поддерживается \b progressive формат.
 *  - Не поддерживаются файлы, имеющие не 3 компонента (Y, Cb, Cr).
 *    Это могут быть некоторые чёрно-белые изображения или изображения с другой цветовой моделью.
 *  - \b Важно! Для корректной работы функции требуется 
 *  перед вызовом включить \b VE (видеоэнкодер),
 *  а после — выключить командами ve_open() и ve_close().
 *  Файлы формата \b JPEG распознаются системой некорректно,
 *	чтобы открыть изображение, введите имя в виде: \b f_name~1.jpe 
 *  
 *  @param fileName Имя файла на диске.
 *  @return Указатель на созданную поверхность \ref surface_t.
 */
surface_t *loadJPEGSurface (char *fileName);

/**
 *  @brief Загрузить поверхность из \b PNG.
 *  @details Функция загружает поверхность данными из \b PNG - файла.
 *  Для работы функции понадобится подключение дополнительных 
 *  библиотек \b -l_png и \b  -l_z в \ref sectionMakefile "Makefile".
 *
 *  @param fileName Имя файла на диске.
 *  @return Указатель на созданную поверхность \ref surface_t.
 */
surface_t *loadPNGSurface (char *fileName);

/**
 *  @brief Загрузить поверхность из файла.
 *  @details Функция загружает поверхность \em сырыми данными из файла.
 *
 *  @param fileName Имя файла на диске.
 *  @param W Ширина в пикселях.
 *  @param H Высота в пикселях.
 *  @param F Формат поверхности из перечисления \ref g2d_data_fmt.
 *  @return Указатель на созданную поверхность \ref surface_t.
 */
surface_t *loadRawSurface (char *fileName, int W, int H, int F);

/**
 *  @brief Закрасить прямоугольную область на поверхности.
 *  @details Функция закрашивает на указанной поверхности прямоугольную
 *  область в указанном месте указанным цветом.
 *
 *  @param dst Указатель на поверхность, в которой требуется проводить действие.
 *  @param dstX,dstY Координаты левого верхнего угла прямоугольника на поверхности.
 *  @param Width,Height Ширина и высота закрашиваемой области.
 *  @param alpha Прозрачность заливки (255 --- не прозрачная, 0 --- полностью прозрачная).
 *  @param color Цвет заливки в формате, выбранном при создании поверхности.
 *  @param options Опции заливки из перечисления \ref g2d_fillrect_flags.
 *  @return \ref OK При нормальном завершении, или код ошибки.
 *  @see \ref subsubsectionGraphSurfaceBugRotate. \ref subsubsectionGraphSurfaceBugColorKeyDst.
 */
int fillRect (HDC dst, int dstX, int dstY, int Width, int Height,
              unsigned char alpha, unsigned int color, int options);

/**
 *  @brief Копирование прямоугольной области.
 *  @details Функция копирует прямоугольную область из одной поверхности в другую.
 *
 *  @param dst Указатель на поверхность -- получатель.
 *  @param dstX,dstY Координаты левого верхнего угла на поверхности -- получателе.
 *  @param Width,Height Ширина и высота копируемой области.
 *  @param src Указатель на поверхность -- источник.
 *  @param srcX,srcY Координаты левого верхнего угла на поверхности -- источнике.
 *  @param alpha Прозрачность копии.
 *  @param color Ключевой цвет в формате, выбранном при создании поверхности.
 *  @param options Опции --- комбинация из значений перечисления \ref g2d_blt_flags.
 *  @return \ref OK При нормальном завершении, или код ошибки.
 *  @see \ref subsubsectionGraphSurfaceBugRotate. \ref subsubsectionGraphSurfaceBugColorKeyDst.
 */
int bitBlt (HDC dst, int dstX, int dstY, int Width, int Height,
            HDC src, int srcX, int srcY,
            unsigned char alpha, unsigned int color, int options);

/**
 *  @brief Копирование прямоугольной области с \b альфа-каналом для видео-потока.
 *  @details Функция копирует прямоугольную область из одной поверхности в другую,
 *  используя при этом данные \b альфа-канала.
 *  @warning Это экспериментальная функция. Использовалась для вывода видео
 *  с прозрачностью на выставках.
 *
 *  @param dst Указатель на поверхность -- получатель.
 *  @param dstX,dstY Координаты левого верхнего угла на поверхности -- получателе.
 *  @param Width,Height Ширина и высота копируемой области.
 *  @param src Указатель на поверхность -- источник.
 *  @param srcX,srcY Координаты левого верхнего угла на поверхности -- источнике.
 *  @param alpha Прозрачность копии.
 *  @param color Ключевой цвет в формате, выбранном при создании поверхности.
 *  @param options Опции из перечисления \ref g2d_blt_flags.
 *  @param pAlpha Указатель на данные альфа-канала.
 *  @return \ref OK При нормальном завершении, или код ошибки.
 *  @see \ref subsubsectionGraphSurfaceBugRotate. \ref subsubsectionGraphSurfaceBugColorKeyDst.
 */
/// @private
int mixBlt (HDC dst, int dstX, int dstY, int Width, int Height,
            HDC src, int srcX, int srcY,
            unsigned char alpha, unsigned int color, int options, int pAlpha);

/**
 *  @brief Копирование прямоугольной области с масштабированием.
 *  @details Функция копирует прямоугольную область
 *  из одной поверхности в другую с изменением масштаба.
 *
 *  @param dst Указатель на поверхность -- получатель.
 *  @param dstX,dstY Координаты левого верхнего угла на поверхности -- получателе.
 *  @param dstW,dstH Размеры на поверхности -- получателе.
 *  @param src Указатель на поверхность -- источник.
 *  @param srcX,srcY Координаты левого верхнего угла на поверхности -- источнике.
 *  @param srcW,srcH Размеры на поверхности -- источнике.
 *  @param alpha Прозрачность копии.
 *  @param color Ключевой цвет в формате, выбранном при создании поверхности.
 *  @param options Опции из перечисления \ref g2d_blt_flags.
 *  @return \ref OK При нормальном завершении, или код ошибки.
 *  @see \ref subsubsectionGraphSurfaceBugRotate. \ref subsubsectionGraphSurfaceBugColorKeyDst.
 */
int stretchBlt (HDC dst, int dstX, int dstY, int dstW, int dstH,
                HDC src, int srcX, int srcY, int srcW, int srcH,
                unsigned char alpha, unsigned int color, int options);

/// @}

/**
 *  @name Указатели на поверхности
 *  @brief Видимая на экране поверхность и конструируемая поверхность также,
 *  как любая другая имеют заголовки типа \ref surface_t. Чтобы получить указатели на них,
 *  можно воспользоваться функциями данного раздела.
 *  @{
 */

#define SCREEN getScreenSurface ()  ///< Видимая поверхность. @details Короткая запись функции getScreenSurface().
#define CONSTR getConstrSurface ()  ///< Конструируемая поверхность. @details Короткая запись функции getConstrSurface().

#define ASCREEN getScreenAlpSurface ()  ///< Видимая поверхность с алфа-каналом. @details Короткая запись функции getScreenAlpSurface().
#define ACONSTR getConstrAlpSurface ()  ///< Конструируемая поверхность c альфа-каналом. @details Короткая запись функции getConstrAlpSurface().

/**
 *  @brief Видимая поверхность.
 *  @details Функция возвращает указатель на видимую на экране поверхность.
 *
 *  @return Указатель на видимую на экране поверхность \ref surface_t.
 */
surface_t *getScreenSurface (void);

/**
 *  @brief Конструируемая поверхность.
 *  @details Функция возвращает указатель на конструируемую поверхность.
 *
 *  @return Указатель на конструируемую поверхность \ref surface_t.
 */
surface_t *getConstrSurface (void);

/**
 *  @brief Видимая поверхность с алфа-каналом.
 *  @details Функция возвращает указатель на видимую на экране поверхность с алфа-каналом.
 *
 *  @return Указатель на видимую на экране поверхность \ref surface_t.
 */
surface_t *getScreenAlpSurface (void);

/**
 *  @brief Конструируемая поверхность c альфа-каналом.
 *  @details Функция возвращает указатель на конструируемую поверхность c альфа-каналом.
 *
 *  @return Указатель на конструируемую поверхность с альфа-каналом \ref surface_t.
 */
surface_t *getConstrAlpSurface (void);

/// @}

/// @private
void flushSurface (surface_t *ps);


// Overlay input format types YUV422

/**
 *  @anchor ancOverlayFormatTypes
 *  @name Входные форматы оверлея YUV420 / YUV422
 *  @{
 */

#define OT_YUV420_COMBINED      (0x00)
#define OT_YUV420_PLANAR        (0x01)
#define OT_YUV420_MB_COMBINED   (0x02)
#define OT_YUV420_MB_PLANAR     (0x03)
#define OT_YUV420_INTERLEAVED   (0x04)

#define OT_YUV422_COMBINED      (0x10)
#define OT_YUV422_PLANAR        (0x11)
#define OT_YUV422_MB_COMBINED   (0x12)
#define OT_YUV422_MB_PLANAR     (0x13)
#define OT_YUV422_INTERLEAVED   (0x14)

#define OT_INRGB888_PLANAR     (0x100)
/// @}


/**
 *  @name Входные форматы оверлея YUV420
 *  @brief Устаревшие макросы - оставлены для совместимости со старыми проектами.
 *  @{
 */
#define OT_UV_COMBINED      OT_YUV420_COMBINED
#define OT_PLANAR           OT_YUV420_PLANAR
#define OT_MB_UV_COMBINED   OT_YUV420_MB_COMBINED
#define OT_MB_PLANAR        OT_YUV420_MB_PLANAR
#define OT_MB_INTERLEAVED   OT_YUV420_INTERLEAVED
/// @}

/**
 *  @name Работа с оверлеями
 *  @{
 */

/**
 *  @brief Инициализация режима оверлея.
 *  @details Функция инициализирует режим аппаратного оверлея.
 *
 */
void OverlayInit (void);

/**
 *  @brief Открыть оверлей на экране.
 *  @details Функция открывает аппаратный оверлей на экране.
 *
 *  @param BaseAddr Адреса в памяти зоны оверлея.
 *  @param X,Y Координаты на экране левого верхнего угла оверлея.
 *  @param srcWidth,srcHeight Размеры оверлея в памяти в точках.
 *  @param dstWidth,dstHeight Размеры оверлея на экране в точках.
 *  @param OType Тип оверлея --- одно из значений, перечисленных
 *  в группе \ref ancOverlayFormatTypes "Входные форматы оверлея YUV422".
 */
void OverlayOpen (int BaseAddr[3], int X, int Y, int srcWidth, int srcHeight,
                  int dstWidth, int dstHeight, int OType);

/**
 *  @brief Открыть оверлей с аппаратным деинтерлейсингом источника.
 *  @details Функция открывает аппаратный оверлей на экране.
 *  Используется функция аппаратного деинтерлейсинга источника.
 *
 *  @param BaseAddr Адреса в памяти зоны оверлея.
 *  @param X,Y Координаты на экране левого верхнего угла оверлея.
 *  @param srcWidth,srcHeight Размеры оверлея в памяти в точках.
 *  @param dstWidth,dstHeight Размеры оверлея на экране в точках.
 *  @param OType Тип оверлея --- одно из значений, перечисленных
 *  в группе \ref ancOverlayFormatTypes "Входные форматы оверлея YUV422".
 */
void OverlayOpenDI (int BaseAddr[3], int X, int Y, int srcWidth, int srcHeight,
                  int dstWidth, int dstHeight, int OType);

/**
 *  @brief Смена области оверлея.
 *  @details Функция позволяет оперативно поменять адрес оверлейной области.
 *
 *  @param A Новые адреса в памяти зоны оверлея.
 */
void OverlaySetAddr (int A[3]);

/**
 *  @brief Смена приоритета области оверлея.
 *  @details Функция позволяет изменить приоритет оверлейной области.
 *  В зависимости от приоритета меняется порядок вывода областей памяти 
 *  на экран. Самый высокий (больший по значению) приоритет выводится поверх остальных.
 *  Функцию следует вызывать перед открытием области оверлей OverlayOpen().
 *
 *  @param P Приоритет области оверлея. Возможные значения от \b 0 (вывести под другими слоями)
 *  до \b 3 (вывести поверх всех -- значение по умолчанию).
 */
int setOverlayPriority (int P);

/**
 *  @brief Закрыть оверлей.
 *  @details Функция закрывает ранее открытый на экране оверлей.
 *
 */
void OverlayClose (void);


/// @}


/// @privatesection
void OverlaySetOffset (int off_x, int off_y);

int flipDisplay (void);

int initDisplay();

int sunxi_wait_retrace ();

int setGraphMode (int width, int height, int bpp);

/**
 *  @brief Техническая информация о настройке частот модуля VE.
 */
void a20DisplayClkReport ();



#ifdef __cplusplus
}
#endif
#endif  // ifndef VIDEOMODES_H_
