/**
    @addtogroup groupMultimedia
    @{
    @file avilib.h
    @brief Работа с AVI файлами.
    @details Библиотека для работы с файлами-контейнерами стандарта \b AVI (<em>Audio Video Interleave</em>).

    <b>Подключение</b>:
    @verbatim
    #include <multimedia/a20graph.h>
    #include <multimedia/avilib.h>
    #include <multimedia/mpeg4codec.h>
    @endverbatim
    \ref sectionMakefile "Makefile":
    @verbatim
    LIBRARIES += -l_a20graph -l_avi -l_mpeg4decode
    @endverbatim

    @see Общее описание работы с <b>AVI</b>-файлами в главе \ref sectionAvi.
    @}
 */
#ifndef AVILIB_H_
#define AVILIB_H_

#ifdef __cplusplus
extern "C" {
#endif

// Создание и покадровая запись AVI файла (XVID)

typedef void *AVI_HANDLE;  ///< Дескриптор <b>AVI</b>-файла.

/**
 *  @brief Установить формат записи.
 *  @details Функция устанавливает формат записи для видео.
 *
 *  @param T Тип записи: \b 0 -- MPEG4, \b 1 -- H.264.
 *  @return \ref OK при удачном завершении.
 *  @return \ref ERROR при ошибке.
 */
int setAVIType (int T);

/**
 *  @brief Создание пустого <b>AVI</b>-файла без звука.
 *  @details Функция создает пустой <b>AVI</b>-файл
 *  с заданными параметрами (для записи без звука).
 *
 *  @param name Имя файла, под которым он будет создан.
 *  @param width,height Размеры кадра.
 *  @return Дескриптор создаваемого <b>AVI</b>-файла \ref AVI_HANDLE.
 */
AVI_HANDLE newAVIFile (const char *name, int width, int height);

/**
 *  @brief Создание пустого <b>AVI</b>-файла со звуком.
 *  @details Функция создает пустой <b>AVI</b>-файл содержащий звуковые данные.
 *
 *  @param name Имя файла, под которым он будет создан.
 *  @param width,height Размеры кадра.
 *  @return Дескриптор создаваемого <b>AVI</b>-файла \ref AVI_HANDLE.
 */
AVI_HANDLE newAVIFileSnd (char *name, int width, int height);

/**
 *  @brief Запись фрейма в <b>AVI</b>-файл.
 *  @details Функция записывает фрейм в файл <b>AVI</b>.
 *
 *  @param handle Дескриптор файла.
 *  @param flags Флаги для кадра — ключевой/промежуточный(не ключевой).
 *  @param buffer Указатель на данные.
 *  @param size Размер данных в байтах.
 *  @return \ref OK при удачном завершении либо код ошибки.
 */
int writeAVIFrame (AVI_HANDLE handle, int flags, char *buffer, int size);

/**
 *  @brief Запись звукового фрагмента в <b>AVI</b>-файл.
 *  @details Функция записывает звуковой фрагмент в файл <b>AVI</b>,
 *  создавать файл нужно функцией newAVIFileSnd().
 *
 *  @param ah Дескриптор файла.
 *  @param buffer Указатель на данные.
 *  @param size Размер данных в байтах.
 *  @return \ref OK при удачном завершении либо код ошибки.
 */
int writeSNDFrame (AVI_HANDLE ah, char *buffer, int size);

/**
 *  @brief Корректировка заголовков, запись индексов и закрытие <b>AVI</b>-файла.
 *  @details Функция корректирует заголовок, записывает индексы и закрывает <b>AVI</b>-файл.
 *  Если <b>AVI</b>-файл был открыт на чтение, то просто закрывает его.
 *
 *  @param handle Дескриптор файла.
 *  @return \ref OK при удачном завершении либо код ошибки.
 */
int closeAVIFile (AVI_HANDLE handle);

// Чтение данных из AVI-файла (Splitter)

/**
 *  @brief Открытие файла <b>AVI</b> на чтение и получение размеров.
 *  @details Функция открывает на чтение имеющийся <b>AVI</b>-файл и получает размеры кадра.
 *
 *  @param name Имя файла, под которым он будет создан.
 *  @param width,height Указатели на переменные, в которых поместятся размеры кадра.
 *  @param frames Указатель на количество кадров в файле.
 *  @return Дескриптор открытого <b>AVI</b>-файла.
 */
AVI_HANDLE openAVIFile (const char *name, int *width, int *height, int *frames);

/**
 *  @anchor ancReadAVIFrameRet
 *  @name Значения, возвращаемые readAVIFrame()
 *  @{
 */
#define AVI_RETURN_ERROR (-2)  ///< @details Ошибка чтения или некорректный контейнер.
#define AVI_RETURN_EOF (-1)    ///< @details Конец файла.
/// @}

/* Чтение фрейма из AVI-файла
 * Возвращает размер считанного фрейма (положительное число или 0, 0 не является ошибкой) или один из дефайнов выше
 */

/**
 *  @brief Чтение фрейма из <b>AVI</b>-файла.
 *  @details Функция читает из <b>AVI</b>-файла очередной кадр в память.
 *
 *  @param handle Дескриптор.
 *  @param buf Указатель на буфер в памяти.
 *  @return Размер считанного буфера в байтах либо одно из значений, описанных \ref ancReadAVIFrameRet "макросами".
 */
int readAVIFrame (AVI_HANDLE handle, char *buf);

/**
 *  @brief Чтение звука из <b>AVI</b>-файла.
 *  @details Функция читает из <b>AVI</b>-файла очередной звуковой буфер в память.
 *
 *  @param handle Дескриптор.
 *  @param buf Указатель на буфер в памяти.
 *  @return Размер считанного буфера в байтах.
 */
int readAVIAudio (AVI_HANDLE handle, char *buf);

/**
 *  @brief Установить указатель в начало файла.
 *  @details Функция устанавливает указатель на первый кадр в <b>AVI</b>-файле.
 *
 *  @param handle Дескриптор.
 *  @return \ref OK при удачном завершении либо код ошибки.
 */
int seekToFirstVideoFrame (AVI_HANDLE handle);

#ifdef __cplusplus
}
#endif
#endif  // ifndef AVIWRITTER_H_
