/**
    @addtogroup groupMultimedia
    @{
    @file cedrus.h
    @brief Работа с кодером/декодером видео h.264 CEDRUS
    @details Библиотека аппаратного кодирования/декодирования видеопотоков в стандарте h.264.
    @see Общее описание работы с кодером/декодером видео в главе \ref sectionCedrus.
    @}

    @page pageMultimedia

    @section sectionCedrus Кодер/декодер видео h.264 CEDRUS
    @see Описание методов работы с \b CEDRUS в файле cedrus.h.

    В состав библиотек \b RTOS @osName входит библиотека аппаратного
    кодирования/декодирования видео потоков в стандарте \b h.264 --- \b CEDRUS.

    Библиотека позволяет осуществлять в реальном времени как
    кодирование видеоинформации, получаемой от камеры, так и декодирование с
    последующим выводом на экран монитора ранее закодированной информации. Это
    позволяет в частности передавать видеоинформацию по сравнительно
    низкоскоростным каналам передачи от системы-сервера, снабженного
    видеокамерой, системе-клиенту с целью вывода полученной информации на экран
    монитора, или записи на диск в виде файла. Такие системы находят широкое
    применение в системах видео наблюдения и системах управления беспилотными
    аппаратами.



 */

#ifndef CEDRUS_H_
#define CEDRUS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>

/**
 *  @brief Открыть кодер.
 *  @details Функция открывает систему аппаратного кодирования/декодирования.
 *
 *  @return \ref OK при успешном завершении или код ошибки.
 */
int ve_open (void);

/**
 *  @brief Закрыть кодер.
 *  @details Функция закрывает (деактивирует) систему
 *  аппаратного кодирования/декодирования.
 */
void ve_close (void);




/**
 *  @name Кодирование видео потока
 *  @{
 */



/**
 *  @brief Форматы видео.
 */
enum color_format { H264_FMT_NV12 = 0,
                    H264_FMT_NV16 = 1 };

/**
 *  @brief Задать формат кодирования.
 *  @details Функция устанавливает формат видео для кодирования (\b NV12 или \b NV16).
 *
 *  @param format Один из форматов перечисленных в \ref color_format.
 */
void h264_set_src_format (int format);

/**
 *  @brief Задать разрешение кодера.
 *  @details Функция подготавливает кодер для указанного разрешения.
 *
 *  @param width,height Размеры кадра для кодирования в пикселях.
 *  @param qp Качество сжатия видео (0 -- мин. 47 -- макс.)
 *  @param key_interval Период следования ключевых кадров (в кадрах).
 *  @return Указатель на входной буфер, в котором надо разместить исходное изображение.
 */
char *h264_encoder_init (int width, int height, int qp, int key_interval);

/**
 *  @brief Изменить качество сжатия видео.
 *  @details Функция изменяет текущее качество сжатия видео
 *  при кодировании видео.
 *
 *  @param qp Качество сжатия видео (1 -- мин. 47 -- макс.)
 *  @return \ref OK при удачном изменении качества сжатия,
 *  иначе \ref ERROR.
 */
int h264_encoder_set_qp (int qp);

/**
 *  @brief Кодировать один кадр.
 *  @details Функция кодирует один кадр исходного изображения.
 *
 *  @param bitstream Указатель на выходной поток кодера.
 *  @return Длина в байтах закодированного в поток видео.
 */
unsigned int h264_encode (char *bitstream);

/**
 *  @brief Определение ключевых кадров.
 *  @details Функция сообщает, был ли последний кодированный кадр ключевым.
 *
 *  @return \b 1 для ключевого кадра.
 *  @return Ненулевое значение для всех прочих кадров.
 */
bool h264_is_keyframe (void);

/**
 *  @brief Освободить ресурсы кодера.
 *  @details Функция освобождает занятые кодером ресурсы.
 *
 *  @return \ref OK при удачном завершении.
 *  @return \ref ERROR если кодер не был создан.
 */
int h264_encoder_free (void);


/// @}


/**
 *  @name Декодирование видео потока
 *  @{
 */

/**
 *  @brief Инициализация декодера.
 *  @details Функция инициализирует декодер.
 *
 *  @param width,height Оригинальные размеры выходного кадра в пикселях.
 *  @param wait_retrace Указатель на функцию ожидания обратного хода
 *  или \ref NULL, если не нужна.
 *  @return Указатель на экземпляр декодера.
 */
int h264_decoder_init (int width, int height, void (*wait_retrace) (void));

/**
 *  @brief Декодировать один кадр.
 *  @details Функция декодирует один кадр.
 *  Если задан указатель на выходное изображение \b output,
 *  то декодированный кадр будет целиком переведён из формата
 *  \b Tiled \b YUV в \b Planar.
 *
 *  @param dd Указатель на экземпляр декодера.
 *  @param bitstream Указатель на входной кодированный поток.
 *  @param len Длина входного потока.
 *  @param output Указатель на выходное изображение, либо \b 0.
 *  @return \ref OK при удачном завершении, иначе \ref ERROR.
 */
int h264_decode (int dd, char *bitstream, unsigned int len, char *output);

/**
 *  @brief Декодировать часть кадра.
 *  @details Функция декодирует один кадр и переводит часть декодированного
 *  кадра из формата \b Tiled \b YUV в формат \b Planar часть кадра, заданную координатами.
 *  Координаты кадра должны быть кратны размеру \em плитки - 32 пикселя.
 *  Функция является обёрткой для h264_decode() и имеет смысл только если
 *  задан указатель на выходное изображение \b output.
 *
 *  @param dd Указатель на экземпляр декодера.
 *  @param bitstream Указатель на входной кодированный поток.
 *  @param len Длина входного потока.
 *  @param output Указатель на выходное изображение, либо \b 0.
 *  @param x,y Координаты верхнего левого угла декодируемого кадра.
 *  @param w,h Размеры декодируемого кадра.
 *  @return \ref OK при удачном завершении, иначе \ref ERROR.
 */
int h264_decode_part (int dd, char *bitstream, unsigned int len, char *output, int x, int y, int w, int h);

/**
 *  @brief Освободить ресурсы декодера.
 *  @details Функция освобождает ресурсы, занятые декодером.
 *
 *  @param dd Указатель на экземпляр декодера.
 *  @return \ref OK при удачном завершении.
 *  @return \ref ERROR при ошибке.
 */
int h264_decoder_free (int dd);

/**
 *  @brief Кадр декодера.
 *  @details Функция возвращает указатель на декодированный кадр.
 *
 *  @param dd Указатель на экземпляр декодера.
 *  @return char* Указатель на декодированный кадр.
 */
char *h264_get_out (int dd);

/// @}


/**
 *  @name Настройки декодера.
 *  @brief Эти функции должны быть вызваны до инициализации декодера.
 *  @{
 */
/**
 *  @brief Взаимодействие 2-х ядер.
 *  @details Функция указывает (в двухядерном режиме) ждать ли первому
 *  ядру завершения пересчета выходного буфера вторым ядром.
 *
 *  @param no_wait \b 1 --- не ждать, \b 0 --- ждать.
 */
void h264_decoder_set_wm (int no_wait);

/**
 *  @brief Использовать второе ядро для пересчета выходного буфера.
 *  @details Функция устанавливает двухядерный режим обработки декодера.
 *
 *  @param use \b 1 --- использовать 2-е ядро, \b 0 --- не использовать.
 */
void h264_decoder_set_mk (int use);

/**
 *  @brief Задать качество декодирования.
 *  @details Функция устанавливает начальное значение параметра качества.
 *
 *  @param qp Начальный параметр качества.
 *  @return \ref OK при удачном завершении.
 *  @return \ref ERROR при ошибке.
 */
int h264_decoder_set_qp (int qp);
/// @}


/// @privatesection

// Подсчёт статистики аппаратных перезагрузок.
// Перезапуск аппаратной части декодера производится
// при получении заголовков кадров с сомнительным содержанием.
// При некоторых обстоятельствах такие заголовки приводят к
// сбою в работе декодера (выводится графика в стиле абстракционизма).
unsigned int h264_decoder_reset_count ();

#ifdef __cplusplus
}
#endif
#endif  // ifndef CEDRUS_H_
