/**
    @addtogroup groupCsi
    @{
    @file csi.h
    @brief Работа с интерфейсом CSI (Camera Sensor Interface)
    @details Файл содержит методы работы с аппаратным интерфейсом подключения цифровых камер.

    <b>Подключение</b>: @verbatim #include <multimedia/csi.h> @endverbatim
    \ref sectionMakefile "Makefile": @verbatim LIBRARIES += -l_csi @endverbatim

    @see Общее описание интерфейса в главе \ref pageCsi.
    @}

    @page pageCsi Поддержка CSI (Camera Sensor Interface)

    Для подключения методов работы с \b CSI к проекту необходимо добавить
    соответствующую библиотеку в \ref sectionMakefile "Makefile":
    @verbatim
    LIBRARIES += -l_csi
    @endverbatim

    @see Описание методов работы с аппаратным интерфейсом подключения цифровых камер
    в файле csi.h.



    @section sectionCsiCamera Работа с цифровыми видеокамерами

    В @osName включена поддержка драйвера \b CSI (Camera Sensor Interface),
    обеспечивающего высокоскоростной параллельный обмен с цифровыми источниками
    видеосигнала, работающими по стандартам:
    - CCIR656
    - BT1120
    - 8 bit raw
    - 16 bit YUV422
    - time-multiplexied ITU-R BT656 с поддержкой двойной буферизации.

    Процессор \b Allwinner \b A20 имеет два независимых порта \b CSI. Драйвер
    может работать одновременно с двумя входами. При этом \b CSI0 поддерживает
    разрешение до \b 1080p@30FPS, а \b CSI1 разрешение \b 720p@30FPS.

    В библиотеке поддержки \b CSI (csi.h) содержатся все необходимые
    функции для работы с видеоканалом. Пример запуска канала \ref CSI_0
    приведён в примере ниже:

    @code
    // Создание устройства 
    void *dev = csiDevCreate (CSI_0, 1920, 1080);
    // Настройка устройства 
    csiSetMode (dev, csiInFmt_YUV422, csiOutFmt_YCbCr_422_frp_UVc);
    csiSetPhase (dev, csiActiveHigh, csiActiveHigh, csiRisingEdge);
    // Запуск захвата видео
    csiRun (dev);
    while (anyCondition) {
        // Ожидание захвата кадра
        csiWaitFrame (dev);
        // Получение указателей на захваченные данные
        void *lBuf = csiGetLuma (dev);
        void *chBuf = csiGetChroma (dev);
        ...
    }
    // Остановка захвата, освобождение ресурсов
    csiStop (dev);
    @endcode

 */

#ifndef SUNXI_CSI_H_
#define SUNXI_CSI_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>


/**
 *  @anchor ancCsiChannels
 *  @name Номер канала CSI
 *  @{
 */
#define CSI_0 0 ///< Канал CSI 0.
#define CSI_1 1 ///< Канал CSI 1.
/// @}



/**
 *  @name Создание устройства
 *  @{
 */

/**
 *  @brief Создание устройства CSI для выбранного канала.
 *  @details При создании устройства создаются структуры
 *  управления аппаратным драйвером и выделяется память
 *  под входные данные.
 *  
 *  @param chNum Номер канал. Рекомендуется брать значение 
 *  из группы \ref ancCsiChannels "макросов".
 *  @param width Ширина принимаемого кадра для создаваемого устройства в пикселях.
 *  @param height Высота принимаемого кадра для создаваемого устройства в пикселях.
 *  @return Идентификатор устройства - указатель на 
 *  выделенную под устройство память.
 */
void *csiDevCreate (unsigned int chNum, unsigned int width, unsigned int height);

/**
 *  @brief Удаление устройства CSI из системы.
 *  @details Функция освобождает выделенную под устройство память.
 *  Перед удалением следует остановить устройство с помощью csiStop().
 *  
 *  
 *  @param dev Идентификатор устройства, полученный при создании.
 */
void csiDevDelete (void *dev);

/// @}



/**
 *  @name Настройка
 *  @brief Устройство CSI создаётся с помощью csiDevCreate(). 
 *  Далее можно настроить созданное устройство с помощью функций 
 *  из этого раздела.
 *  @{
 */

/**
 *  @brief Формат входных данных.
 *  @details Формат входных данных модуля CSI.
 *  
 */
typedef enum {
    csiInFmt_RAW            = 0b000,   ///< RAW stream.
    csiInFmt_CCIR656_1ch    = 0b010,   ///< CCIR656 (1 канал).
    csiInFmt_YUV422         = 0b011,   ///< YUV422.
    csiInFmt_YUV422_16bit   = 0b100,   ///< YUV422 шина данных 16-бит.
    csiInFmt_CCIR656_2ch    = 0b101,   ///< CCIR656 (2 канала).
    csiInFmt_CCIR656_4ch    = 0b111    ///< CCIR656 (4 канала).
} eCsiInputFormat;

/**
 *  @brief Формат выходных данных.
 *  @details Формат выходных данных зависит от выбранного входного формата.
 *  Описание значение взято из документации.
 *  
 */
typedef enum {

// When the input format is set RAW stream
    csiOutFmt_PathThrough = 0b0000,    ///< Pass-through for RAW.

// When the input format is set CCIR656 interface
    csiOutFmt_YCbCr_422_fldp = 0b0000,    ///< For CCIR656 input. Field planar YCbCr 422.
    csiOutFmt_YCbCr_420_fldp = 0b0001,    ///< For CCIR656 input. Field planar YCbCr 420.
    csiOutFmt_YCbCr_420_frp  = 0b0010,    ///< For CCIR656 input. Frame planar YCbCr 420.
    csiOutFmt_YCbCr_422_frp  = 0b0011,    ///< For CCIR656 input. Frame planar YCbCr 422.
    csiOutFmt_YCbCr_422_fldp_UVc  = 0b0100,     ///< For CCIR656 input. Field planar YCbCr 422 UV combined.
    csiOutFmt_YCbCr_420_fldp_UVc  = 0b0101,     ///< For CCIR656 input. Field planar YCbCr 420 UV combined.
    csiOutFmt_YCbCr_420_frp_UVc   = 0b0110,     ///< For CCIR656 input. Frame planar YCbCr 420 UV combined.
    csiOutFmt_YCbCr_422_frp_UVc   = 0b0111,     ///< For CCIR656 input. Frame planar YCbCr 422 UV combined.
    csiOutFmt_YCbCr_422_inter = 0b1111,         ///< For CCIR656 input. Interlaced interleaved YCbCr422. In this mode, capturing
                                                /// interlaced input and output the interlaced fields from individual
                                                /// ports. Field 1 data will be wrote to FIFO0 output buffer and field 2
                                                /// data will be wrote to FIFO1 output buffer.
    csiOutFmt_YCbCr_422_fldt = 0b1000,    ///< For CCIR656 input. Field tiled YCbCr 422
    csiOutFmt_YCbCr_420_fldt = 0b1001,    ///< For CCIR656 input. Field tiled YCbCr 420
    csiOutFmt_YCbCr_420_frt  = 0b1010,    ///< For CCIR656 input. Frame tiled YCbCr 420
    csiOutFmt_YCbCr_422_frt  = 0b1011,    ///< For CCIR656 input. Frame tiled YCbCr 422

// When the input format is set YUV422
    csiOutFmt_YUV_422_p  = 0b0000,    ///< For YUV422 input. Planar YUV 422.
    csiOutFmt_YUV_420_p  = 0b0001,    ///< For YUV422 input. Planar YUV 420.
    csiOutFmt_YUV_422_pc = 0b0100,    ///< For YUV422 input. Planar YUV 422 UV combined.
    csiOutFmt_YUV_420_pc = 0b0101,    ///< For YUV422 input. Planar YUV 420 UV combined.
    csiOutFmt_YUV_422_t  = 0b1000,    ///< For YUV422 input. Tiled YUV 422.
    csiOutFmt_YUV_420_t  = 0b1001     ///< For YUV422 input. Tiled YUV 420.

} eCsiOutputFormat;



/**
 *  @brief Выбор активного уровня сигнала.
 */
typedef enum {
    csiActiveLow  = 0,  ///< Активный уровень сигнала - низкий.
    csiActiveHigh = 1   ///< Активный уровень сигнала - высокий.
} eCsiPolarity;

/**
 *  @brief Выбор активного фронта сигнала.
 */
typedef enum {
    csiFallingEdge  = 0,    ///< Активный фронт сигнала - убывающий.
    csiRisingEdge   = 1     ///< Активный фронт сигнала - возрастающий.
} eCsiClockEdge;


/**
 *  @brief Последовательность компонент входных данных.
 */
typedef enum {
    
    csiSequence_YUYV=0, ///< Последовательность компонент входных данных YUYV (только для входного формата YUV422).
    csiSequence_YVYU,   ///< Последовательность компонент входных данных YVYU (только для входного формата YUV422).
    csiSequence_UYVY,   ///< Последовательность компонент входных данных UYVY (только для входного формата YUV422).
    csiSequence_VYUY,   ///< Последовательность компонент входных данных VYUY (только для входного формата YUV422).

    sciSequence_RGRG=0, ///< Входная последовательность RGRG (только для формата Byer).
    sciSequence_GRGR,   ///< Входная последовательность GRGR (только для формата Byer).
    sciSequence_BGBG,   ///< Входная последовательность BGBG (только для формата Byer).
    sciSequence_GBGB    ///< Входная последовательность GBGB (только для формата Byer).
} eSciInputSequence;


/**
 *  @brief Выбор захватываемых строк.
 *  @details Выбор строк актуален только для входного формата CCIR656.
 */
typedef enum {
    csiField_odd,       ///< Только нечётные строки.
    csiField_even,      ///< Только чётные строки.
    csiField_either     ///< Как чётные так и нечётные строки.
} eCsiFieldSelection;


/**
 *  @brief Установка последовательности компонент.
 *  @details Установка последовательности компонент входных данных.
 *  
 *  @param dev Идентификатор устройства, полученный при создании.
 *  @param seq Задаваемая последовательность.
 */
void csiSetSeq (void *dev, eSciInputSequence seq);

/**
 *  @brief Выбор полярности сигнала.
 *  @details Функция задаёт полярность сигналов синхронизации и активный фронт
 *  сигнала CLK.
 *  
 *  @param dev Идентификатор устройства, полученный при создании.
 *  @param vRef Полярность сигнала кадровой синхронизации.
 *  @param hRef Полярность сигнала строчной синхронизации.
 *  @param clock Активный фронт сигнала CLK.
 */
void csiSetPhase (void *dev, eCsiPolarity vRef, eCsiPolarity hRef, eCsiClockEdge clock);

// Установка форматов входа/выхода

/**
 *  @brief Выбор форматов входного и выходного сигнала.
 *  @details Функция позволяет указать формат входного сигнала 
 *  и выбрать формат выходного. Преобразование осуществляется
 *  аппаратно.
 *  
 *  @param dev Идентификатор устройства, полученный при создании.
 *  @param inputFormat Формат входного сигнала.
 *  @param outputFormat Формат выходного сигнала.
 */
void csiSetMode (void *dev, eCsiInputFormat inputFormat, eCsiOutputFormat outputFormat);
/// @}



/**
 *  @name Запуск драйвера и получение данных
 *  @{
 */

/**
 *  @brief Запуск выбранного устройства.
 *  @details Запуск драйвера и начало получения данных от аппаратной части.
 *  Внутри драйвера ведётся двойная буферизация для каждого устройства.
 *  
 *  @param dev Идентификатор устройства, полученный при создании.
 *  @return \ref OK при успешном запуске, иначе \ref ERROR.
 */
STATUS csiRun (void *dev);

/**
 *  @brief Остановка выбранного устройства.
 *  @details Остановка драйвера для выбранного устройства.
 *  
 *  @param dev Идентификатор устройства, полученный при создании.
 */
void csiStop (void *dev);

/**
 *  @brief Сброс ожидания выбранного устройства.
 *  @details Функция сбрасывает флаг ожидания кадра выбранного устройства.
 *  При этом задача ожидающая очередного кадра на csiWaitFrame() получит управление.
 *  
 *  
 *  @param dev Идентификатор устройства, полученный при создании.
 */
void csiFlush (void *dev);



/**
 *  @brief Ожидание приёма кадра.
 *  @details Функция ожидает приёма следующего кадра от выбранного устройства.
 *  Во время ожидания другие задачи не блокируются.
 *  Выбранной устройство должно быть запущено с помощью csiRun().
 *  
 *  @param dev Идентификатор устройства, полученный при создании.
 *  @return Всегда \ref OK.
 */
int csiWaitFrame (void *dev);

/**
 *  @brief Получение указателя на массив яркостного сигнала.
 *  @details Функция возвращает указатель на на полученный массив яркостного сигнала (\b Y').
 *  
 *  @param dev Идентификатор устройства, полученный при создании.
 *  @return указатель на массив \b Y'.
 */
void *csiGetLuma (void *dev);

/**
 *  @brief Получение указателя на массив цветоразностного сигнала.
 *  @details Функция возвращает указатель на на полученный массив цветоразностного сигнала (\b CbCr).
 *  
 *  @param dev Идентификатор устройства, полученный при создании.
 *  @return указатель на массив \b CbCr.
 */
void *csiGetChroma (void *dev);


/// @}











#ifdef __cplusplus
}
#endif
#endif  // ifndef SUNXI_CSI_H_
