/**
    @addtogroup groupMultimedia
    @{
    @file de2.h
    @brief Allwinner DE2
    @details Работа с аппаратным модулем <b>"Display Enginne 2"</b> --- 
    вывод графики на дисплей \b LCD.

    <b>Подключение</b>: @verbatim #include <multimedia/de2.h> @endverbatim
    \ref sectionMakefile "Makefile": @verbatim LIBRARIES += -l_de2 @endverbatim
    
    @see Общее описание работы с графической подсистемой в главе \ref sectionA20graph.

    История \b версий библиотеки:
    - \b 1.2 --- Полная настройка аппаратного модуля с использованием
        одного канала \em Overlay. 

    @}
 */


#ifndef _DE2_H_
#define _DE2_H_

#ifdef __cplusplus
extern "C" {
#endif

/**
 *  @brief Пресеты параметров LCD дисплеев
 */
typedef enum {
    screenType_Lcd_480x272,     ///< @details Описание стандартного LCD дисплея 480x272
    screenType_Lcd_800x480,     ///< @details Описание стандартного LCD дисплея 800x480
    screenType_TM043NBH02,      ///< @details LCD дисплей \b TIANMA 4,3" 480x272
    screenType_TM070RxH10,      ///< @details LCD дисплей \b TIANMA 7" 800x480
} eScreenDeviceType;

/**
 *  @brief Формат данных \em Overlay.
 *  @details Поддерживаемые аппаратным модулем форматы цветопередачи.
 *  
 */
typedef enum {
    ovDataFormat_ARGB_8888  = 0x00,
    ovDataFormat_ABGR_8888  = 0x01,
    ovDataFormat_RGBA_8888  = 0x02,
    ovDataFormat_BGRA_8888  = 0x03,
    ovDataFormat_XRGB_8888  = 0x04,
    ovDataFormat_XBGR_8888  = 0x05,
    ovDataFormat_RGBX_8888  = 0x06,
    ovDataFormat_BGRX_8888  = 0x07,
    ovDataFormat_RGB_888    = 0x08,
    ovDataFormat_BGR_888    = 0x09,
    ovDataFormat_RGB_565    = 0x0A,
    ovDataFormat_BGR_565    = 0x0B,
    ovDataFormat_ARGB_4444  = 0x0C,
    ovDataFormat_ABGR_4444  = 0x0D,
    ovDataFormat_RGBA_4444  = 0x0E,
    ovDataFormat_BGRA_4444  = 0x0F,
    ovDataFormat_ARGB_1555  = 0x10,
    ovDataFormat_ABGR_1555  = 0x11,
    ovDataFormat_RGBA_5551  = 0x12,
    ovDataFormat_BGRA_5551  = 0x13
} eOverlayDataFormat;

/**
 *  @brief Структура настройки подключения дисплея \b LCD.
 *  @details Параметры для конкретного дисплея следует брать
 *  из документации на этот дисплей.
 */
typedef struct {
	int xres;           ///< @details Количество отображаемых точек по вертикали.
	int yres;           ///< @details Количество отображаемых точек по горизонтали.
    int bpp;            ///< @details Количество бит на пиксель (обычно - 32).
	int pixclock_khz;	///< @details Частота pixel clock в КГц (Влияет на fps, некоторые мониторы можно разгонять выше заявленной).
	int left_margin;    ///< Horizontal back porch  @details Отступ слева - интервал между сигналом строчной синхронизации и началом видимого сигнала (Horizontal back porch, задаётся в пикселях).
	int right_margin;	///< Horizontal front porch @details Отступ справа - интервал после окончания видимого сигнала до следующего сигнала строчной синхронизации (Horizontal front porch, задаётся в пикселях).
	int upper_margin;	///< Vertical back porch    @details Отступ сверху - интервал от сигнала вертикальной синхронизации до начала вывода нового кадра (Vertical back porch, задаётся в количестве строк).
	int lower_margin;   ///< Vertical front porch   @details Отступ снизу - интервал после окончания вывода кадра до сигнала вертикальной синхронизации (Vertical front porch, задаётся в количестве строк).
	int hsync_len;		///< @details Длительность сигнала строчной синхронизации.
	int vsync_len;		///< @details Длительность сигнала вертикальной синхронизации.
	int sync;		    ///< @details Флаги синхронизации: 1 - строчная синхронизация; 2 - вертикальная синхронизация. Обычно оба флага должны быть установлены (значение переменной равно 3).
	int vmode;		    ///< @details Флаги видео режимов: 1 - чересстрочная развёртка. Для LCD дисплеев - значение 0.
} tScreenDeviceMode;



/**
 *  @name Инициализация и управление аппаратным модулем
 *  @{
 */

/**
 *  @brief Инициализация модуля.
 *  @details Подготовка памяти экранной области, инициализация переменных.
 *  @param mode Параметры дисплея (можно воспользоваться de2GetDefaultScreenMode()
 *  для получения значений по умолчанию).
 *  @param format Формат данных изображения, используемый в 
 *  аппаратном модуле \b Display-Engine.
 * 
 *  @return Указатель на область видеопамяти конструируемой поверхности. 
 */
int *de2Init (const tScreenDeviceMode *mode, eOverlayDataFormat format);

/**
 *  @brief Инициализация структуры описания дисплея.
 *  @details Функция заполняет структуру описания дисплея
 *  значениями по умолчанию --- используются настройки, 
 *  подходящие для большинства дисплеев. Заполненную структуру
 *  следует использовать при инициализации модуля de2Init().
 *  При необходимости некоторые параметры можно изменить
 *  до инициализации модуля.
 *  
 *  @param mode Заполняемая структура описания дисплея.
 *  @param screen Выбранный тип дисплея.
 */
void de2GetDefaultScreenMode (tScreenDeviceMode *mode,
                              eScreenDeviceType screen);

/**
 *  @brief Смена поверхностей.
 *  @details Функция меняет местами видимую (\ref SCREEN) и
 *  конструируемую (\ref CONSTR) поверхности.
 *
 *  @return Указатель на область видеопамяти конструируемой поверхности. 
 *  Изменяется каждый раз при смене поверхностей.
 */
int *de2FlipScreenAndConstr ();

/**
 *  @brief Ожидание обратного хода луча.
 *  @details Функция ожидает начала обратного хода луча
 *  на мониторе для избежания появления строба.
 *
 *  @return \ref OK, либо код ошибки, возвращаемый semTake().
 */
int de2WaitVerticalRetrace ();


/**
 *  @brief Управление подсветкой дисплея.
 *  @details Для управления подсветкой в \ref sectionArch
 *  "Конфигурации проекта" должен быть указан используемый 
 *  для этого канал ШИМ. Настройка канала ШИМ производится
 *  в функции инициализации de2Init(). Управление подсветкой
 *  возможно только после успешной инициализации канала.
 *  
 *  @param brightness Значение яркости от \b 0 до \b 100.
 */
void de2SetBacklight (unsigned int brightness);

/// @}

/// @privatesection
/**
 *  @brief Отладка прерывания.
 *  @details Метод позволяет указать функцию отладки прерывания.
 *  Функция отладки должна быть максимально короткой -- инкремент счётчика
 *  или изменение состояния GPIO. Пример использования:
 *  
 *  @code
 *  void handler(void) { int_counter++; }
 *  de2_set_handler_debug (&handler); 
 *  @endcode
 *  
 *  @param handler Функция отладки.
 *  @private
 */
void de2_set_handler_debug (void (*handler)(void));

#ifdef __cplusplus
}
#endif
#endif // ifndef _SOFTGRAPH_H_
