/**
    @addtogroup groupMultimedia
    @{
    @file mpeg4codec.h
    @brief Программно-аппаратный декодер видео файлов формата MP4.
    Использует аппаратный видео-енкодер процессора и препроцессор \b NEON.
    @}
 */

/*
    Авторы:
    - Соколов Сергей Валерьевич
    - Ларюшин Александр Андреевич 
  	    364961@mail.ru

 */

#ifndef _MPEG4DECODER
#define _MPEG4DECODER

#include <multex.h>
#include <stdio.h>
#include <memlib.h>

/**
 *  @brief Запуск декодера MP4.
 *  @details Инициализация переменных декодера.
 *  
 *  @param wWidth,wHeight Размеры кадра в пикселях.
 *  @return Дескриптор созданного декодера.
 */
int mpeg4InitDecoder (int wWidth, int wHeight);

/**
 *  @brief Указатель на входной буфер декодера.
 *  @details Функция возвращает указатель на входной
 *  буфер данных декодера. Во входной буфер будут помещаться
 *  кадры для декодирования.
 *  
 *  @param dd Дескриптор декодера, 
 *  полученный при создании mpeg4InitDecoder().
 *  @return Указатель на буфер памяти.
 */
unsigned char *getMpeg4InptFrameBuff (int dd);

/**
 *  @brief Указатель на выходной буфер декодера.
 *  @details Функция возвращает указатель на выходной
 *  буфер данных декодера. В выходном буфере содержатся
 *  готовые кадры после процесса декодирования. 
 *  Выходной буфер может использоваться для вывода
 *  в \b overlay в режиме \b tiled.
 *  
 *  @param dd Дескриптор декодера, 
 *  полученный при создании mpeg4InitDecoder().
 *  @return Указатель на буфер памяти.
 */
int getMpeg4OutFrame (int dd);

/**
 *  @brief Декодировать блок данных (обычно кадр).
 *  @details Функция декодирует одина кадр,
 *  расположенный во входном буфере и кладёт его
 *  в выходной. Дополнительно может быть сделано 
 *  преобразование в nontiled формат - результат
 *  будет помещён в \b pOutData.
 *  
 *  @param dd Дескриптор декодера, 
 *  полученный при создании mpeg4InitDecoder().
 *  @param pOutData Буфер для преобразования выходного буфера в \b nontiled
 *  формат, пригодный для вывода на \b 2D-поверхность
 *  с помощью \ref ancA20GraphSurfaces "методов" работы с поверхностями.
 *  @param len Размер данных во входном буфере в байтах.
 *  @return \ref OK при удачном завершении декодирования, иначе \ref ERROR.
 */
int mpeg4DecodeBlock (int dd, unsigned char* pOutData, int len);

/**
 *  @brief Освобождение ресурсов, выделенных для работы с кадром.
 *  @details Функция освобождает память, выделенную для работы
 *  с текущим кадром.
 *  
 *  @param dd Дескриптор декодера, 
 *  полученный при создании mpeg4InitDecoder().
 */
void freeMpeg4FrameMem (int dd);

#endif

