/**
    @addtogroup groupMultimedia
    @{
    @file softgraph.h
    @brief Аппаратная реализация работы с поверхностями.
    @details Аппаратная реализация работы с поверхностями для процессоров,
    не имеющих аппаратного модуля работы с 2D-графикой.

    <b>Подключение</b>: @verbatim #include <multimedia/softgraph.h> @endverbatim
    \ref sectionMakefile "Makefile": @verbatim LIBRARIES += -l_softgraph -l_png -l_z @endverbatim
    
    @see Общее описание работы с графической подсистемой в главе \ref sectionA20graph.
    @}
 */


#ifndef _SOFTGRAPH_H_
#define _SOFTGRAPH_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>



/**
 *  @brief Описание параметров дисплея.
 */
typedef struct {
    int width;                 ///< Ширина экрана в пикселях.
    int height;                ///< Высота экрана в пикселях.
    int bpp;                   ///< Количество бит на пиксель.
} sDisplayInfo;

/**
 * @brief Описание параметров поверхности.
 */
typedef struct tagSURFACE {
    unsigned int sfBPP;          // Байт на точку.
    unsigned int sfWidth;        // Ширина в точках.
    unsigned int sfHeight;       // Высота в точках.
    unsigned int sfX, sfY;       // Координаты карандаша.
    void *sfData;                // Указатель на массив данных (изображение).
} SURFACE;

// Указатель на поверхность SURFACE
typedef SURFACE *HDCp;

/**
 *  @name Инициализация модуля
 *  @{
 */

/**
 * @brief Инициализация графической поверхности.
 * 
 * @param info Параметры дисплея .
 * @param constr Указатель на конструируемую поверхность.
 */
void softGraphInit (const sDisplayInfo *info, void *constr);

/**
 * @brief Получить указатель на конструируемую поверхность.
 * 
 * @return Указатель на поверхность \ref SURFACE.
 */
HDCp softGraphConstr ();

/**
 * @brief Изменить изображение конструируемой поверхности.
 * 
 * @param constr Указатель на новое изображение.
 */
void softGraphConstrUpdate (void *constr);

/// @private
const sDisplayInfo *displayInfo ();

/// @}


/**
 *  @anchor ancSoftGraphSurfaces
 *  @name Работа с поверхностями
 *  @{
 */

/**
 * @brief Загрузить поверхность из \b BMP.
 * @details Функция загружает поверхность данными из \b BMP - Файла. \n
 * Поддерживаемые форматы файла: <b>RGB 24-бита, RGBX и RGBA 32-бита</b>.
 * 
 * \b Важно! Если формат файла не \b RGBA,
 * то в поверхность записывается альфа-канал со стандартным значением \b 0xff. \n
 * 
 * Форматы 16-бит и ниже не поддерживаются. \n
 * Вызывает emptySurface(), если не удалось декодировать файл.
 *
 * @param fileName Имя файла на диске.
 * @return Указатель на созданную поверхность \ref SURFACE.
 */
HDCp loadFromBitmap (char *fileName);   

/**
 * @brief Загрузить поверхность из \b JPG.
 * @details Функция загружает поверхность данными из \b JPG - Файла.
 * 
 * \b Важно! В поверхность также записывается альфа-канал со стандартным значением \b 0xff.
 * 
 * Файлы формата \b JPEG распознаются системой некорректно,
 * чтобы открыть изображение, введите имя в виде: \b f_name.jpe \n
 * Вызывает emptySurface(), если не удалось декодировать файл.
 * 
 * @param fileName Имя файла на диске.
 * @return Указатель на созданную поверхность \ref SURFACE.
 */
HDCp loadFromJPG (char *fileName);

/**
 * @brief Загрузить поверхность из \b PNG.
 * @details Функция загружает поверхность данными из \b PNG - Файла. \n
 * Поддерживаемый формат: <b>8-bit RGBA</b>.
 * 
 * Вызывает emptySurface(), если не удалось декодировать файл.
 * 
 * @param name Имя файла на диске.
 * @return Указатель на созданную поверхность \ref SURFACE.
 */
HDCp loadFromPNG (char *name);           

/**
 * @brief Создать поверхность со стандартными параметрами.
 * 
 * @param XRes Ширина поверхности.
 * @param YRes Высота поверхности.
 * @return указатель на новую поверхность \ref SURFACE.
 */
HDCp createSHdr (int XRes, int YRes);

/**
 * @brief Создание первичной (конструируемой) поверхности.
 * 
 * @return Указатель на поверхность \ref SURFACE.
 */
HDCp createScreenSurface (void);

/**
 * @brief Создать пустую поверхность.
 * @details Функция создаёт поверхность с указанными сторонами
 * и выделяет память для изображения.
 * 
 * @param X Ширина поверхности.
 * @param Y Высота поверхности.
 * @return Указатель на созданную поверхность \ref SURFACE.
 */
HDCp createSurface (int X, int Y);                       

/**
 * @brief Создать поверхность--заглушку.
 * @details Функция создаёт поверхность с размерами 100x100 
 * и заполняет чёрным цветом.
 * 
 * @return Указатель на созданную поверхность \ref SURFACE. 
 */
HDCp emptySurface ();

/**
 * @brief Удалить поверхность.
 * 
 * @param sf Указатель на поверхность \ref SURFACE.
 */
void freeSurface (HDCp sf);                             

/**
 * @brief Очистить поверхность.
 * @details Функция заполняет поверхность сплошным цветом. \n
 * Цвет соответствует значению \b sfBrushColor в структуре \ref SURFACE.
 * 
 * @param psf Указатель на поверхность \ref SURFACE.
 */
void clearSurface (HDCp psf);                            

/**
 * @brief Залить поверхность цветом.
 * @details Функция заполняет поверхность указанным цветом.
 * 
 * @param psf Указатель на поверхность \ref SURFACE.
 * @param color Цвет заливки.
 */
void fillSurface (HDCp psf, int color);

/**
 * @brief Создание клона с копией содержимого.
 * 
 * @param SF Указатель на клонируемую поверхность \ref SURFACE.
 * @return Указатель на поверхность \ref SURFACE.
 */
HDCp sfClone (HDCp SF);

/**
 * @brief Создание из зоны источника.
 * @details Отрицательные X и Y приравниваются к 0.
 * 
 * Если W или H < 0, функция вызывает \ref emptySurface().
 * 
 * Если X + W больше ширины поверхности, функция скопирует источник от X до правого края. \n
 * Если Y + H больше высоты поверхности, функция скопирует источник от Y до нижнего края.
 * 
 * @param SF Указатель на клонируемую поверхность \ref SURFACE.
 * @param X,Y Координаты левого верхнего угла на поверхности-источнике.
 * @param W Ширина поверхности.
 * @param H Высота поверхности.
 * 
 * @return Указатель на поверхность \ref SURFACE.
 */
HDCp sfCloneZone (HDCp SF, int X, int Y, int W, int H);  

/**
 * @brief Создание из зоны по образцу.
 * @details Функция вызывает \b sfCloneZone,
 * передавая ширину и высоту поверхности \b SAM.
 * 
 * @param SF Указатель на поверхность-источник \ref SURFACE.
 * @param X,Y Координаты левого верхнего угла на поверхности-источнике.
 * @param SAM Указатель на поверхность-пример. Новая поверхность будет иметь стороны примера.
 *  
 * @return Указатель на поверхность \ref SURFACE.
 */
HDCp sfCloneSample (HDCp SF, int X, int Y, HDCp SAM);

/**
 * @brief Копирование содержимого одинакового размера.
 * 
 * @param DST Указатель на поверхность-приёмник.
 * @param SRC Указатель на поверхность-источник.
 *  
 * @return \ref ERROR, если приёмник и источник разных размеров, иначе возвращает \ref OK.
 */
STATUS sfCopy (HDCp DST, HDCp SRC);


/// @}

/**
 *  @name Преобразование цвета
 *  @brief Функции преобразования цвета в используемый формат,
 *  который определяется при инициализации библиотеки softGraphInit().
 *  @{
 */

/**
 *  @brief Получить значение цвета в используемом формате.
 *  @details Функция собирает значение цвета из значений яркости для каждого канала.
 *  
 *  @param R,G,B Значения цвета каждого из каналов в пределах от 0 до 255.
 *  @return Результирующее значение.
 */
unsigned int RGB (unsigned int R, unsigned int G, unsigned int B);

/**
 *  @brief Разобрать цвет на составляющие.
 *  @details Функция разбирает значение на \b RGB компоненты.
 *  
 *  @param c Исходное значение.
 *  @param R,G,B Полученные значения.
 */
void fromRGB (unsigned int c, unsigned int *R, unsigned int *G, unsigned int *B);


/// @}

/**
 *  @name Рисование на поверхности
 *  @{
 */


/**
 *  @brief Рисование точки заданного цвета на поверхности.
 *  @details Функция закрашивает точку заданным цветом на указанной поверхности.
 *  
 *  @param psf Поверхность для рисования.
 *  @param X,Y Координаты точки.
 *  @param color Цвет точки с альфа-каналом.
 */
void sfPutPixel (HDCp psf, unsigned int X, unsigned int Y, unsigned int color);

/**
 * @brief Рисование произвольной закрашенной области
 * @details Функция может отрисовывать выпуклые, вогнутые и пересекающие сами себя фигуры. 
 * Можно указывать точки за пределами экрана. 
 * Функция использует библиотеку \b vector. \n
 * 
 * \b Важно! Если некоторые из рёбер фигуры находятся внутри неё и создают отдельную область, 
 * то эта область не будет закрашена. 
 * Если \b n указан меньше реального размера массива, область отрисуется по оставшимся точкам. 
 * Если \b n указан больше реального размера массива, система упадёт.
 * 
 * @param hdc Указатель на повехность.
 * @param points Массив точек с координатами X и Y. Каждая точка - это массив из двух элементов.
 * @param n Размер массива.
 * @param color Цвет заливки с альфа-каналом.
 * @return Возвращает 1 в случае ошибки.
 */
BOOL sfDrawPolygon (HDCp hdc, int points[][2], int n, unsigned int color);     

/**
 *  @brief Считывание цвета точки на поверхности.
 *  @details Функция получает цвет точки на указанной поверхности.
 *  
 *  @param psf Поверхность для рисования.
 *  @param X,Y Координаты точки.
 *  @return Цвет точки.
 */
unsigned int sfGetPixel (HDCp psf, unsigned int X, unsigned int Y);


/**
 * @brief Рисование прямоугольника сплошным цветом.
 * @details Зарисовать прямоугольную область сплошным цветом.
 * Можно указывать отрицательные координаты, 
 * тогда будет отрисовываться только видимая часть прямоугольника.
 * 
 * @param psf Указатель на поверхность.
 * @param X,Y Координаты левого верхнего угла на поверхности.
 * @param W Ширина прямоугольника.
 * @param H Высота прямоугольника.
 * @param color Цвет заливки с альфа-каналом.
 */
void sfBar (HDCp psf, int X, int Y, int W, int H, unsigned int color);

/**
 * @brief Рисование прямоугольника с рамкой цвета карандаша и заливкой цвета кисти.
 * @details Можно указывать отрицательные координаты, 
 * тогда будет отрисовываться только видимая часть прямоугольника.
 * 
 * Фигура будет полой, если значение \b sfBClear равно \ref true.
 * 
 * @param psf Указатель на поверхность.
 * @param X,Y Координаты левого верхнего угла на поверхности.
 * @param W Ширина прямоугольника.
 * @param H Высота прямоугольника.
 * @param fillColor Цвет заливки с альфа-каналом.
 * @param borderColor Цвет границы с альфа-каналом.
 */
void sfRectangle (HDCp psf, int X, int Y, int W, int H, unsigned int fillColor, unsigned int borderColor);


/**
 * @brief Рисование прямоугольника с закругленными краями.
 * @details Можно указывать отрицательные координаты,
 * тогда будет отрисовываться только видимая часть прямоугольника.
 * Границы будут отрисовыватся "внутри" указанных координат.
 *
 * @param psf Указатель на поверхность.
 * @param X,Y Координаты левого верхнего угла на поверхности.
 * @param W,H Ширина и высота прямоугольника.
 * @param curveRadius Радиус кривизны краев прямоуголника (радиус будет усечен, если он будет слишком большим).
 * @param fillColor Цвет заливки с альфа-каналом.
 * @param borderWidth Толщина границы.
 * @param borderColor Цвет границы с альфа-каналом.
 */
void sfCurvedRectangle (HDCp psf, int X, int Y, unsigned W, unsigned H, unsigned curveRadius, uint32_t fillColor, unsigned borderWidth, uint32_t borderColor);

void sfCurvedRectangleTest(HDCp psf, int x, int y, unsigned width, unsigned height, unsigned curveRadius, uint32_t fillColor, unsigned borderWidth, uint32_t borderColor);

/**
 * @brief Рисование линии.
 * @details Можно указывать координаты за пределами поверхности.
 * 
 * @param psf Указатель на поверхность.
 * @param X1,Y1 Координаты первой точки.
 * @param X2,Y1 Координаты последней точки.
 * @param color Цвет линии с альфа-каналом
 */
void sfLine (HDCp psf, int X1, int Y1, int X2, int Y2, unsigned int color);

/**
 * @brief Рисование окружности.
 * @details Можно указывать координаты за пределами поверхности, 
 * тогда будет отрисовываться только видимая часть окружности.
 * 
 * Фигура не отрисуется, если \b r <= 0.
 * 
 * @param psf Указатель на поверхность.
 * @param xc,yc Координаты центра окружности.
 * @param r Радиус.
 * @param color Цвет линии с альфа-каналом.
 */
void sfCircle (HDCp psf, int xc, int yc, int r, unsigned int color);

/**
 * @brief Рисование заполненной окружности с границей.
 * @details Можно указывать координаты за пределами поверхности,
 * тогда будет отрисовываться только видимая часть окружности.
 *
 * Фигура не отрисуется, если \b r <= 0.
 * Итоговый радиус фигуры будет равен r, граница будет отрисована "внутри" этого радиуса.
 *
 * @param psf Указатель на поверхность.
 * @param xc,yc Координаты центра окружности.
 * @param r Радиус окружности.
 * @param fillColor Цвет окружности.
 * @param borderWidth Толщина границы окружности. Может быть нулевой.
 * @param borderColor Цвет границы окружности.
 */
void sfFilledCircle(HDCp psf, int xc, int yc, unsigned r, uint32_t fillColor, unsigned borderWidth, uint32_t borderColor);

/**
 * @brief Рисование гладкой заполненной окружности с границей.
 * @details Можно указывать координаты за пределами поверхности,
 * тогда будет отрисовываться только видимая часть окружности.
 *
 * Фигура не отрисуется, если \b r <= 0.
 * Итоговый радиус фигуры будет равен не r, гладкая граница может незначительно (не более чам на 1 пиксель) выходить за эти границы.
 *
 * Функция работает существенно дольше, чем @ref sfFilledCircle.
 *
 * @param psf Указатель на поверхность.
 * @param xc,yc Координаты центра окружности.
 * @param r Радиус окружности.
 * @param fillColor Цвет окружности.
 * @param borderWidth Толщина границы окружности. Может быть нулевой.
 * @param borderColor Цвет границы окружности.
 */
void sfSmoothCircle(HDCp psf, int xc, int yc, unsigned r, uint32_t fillColor, unsigned borderWidth, uint32_t borderColor);

/**
 * @brief Перемещение карандаша.
 * @details Можно указывать координаты за пределами поверхности.
 * 
 * @param psf Указатель на поверхность.
 * @param X,Y Новые координаты карандаша.
 */
void sfMoveTo (HDCp psf, int X, int Y);

/**
 * @brief Нарисовать линию от текущей позиции карандаша
 * и переместить его в конец линии.
 * @details Можно указывать координаты за пределами поверхности.
 * 
 * @param psf Указатель на поверхность.
 * @param X,Y Координаты последней точки линии.
 * @param color Цвет линии с альфа-каналом.
 */
void sfLineTo (HDCp psf, int X, int Y, unsigned int color);


/**
 * @brief Наложение прямоугольной зоны.
 * @details Функция обрабатывает источники с альфа-каналом.
 * 
 * Можно указывать координаты за пределами поверхности, 
 * тогда будет отрисовываться только видимая часть источника.
 * 
 * @param hdcDest Указатель на поверхность-приёмник.
 * @param nXDest,nYDest Координаты левого верхнего угла на поверхности-приёмнике.
 * @param nWidth Ширина поверхности-приёмника.
 * @param nHeight Высота поверхности-приёмника.
 * 
 * @param hdcSrc Указатель на поверхность-источник.
 * @param nXSrc,nYSrc Координаты левого верхнего угла на поверхности-источника.
 * 
 * @return \ref false, если не удалось наложить источник, иначе \ref true.
 */
BOOL sfBitBlt (
    HDCp hdcDest,  
    int nXDest,    
    int nYDest,    
    int nWidth,    
    int nHeight,   
    HDCp hdcSrc,   
    int nXSrc,     
    int nYSrc      
);

/**
 * @brief Отрисовка источника целиком в приёмнике.
 * @details Функция обрабатывает источники с альфа-каналом.
 * 
 * Можно указывать координаты за пределами поверхности, 
 * тогда будет отрисовываться только видимая часть источника.
 * 
 * @param hdcDest Указатель на поверхность-приёмник.
 * @param nXDest,nYDest Координаты левого верхнего угла на поверхности-приёмнике.
 * @param hdcSrc Указатель на поверхность-источник.
 * 
 * @return \ref false, если не удалось отрисовать источник, иначе \ref true.
 */
BOOL sfDraw (
    HDCp hdcDest,  
    int nXDest,    
    int nYDest,    
    HDCp hdcSrc    
);

/**
 * @brief Отрисовка с заданной степенью прозрачности.
 * @details Указанная прозрачность смешивается с альфа-каналом источника.
 * 
 * Можно указывать координаты за пределами поверхности, 
 * тогда будет отрисовываться только видимая часть источника.
 * 
 * @param hdcDest Указатель на поверхность-приёмник.
 * @param nXDest,nYDest Координаты левого верхнего угла на поверхности-приёмнике.
 * @param hdcSrc Указатель на поверхность-источник.
 * @param nTsp Прозрачность, от 0 до 0xff, где 0 - полностью прозрачный.
 * 
 * @return \ref false, если не удалось наложить источник, иначе \ref true.
 */
BOOL sfDrawTransparent (
    HDCp hdcDest,  
    int nXDest,    
    int nYDest,    
    HDCp hdcSrc,   
    unsigned char nTsp
);


/**
 * @brief Отрисовка с масштабированием.
 * @details Функция обрабатывает источники с альфа-каналом.
 * 
 * Можно указывать координаты за пределами поверхности, 
 * тогда будет отрисовываться только видимая часть источника.
 * 
 * \b Важно! Если обе поверхности одинакового размера, вызывается функция sfBitBlt. 
 * Эта функция не учитывает параметры \b srcW,srcH
 * 
 * @param hdc Указатель на поверхность-приёмник.
 * @param hdcX,hdcY Координаты левого верхнего угла на поверхности-приёмнике.
 * @param hdcW,hdcH Ширина и высота поверхности-приёмника.
 * 
 * @param src Указатель на поверхность-источник.
 * @param srcX,srcY Координаты левого верхнего угла на поверхности-источника.
 * @param srcW,srcH Ширина и высота поверхности-источника.
 * 
 * @return false, если не удалось отрисовать источник, иначе true
 */
BOOL sfStretchBlt (HDCp hdc, int hdcX, int hdcY, int hdcW, int hdcH, 
                   HDCp src, int srcX, int srcY, int srcW, int srcH);


/**
 * @brief Наложение цвета с использованием альфа-канала из прямоуголной зоны.
 * @details Предназначение функции - быстрый и экономичный вывод глифов текста.
 * Функция использует значение альфа-канала из зоны hdcSrc, но игнорирует цветовую
 * составляющую этой зоны. В качестве цвета вывода используется значение color.
 *
 * Можно указывать координаты за пределами поверхности,
 * тогда будет отрисовываться только видимая часть источника.
 *
 * @param hdcDest Указатель на поверхность-приёмник.
 * @param nXDest,nYDest Координаты левого верхнего угла на поверхности-приёмнике.
 * @param nWidth Ширина поверхности-приёмника.
 * @param nHeight Высота поверхности-приёмника.
 *
 * @param hdcSrc Указатель на поверхность-источник, из которой будет браться значение
 * альфа-канала.
 * @param nXSrc,nYSrc Координаты левого верхнего угла на поверхности-источника.
 *
 * @param color Цвет, который будет использоваться при наложении.
 *
 * @return \ref false, если не удалось наложить источник, иначе \ref true.
 */
bool sfBitBltAlphaColor (
    HDCp hdcDest,
    int nXDest,
    int nYDest,
    int nWidth,
    int nHeight,
    HDCp hdcSrc,
    int nXSrc,
    int nYSrc,
    uint32_t color
);

/// @}



#ifdef __cplusplus
}
#endif
#endif // ifndef _SOFTGRAPH_H_
