/**
    @addtogroup groupDrivers
    @{
    @file pwm.h
    @brief Управление линиями ШИМ (PWM).
    @details Настройка выводов \b PWM, в зависимости от выбранного процессора.

    <b>Подключение</b>: @verbatim #include <pwm.h> @endverbatim

    @see Общее описание работы с модулем ШИМ в разделе \ref sectionPwm.
    @}

    @page pageDrivers

    @section sectionPwm PWM (ШИМ)
    @see Описание методов работы с линиями вывода ШИМ в файле pwm.h.

    Поддержка аппаратных модулей \b ШИМ в @osName реализована в двух режимах
    --- импульсном и непрерывном. Для каждого режима необходимо сначала вызвать
    функцию инициализации аппаратного модуля для выбранного канала. Далее в
    непрерывном режиме следует указать коэффициент заполнения ШИМ. Изменяя
    коэффициент заполнения в непрерывном режиме можно, например, управлять
    яркостью дисплея. В импульсном режиме для запуска каждого импульса следует
    вызывать запускающую функцию. Подробное описание функций можно найти в
    файле pwm.h.

    В различных процессорах для выходных линий ШИМ используются разные пины.
    \ref sectionArch "Конфигурация" линий конкретного процессора считывается
    при старте операционной системы. И далее, при вызове функции инициализации
    для одного из каналов ШИМ, соответствующий каналу вывод процессора будет
    настроен должным образом.

 */

/*
    @date Created on: 5 апр. 2021 г.
    @author Dubovich Timofey, Александр Матяш.
*/


#ifndef __PWM_H_
#define __PWM_H_

#include <multex.h>

/**
 *  @name Макросы номеров каналов ШИМ
 *  @brief Синонимы номеров каналов возможно использовать 
 *  в качестве параметров функций \b channel
 *  для улучшения читаемости кода.
 *  @{
 */
#define PWM_0 0 ///< @details Канал ШИМ 0.
#define PWM_1 1 ///< @details Канал ШИМ 1.
/// @}

/**
 *  @name Выбор начального уровня сигнала
 *  @brief Данные макросы рекомендуется использовать
 *  в качестве параметров функций \b activeHigh
 *  для улучшения читаемости кода.
 *  @{
 */
#define PWM_ACTIVE_HIGH true    ///< @details Активный уровень сигнала -- высокий.
#define PWM_ACTIVE_LOW  false   ///< @details Активный уровень сигнала -- низкий.
/// @}

/**
 *  @name Непрерывный режим
 *  @{
 */

/**
 *  @brief Настройка канала ШИМ.
 *  @details Функция настраивает режим работы канала ШИМ.
 *  Возможен повторный вызов функции для перенастройки канала в процессе работы.
 *  
 *  @param channel Канал ШИМ.
 *  @param frequency Частота следования импульсов в герцах. Возможный диапазон
 *  частот от 0 Гц до 1 МГц.
 *  Коэффициент заполнения задаётся с помощью pwmSetFillFactor().
 *  @param activeHigh Уровень активной части сигнала: \ref true -- высокий уровень,
 *  иначе низкий уровень сигнала.
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS pwmInit (unsigned int channel, unsigned int frequency, bool activeHigh);

/**
 *  @brief Задать коэффициент заполнения ШИМ.
 *  @details Функция устанавливает коэффициент заполнения ШИМ
 *  в процентах (всего 100 градаций). Период ШИМ предварительно задаётся
 *  в настройках pwmInit().
 *  
 *  @param channel Канал ШИМ.
 *  @param fillFactor Коэффициент заполнения от 0 до 100 (максимальное заполнение).
 *  @return \ref OK если коэффициент успешно выставлен,
 *  иначе \ref ERROR.
 */
STATUS pwmSetFillFactor (unsigned int channel, unsigned int fillFactor);

/// @}

/**
 *  @name Импульсный режим
 *  @{
 */

/**
 *  @brief Инициализация канала ШИМ в импульсном режиме.
 *  @details Функция настраивает канал шим на запуск импульсов
 *  заданной длительности по команде pwmPulseStart().
 *  
 *  @param channel Канал ШИМ.
 *  @param activeHigh Уровень активной части сигнала: \ref true -- высокий уровень,
 *  иначе низкий уровень сигнала.
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS pwmInitPulse (unsigned int channel, bool activeHigh);

/**
 *  @brief Расчёт значения регистра периода для импульсного режима.
 *  @details Функция рассчитывает значение, записываемое в регистр
 *  определения периода импульса. Рассчитанное значение следует 
 *  использовать в функции запуска импульса pwmPulseStart().
 *  
 *  @param duration_ns Длительность импульса в  наносекундах.
 *  Возможный диапазон значений от 50 нс до 2,73 мс.
 *  @return Значение для функции pwmPulseStart().
 */
unsigned int pwmPulseDurationCalc (unsigned int duration_ns);


/**
 *  @brief Запуск импульса заданной длительности в выбранном канале ШИМ.
 *  @details Функция запускает одиночный импульс в канале ШИМ.
 *  Канал должен быть сконфигурирован с помощью pwmInitPulse().
 *  Для обеспечения максимального быстродействия при вызове функции
 *  в прерываниях следует использовать заранее рассчитанные
 *  с помощью функции pwmPulseDurationCalc()
 *  значения периода, которые будут записаны непосредственно 
 *  в регистр аппаратного модуля ШИМ.
 *  
 *  @param channel Канал ШИМ.
 *  @param period_value Значение регистра периода ШИМ, 
 *  рассчитанное с помощью pwmPulseDurationCalc().
 *  @return \ref OK при успешном запуске, иначе \ref ERROR.
 */
STATUS pwmPulseStart (unsigned int channel, unsigned int period_value);


/// @}

/// @privatesection

// Системные функции

/**
 *  @brief Чтение конфигурации аппаратной части.
 *  @details Чтение текущей конфигурации аппаратной части,
 *  настройка модуля соответствующим образом.
 *  Вызывается при старте операционной системы
 *  после загрузки конфигурации.
 *  @return \ref OK если в конфигурации найдены
 *  нужные параметры (известный тип процессора),
 *  иначе \ref ERROR.
 */
STATUS pwmArchInit ();




// Устаревшие функции

/**
 * Initialize PWM channel.
 * @deprecated Устаревшая функция - вызывает pwmInit()
 * без возможности настройки частоты. Оставлена для совместимости.
 * @param channel Channel number, 0 or 1.
 * @return OK if succeeded, ERROR otherwise.
 */
STATUS __attribute__((deprecated)) pwm_Init (size_t channel);


/**
 * Setup duty for PWM channel.
 * @deprecated Устаревшая функция - вызывает pwmSetFillFactor()
 * без возможности настройки частоты. Оставлена для совместимости.
 * @param X Some kind of value.
 * @param channel Channel number, 0 or 1.
 * @return OK if succeeded, ERROR otherwise.
 * @details PWM should be initialized first.
 */
STATUS __attribute__((deprecated)) pwm_Duty (unsigned char X, size_t channel);




#endif /* __PWM_H_ */
