/**
    @addtogroup groupCore
    @{
    @file ringBuffer.h
    @brief Кольцевой буфер.
    @details Кольцевой буфер -- альтернатива очередей сообщений для использования в прерываниях.
    Буфер может содержать заданное количество элементов фиксированной длины.
    
    @}
 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 04.07.2023
*/

#ifndef _RINGBUFFER_H_
#define _RINGBUFFER_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>


/**
 *  @brief Структура колцевого буфера.
 *  @details 
 *  
 */
typedef struct {
    unsigned int maxCnt;        ///< @details Количество элементов буфера.
    unsigned int nSize;         ///< @details Размер одного элемента дынных буфера.
    unsigned int inCnt;         ///< @details Счётчик входящих элементов буфера.
    unsigned int outCnt;        ///< @details Счётчик исходящих элементов буфера.
    int delta;                  ///< @details Дельта между входящими и исходящими элементами (количество данных).
    char *data;                 ///< @details Указатель на данные.
} tRingBuffer;


/**
 *  @name Управление буферами
 *  @{
 */

/**
 *  @brief Создать кольцевой буфер.
 *  @details Функция создаёт буфер и выделяет память
 *  под данные.
 *  
 *  @param dataSize Размер одного элемента данных в буфере.
 *  @param maxCount Количество элементов в буфере.
 *  Количество элементов должно быть больше одного.
 *  @return Указатель на созданный буфер.
 */
tRingBuffer *newRingBuffer (unsigned int dataSize, unsigned int maxCount);


/**
 *  @brief Удалить кольцевой буфер.
 *  @details Функция освобождает память и удаляет буфер.
 *  
 *  @param buf Удаляемый буфер.
 *  @return \ref OK в случае успешного удаления буфера, иначе \ref ERROR.
 */
STATUS deleteRingBuffer (tRingBuffer *buf);

/// @}

/**
 *  @name Запись / чтение данных
 *  @{
 */


/**
 *  @brief Запись элемента данных в буфер.
 *  @details Функция записывает один элемент данных в кольцевой буфер
 *  и инкрементирует входной счётчик данных. Запись возможна только пока
 *  в буфере есть свободное место. Если место закончилось необходимо
 *  прочитать все записи либо очистить буфер.
 *  
 *  @param buf Указатель на кольцевой буфер.
 *  @param data Записываемые данные -- указатель на записываемый элемент.
 *  @param timeout Таймаут ожидания появления свободного места в буфере в тиках системного таймера.
 *  Если свободное место для записи не появилось в течение заданного времени функция вернёт ошибку.
 *  В качестве параметра можно также использовать значения \ref NO_WAIT и \ref WAIT_FOREVER.
 * 
 *  \b ВАЖНО! При записи данных в обработчиках прерываний следует использовать 
 *  значение таймаута \ref NO_WAIT.
 * 
 *  @return \ref OK в случае удачной записи, либо \ref ERROR
 *  при переполнении буфера.
 *  
 */
STATUS ringBufferWrite (tRingBuffer *buf, const void *data, int timeout);

/**
 *  @brief Чтение элемента данных из буфера.
 *  @details Функция читает один элемент данных из кольцевого буфера
 *  и декрементирует счётчик данных. Чтение возможно только если в 
 *  буфере есть не считанные данные. Если задан параметр \b timeout
 *  больший нуля -- функция будет ждать появления данных заданное количество 
 *  времени.
 * 
 *  \b ВАЖНО! При чтении данных в обработчиках прерываний следует использовать 
 *  значение таймаута \ref NO_WAIT.
 *  
 *  @param data Указатель на данные, в который будет считан элемент буфера.
 *  @param buf Указатель на кольцевой буфер.
 *  @param timeout Таймаут ожидания наличия данных в тиках системного таймера.
 *  Если данные не появились в течение заданного времени функция вернёт ошибку.
 *  В качестве параметра можно также использовать значения \ref NO_WAIT и \ref WAIT_FOREVER.
 *  @return \ref OK в случае удачного чтения, либо \ref ERROR
 *  если данных нет и они не появились за заданный промежуток времени.
 */
STATUS ringBufferRead (void *data, tRingBuffer *buf, int timeout);


/**
 *  @brief Количество записей в буфере.
 *  @details Функция возвращает количество не прочитанных записей в кольцевом буфере.
 *  Возвращаемое значение должно быть строго не отрицательным. Отрицательное число
 *  в результате говорит о некорректной работе буфера.
 *  
 *  @param buf Указатель на кольцевой буфер.
 *  @return Количество записей.
 */
int ringBufferCounter (const tRingBuffer *buf);

/**
 *  @brief Очистка буфера.
 *  @details Функция выравнивает счётчики входящих и исходящих записей
 *  колцевого буфера и обнуляет счётчик имеющихся записей.
 *  
 *  @param buf 
 *  @return Всегда \ref OK.
 */
STATUS ringBufferFlush (tRingBuffer *buf);

/// @}


#ifdef __cplusplus
}
#endif

#endif
