/**
    @addtogroup groupDrivers
    @{
    @file sata.h
    @brief Драйвер \b SATA.
    @details Драйвер работы с одним устройством \b SATA.

    Пример подключения и использования устройства показан в листинге:

    @code
    // Запуск устройства
    sataMount (0, 0);

    // ...

    // Запись массива в файл на подключенный диск 
    char data[512];
    char fileName[20];
    sprintf (fileName, "%s/test.bin", sataLabelGet (0, 0));
    int f = open (fileName, O_WRONLY | O_CREAT);
    write (f, data, 512);
    close (f);

    // ...

    // Отключение устройства
    sataUmount (0, 0);
    @endcode

    @see \ref sectionIoLib
    @}
 */

//  @date Created on: 15 Sep 2021
//  @author Dubovich Timofey
//  @author Матяш А.В.

#ifndef __SATA_H_
#define __SATA_H_

#include <multex.h>
#include <iolib.h>


/**
 *  @brief Подключить диск \b SATA к текущей файловой системе.
 *  @details Функция создаёт устройство с помощью createSATADev()
 *  и подключает к файловой системе. Файловая система уже должна быть
 *  создана, например с помощью dosInit().
 *  Повторное подключение одного и того-же устройства -- возможно.
 *  Для удаления устройства перед повторным подключением воспользуйтесь
 *  функцией sataUmount();
 *  
 *  @param n Номер физического диска \b SATA из описания на процессор (плату).
 *  @param p Номер монтируемого раздела на диске, начиная с нулевого. 
 *  Для дисков с одним разделом -- \b 0.
 * 
 *  @return \ref true при удачном подключении диска, иначе \ref false.
 */
STATUS sataMount (int n, int p);

/**
 *  @brief Поиск имени раздела \b SATA диска в списке подключенных устройств.
 *  @details Функция позволяет найти имя уже подключенного к файловой системе
 *  раздела \b SATA по номеру физического диска и номеру раздела. Функция является
 *  обёрткой функции iosTypedVolumeLabelFind(), предназначенной для поиска именно
 *  диска \b SATA.
 *  @warning На данный момент поддерживается подключение только одного раздела одного диска.
 *  Значения \b n и \b p рекомендуется задавать равными нулю.
 *  
 *  @param n Номер физического диска \b SATA из описания на процессор (плату).
 *  @param p Номер монтируемого раздела на диске, начиная с нулевого. 
 *  Для дисков с одним разделом -- \b 0.
 *  @return Указатель на имя раздела диска (например \b С: или \b D: ) 
 *  если указанный раздел найден, иначе \ref NULL.
 */
const char *sataLabelGet (int n, int p);

/**
 *  @brief Отключение заданного раздела указанного диска \b SATA.
 *  @details Функция отключает заданный раздел указанного диска и освобождает 
 *  выделенные ресурсы.
 *  
 *  @param n Номер физического диска \b SATA из описания на процессор (плату).
 *  @param p Номер смонтированного раздела на диске, начиная с нулевого. 
 *  Для дисков с одним разделом -- \b 0.
 *  @return \ref OK при успешном отключении, иначе \ref ERROR.
 */
STATUS sataUmount (int n, int p);


/// @privatesection

/**
 *  @brief Создать и подключить устройство \b SATA.
 *  @details Функция ищет и подключает \b SATA устройство.
 *  Повторное подключение одного и того-же устройства -- возможно.
 *  Для удаления устройства перед повторным подключением воспользуйтесь
 *  функцией sataDevDelete();
 *  @deprecated Не рекомендуется использовать -- это устаревшая функция. Используйте sataMount().
 *  
 *  @return \ref true если устройство создано, иначе \ref false.
 */
bool sataDevCreate () __attribute__ ((deprecated("This function is deprecated, use sataMount() instead")));

/**
 *  @brief Удаление ранее созданного устройства \b SATA.
 *  @details Функция отключает раздел \b первого зарегистрированного в файловой системе 
 *  устройства \b SATA и освобождает выделенную память.
 *  @deprecated Не рекомендуется использовать -- это устаревшая функция. Используйте sataUmount().
 *  
 *  @return \ref true если устройство удалено, иначе \ref false.
 */
bool sataDevDelete ()  __attribute__ ((deprecated("This function is deprecated, use sataUmount() instead")));

/**
 *  @brief Подключение устройства \b SATA.
 *  @details Функция ищет и подключает \b SATA устройство.
 *  @return \ref OK при удачном подключении, иначе \ref ERROR.
 */
STATUS sata_FindInitAndMount (void) __attribute__ ((deprecated("This function is deprecated, use sataDevCreate() instead")));

/**
 *  @brief Получить имя диска.
 *  @details Функция возвращает имя ранее созданного \b SATA устройства.
 *  Функция ищет нулевой раздел первого зарегистрированного в системе 
 *  диска \b SATA. 
 *  @deprecated Не рекомендуется использовать -- это устаревшая функция. Используйте sataLabelGet().
 *  
 *  @return Строка вида 'E:' если диск найден, иначе \ref NULL.
 */
const char *sataVolumeGet ()  __attribute__ ((deprecated("This function is deprecated, use sataLabelGet() instead")));

/**
 *  @brief Получить символ подключенного диска.
 *  @brief Функция возвращает символ подключенного \b SATA устройства.
 *  Обычно это буква \b E.
 *  @return Символ подключенного диска, либо \b 0 в случае ошибки.
 */
char sata_GetSataVolumeLetter (void) __attribute__ ((deprecated("This function is deprecated, use sataVolumeGet() instead")));



// Создать устройство \b SATA.
// Функция вызывается в sata_FindInitAndMount()
STATUS createSATA (void) __attribute__ ((deprecated("This function is deprecated, use sataDevCreate() instead")));

#endif /* __SATA_H_ */
