/**
    @addtogroup groupCore 
    @{
    @file setjmp.h
    @brief Нелокальные переходы.
    @details В файле описаны функции нелокальных переходов по стандарту 
    <a href="https://www.iso.org/standard/57853.html">C11 standard 7.13</a>.
    @see Общее описание в главе \ref sectionSetJmp.
    @}

    @page pageCore

    @section sectionSetJmp Нелокальные переходы
    @see Описание функций нелокальных переходов в файлах setjmp.h и stdlib.h.

    В ядре @osName содержится библиотека стандартной поддержки нелокальных
    переходов \b setjmp. Библиотека объявляет тип данных \ref jmp_buf, который является
    массивом и который может использоваться для сохранения и восстановления
    контекста выполнения программы. Для установки точки сохранения служит функция setjmp().
    Вернуться в точку сохранения можно с помощью функции longjmp(). Для сохранения и восстановления
    контекста при переключении между задачами служат функции setexit() и exit().

 */


/*
    @author Sergey Sokolov
    @author Dubovich Timofey
*/

#ifndef SETJMP_H_
#define SETJMP_H_

#ifdef __cplusplus
extern "C" {
#endif

/* Public declarations */
#include "stdnoreturn.h"

/**
 * Тип, описывающий набор регистров ARMv7.
 */
typedef struct /* REG_SET - ARMv7 register set */
{
    int r2;
    int r3;
    int r4;
    int r5;
    int r6;
    int r7;
    int r8;
    int r9;
    int r10;
    int fp;
    int sp;
    int lr;
    int pc;
} REG_SET;

/**
 * Тип, описывающий информацию необходимую для нелокального перехода и позволяющий сохранить контекст.
 */
typedef struct
{
    REG_SET REGS;
} jmp_buf[1];


/**
 * Сохранить контекст для дальнейшего перехода.
 * @param env Буфер для сохранения контекста.
 * @return \b 0, если функция установила точку, \b не-0, если возврат функции был вызван с помощью longjmp().
 */
int	setjmp (jmp_buf env);

/**
 * Перейти к сохраненному контексту.
 * @param env Точка сохранения контекста.
 * @param val Значение, которое будет передано в функцию setjmp().
 */
noreturn void longjmp (jmp_buf env, int val);

#include "stdlib.h" // Для обратной совместимости, декларации exit, atexit, _Exit, setexit, exitcode, exitbuf


#ifdef __cplusplus
}
#endif
#endif // ifndef SETJMP_H_
