/**
    @addtogroup groupCore
    @{
    @file signal.h
    @brief Обработка сигналов (<b>C11</b> + частично <b>POSIX</b>).
    @details
    @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.14</a>.
    @see Подробнее о сигналах см. в главе \ref sectionSignals.
    @}
 */

/*
    @author Sergey Sokolov
    @author Dubovich Timofey
*/

#ifndef SIGNAL_H_
#define SIGNAL_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "stdint.h"
// #include "tasklib.h"

//! @details Целочисленный тип к которому можно получить доступ как к атомарной сущности
//! даже при наличии асинхронных прерываний по сигналам.
typedef int sig_atomic_t;

//! @details Целочисленный тип, описывающий объект, 
//! используемый для хранения набора сигналов.
typedef uint32_t sigset_t;

/**
 * @name Аргументы и возвращаемые значения для функции signal()
 * @{
 */
//! @details Возвращаемое значение, свидетельствующее об ошибке.
#define SIG_ERR ((void *)-1)
//! @details Значение для установки поведения по умолчанию.
#define SIG_DFL ((void *)0)
//! @details Значение для игнорирования сигнала.
#define SIG_IGN ((void *)1)
/**
 * @}
 */


/**
 * @name Стандартные для Си сигналы
 * @anchor labelSignalStdSet
 * @{
 */

/**
 *  @brief Запрос на ненормальное завершение программы.
 *  @details Ненормальное завершение работы. Например такое, 
 *  которое генерируется функцией \b abort.
 */
#define SIGABRT 6

/**
 *  @brief Ошибка арифметики.
 *  @details Ошибка операции с плавающей точкой, например переполнение, 
 *  или деление на ноль.
 */
#define SIGFPE 8

/**
 *  @brief Выполнение недопустимой инструкции.
 *  @details Неправильный образ функции, например, неправильная
 *  инструкция. Такой сигнал может быть выдан в результате повреждения кода или при
 *  попытке исполнить данные вместо кода.
 *  
 */
#define SIGILL 4

/**
 *  @brief Получение интерактивного сигнала.
 *  @details Сигнал-прерывание. Обычно генерируется пользователем
 *  приложения.
 */
#define SIGINT 2

/**
 *  @brief Ошибка доступа к памяти.
 *  @details Ошибка доступа к памяти. Программа пытается использовать 
 *  ту область памяти, которая ей не принадлежит.
 *  
 */
#define SIGSEGV 11

/**
 *  @brief Запрос на завершение программы.
 *  @details Запрос на прекращение работы программы.
 *  
 */
#define SIGTERM 15
/**
 * @}
 */

/**
 * @name Дополнительные сигналы (в т.ч. POSIX)
 * @{
 */
//! @details Особое значение для отсутствия сигнала.
#define SIGNONE 0
//! @details Освобождение линии терминала.
#define SIGHUP 1
//! @details Прерывание с аварийным завершением.
#define SIGQUIT 3
//! @details Отладка.
#define SIGTRAP 5
//! @details Инструкция EMT.
#define SIGEMT 7
//! @details Немедленное завершение работы. Не может быть обработан, пойман или проигнорирован.
#define SIGKILL 9
//! @details Ошибка шины памяти.
#define SIGBUS 10
//! @details Неправльный системный вызов.
#define SIGSYS 12
//! @details Запись в разорваное соединение.
#define SIGPIPE 13
//! @details Истечение времени, заданного alarm().
#define SIGALRM 14
//! @details Пользовательский сигнал 1.
#define SIGUSR1 16
//! @details Пользовательский сигнал 2.
#define SIGUSR2 17
//! @details Дочерний процесс завершен или остановлен.
#define SIGCHLD 18
//! @details Ошибка формата стека.
#define SIGFMT 19
//! @details Остановка с терминала.
#define SIGTSTP 20
//! @details Получены срочные данные.
#define SIGURG 21
//! @details Событие, отслеживаемое poll().
#define SIGPOLL 22
//! @details Нетерминальная остановка. Не может быть обработан, пойман или проигнорирован.
#define SIGSTOP 23
//! @details Максимальное значение для сигналов реального времени.
#define SIGRTMIN 24
//! @details Минимальное значение для сигналов реального времени.
#define SIGRTMAX 24
//! @details Возобновление приостановленного процесса.
#define SIGCONT 25
//! @details Попытка чтения с терминала фоновым процессом.
#define SIGTTIN 26
//! @details Попытка записи на терминал фоновым процессом.
#define SIGTTOU 27
//! @details Истечение виртуального таймера.
#define SIGVTALRM 28
//! @details Истечение таймера профилирования.
#define SIGPROF 29
//! @details Процесс привысил лимит процессорного времени.
#define SIGXCPU 30
//! @details Процесс превысил допустимый размер файла.
#define SIGXFSZ 31
/**
 * @}
 */

//! @details Тип, описывающий функцию-обработчик сигнала.
typedef void (*sig_handle) (int);

/**
 * @brief Установить обработчик сигнала.
 * @details Функция принимает в качестве аргумента сигнал и указатель 
 * на \b void функцию, которая принимает сигнал.
 * @param sig Сигнал, для которого устанавливается обработчик.
 * @param func \ref SIG_IGN, для игнорирования сигнала, \ref SIG_DFL, 
 * для обработки сигнала по-умолчанию или указатель на обработчик сигнала.
 * @return Значение \b func для прошлого успешного вызова функции signal() 
 * или \ref SIG_ERR, в случае ошибки.
 */
sig_handle signal (int sig, sig_handle func);

/**
 * @brief Отправить сигнал текущему процессу.
 * @details Функция позволяет послать из программы сигнал.
 * @param sig Сигнал.
 * @return \ref OK при успешной отправке, либо код ошибки.
 */
int raise (int sig);

#include "tasklib.h"
/**
 * @brief Отправить сигнал процессу.
 * @details Функция посылает сигнал указанной задаче.
 * @param ti Идентификатор задачи, которой требуется послать сигнал.
 * @param sig Сигнал.
 * @return \ref OK при успешной отправке, либо код ошибки.
 */
int kill (TASK_ID ti, int sig);

/**
 * @brief Приостановить выполнение текущей задачи.
 * @details Функция приостанавливает выполнение вызвавшей задачи до тех пор, 
 * пока не прекратит выполнение один из её потомков.
 * @param status_p Буфер под статус задачи-потомка.
 * @return Указатель на структуру задачи-потомка.
 */
TASK_ID wait (int *status_p);

/**
 * @name Значения поля sa_flags в структуре sigaction
 * @{
 */
#define SA_NOCLDSTOP 0x0001 ///< @details Do not generate SIGCHLD when children stop.
#define SA_SIGINFO 0x0002   ///< @details Pass additional siginfo structure.
#define SA_ONSTACK 0x0004   ///< @details Run on sigstack.
#define SA_INTERRUPT 0x0008 ///< @details Don't restart the function.
#define SA_RESETHAND 0x0010 ///< @details Reset the handler, like sysV.
/** @} */

/**
 * @name Значения si_code возвращаемые siginfo
 * @{
 */
#define SI_SYNC 0    ///< @details (Not posix) gernerated by hardware.
#define SI_KILL 1    ///< @details signal from .1 kill() function.
#define SI_QUEUE 2   ///< @details signal from .4 sigqueue() function.
#define SI_TIMER 3   ///< @details signal from expiration of a .4 timer.
#define SI_ASYNCIO 4 ///< @details signal from completion of an async I/O.
#define SI_MESGQ 5   ///< @details signal from arrival of a message.
/** @} */

/**
 * @name Структуры и макросы для POSIX-совместимой обработки сигналов
 * @{
 */

union sigval {
    int sival_int;
    void *sival_ptr;
};

/**
 *  @brief Структура данных сигнала.
 *  @details Структура данных сигнала.
 */
typedef struct siginfo {
    int si_signo;           ///< @details Номер передаваемого сигнала.
    int si_code;            ///< @details Код сигнала.
    union sigval si_value;  ///< @details Значение сигнала.
} siginfo_t;

/**
 *  @brief Структура обработчика сигнала.
 *  @details Структура обработчика сигнала.
 */
struct sigaction {
    void (*sa_handler) (int);                        ///< @details Функция обработчика.
    sigset_t sa_mask;                                ///< @details Сигналы, блокир-ся во время обработки сигнала. @see \ref labelSignalSaMask "Функции создания наборов сигналов для поля sa_mask".
    int sa_flags;                                    ///< @details Флаги, влияющие на поведение сигнала.
    void (*sa_sigaction) (int, siginfo_t *, void *); ///< @details Указатель на обработчик сигнала.
};

#define SIG_BLOCK 1
#define SIG_UNBLOCK 2
#define SIG_SETMASK 3

/** @} */

/**
 * @brief Установить обработчик сигнала.
 * @details Функция задает обработчик сигнала.
 * @param signum Номер сигнала.
 * @param act Новый обработчик, или \ref NULL, если установка обработчика не требуется.
 * @param oldact Буфер под старый обработчик, или \ref NULL, 
 * если сохранение старого обработчика не требуется.
 * @return \ref OK при успешной установке, либо код ошибки.
 */
int sigaction (int signum, const struct sigaction *act, struct sigaction *oldact);


/**
 *  @name Операции над наборами сигналов
 *  @anchor labelSignalSaMask
 *  @brief Функции, позволяющие создавать наборы сигналов в стандарте \b POSIX 
 *  для формирования поля \ref sigaction::sa_mask "sa_mask" в структуре \ref sigaction.
 *  @{
 */

/**
 * @brief Проинициализировать набор сигналов.
 * @details Функция инициализирует набор сигналов, указанный в \b set,
 * и "очищает" его от всех сигналов.
 * @param set Буфер набора сигналов.
 * @return \ref OK При успешном завершении.
 * @return \ref ERROR При ошибке.
 */
int sigemptyset (sigset_t *set);

/**
 * @brief Проинициализировать набор сигналов.
 * @details Функция полностью инициализирует набор \b set,
 * в котором содержатся все сигналы.
 * @param set Буфер набора сигналов.
 * @return \ref OK При успешном завершении.
 * @return \ref ERROR При ошибке.
 */
int sigfillset (sigset_t *set);

/**
 * @brief Добавить сигнал к набору сигналов.
 * @details Функция добавляет сигнал \b signum к \b set.
 * @param set Буфер набора сигналов.
 * @param signum Добавляемый сигнал.
 * @return \ref OK При успешном завершении.
 * @return \ref ERROR При ошибке.
 */
int sigaddset (sigset_t *set, int signum);

/**
 * @brief Удалить сигнал из набора сигналов.
 * @details Функция удаляет сигнал \b signum из набора \b set.
 * @param set Буфер набора сигналов.
 * @param signum Удаляемый сигнал.
 * @return \ref OK При успешном завершении.
 * @return \ref ERROR При ошибке.
 */
int sigdelset (sigset_t *set, int signum);

/**
 * @brief Проверить, является ли сигнал членом набора сигналов.
 * @details Функция проверяет, является ли \b signum членом набора \b set.
 * @param set Указатель на набор сигналов.
 * @param signum Проверяемый сигнал.
 * @return \b 1 --- если \b signum является членом набора \b set.
 * @return \b 0 --- если \b signum не является членом набора.
 * @return \b -1 --- при ошибке.
 */
int sigismember (sigset_t *set, int signum);

/// @}

#ifdef __cplusplus
}
#endif
#endif  // ifndef SIGNAL_H_
