/*
        RTOS MULTEX32

        Serial device driver

        Author Sergey V.Sokolov 2000 - 2012
                Co-author Dmitry 'Boko' Popov  2006

*/

#ifndef _SIODRV
#define _SIODRV

#ifdef __cplusplus
extern "C" {
#endif


#define SIOSETINFO  0x02700031
#define SIOGETINFO  0x02700032
#define SIOREINIT   0x02700033
#define SIOFLUSH    0x02700034



// DaVinchi
#define UART_D_BASE 0x49020000  // UART3
#define UART_D_IRQ 74
// Sitara
#define UART_S_BASE 0x44E09000  // UART0
#define UART_S_IRQ 72


#define FCR_FIFO_EN 0x01 /* Fifo enable */
#define FCR_RXSR 0x02    /* Receiver soft reset */
#define FCR_TXSR 0x04    /* Transmitter soft reset */

#define MCR_DTR 0x01
#define MCR_RTS 0x02
#define MCR_DMA_EN 0x04
#define MCR_TX_DFR 0x08

#define LCR_WLS_MSK 0x03 /* character length slect mask */
#define LCR_WLS_5 0x00   /* 5 bit character length */
#define LCR_WLS_6 0x01   /* 6 bit character length */
#define LCR_WLS_7 0x02   /* 7 bit character length */
#define LCR_WLS_8 0x03   /* 8 bit character length */
#define LCR_STB 0x04     /* Number of stop Bits, off = 1, on = 1.5 or 2) */
#define LCR_PEN 0x08     /* Parity eneble */
#define LCR_EPS 0x10     /* Even Parity Select */
#define LCR_STKP 0x20    /* Stick Parity */
#define LCR_SBRK 0x40    /* Set Break */
#define LCR_BKSE 0x80    /* Bank select enable */

#define LSR_DR 0x01   /* Data ready */
#define LSR_OE 0x02   /* Overrun */
#define LSR_PE 0x04   /* Parity error */
#define LSR_FE 0x08   /* Framing error */
#define LSR_BI 0x10   /* Break */
#define LSR_THRE 0x20 /* Xmit holding register empty */
#define LSR_TEMT 0x40 /* Xmitter empty */
#define LSR_ERR 0x80  /* Error */

/* useful defaults for LCR */
#define LCR_8N1 0x03

#define LCRVAL LCR_8N1                             /* 8 data, 1 stop, no parity */
#define MCRVAL (MCR_DTR | MCR_RTS)                 /* RTS/DTR */
#define FCRVAL (FCR_FIFO_EN | FCR_RXSR | FCR_TXSR) /* Clear & enable FIFOs */

#define RHR_IT 0x01 /* RHR INTERRUPT ENABLE */
#define THR_IT 0x02 /* THR INTERRUPT ENABLE */
#define ERR_IT 0x04 /* ERR INTERRUPT ENABLE */



#define thr rbr
#define iir fcr
#define dll rbr
#define dlm ier


struct NS16550 {
    unsigned char rbr; /* 0 */
    int pad1 : 24;
    unsigned char ier; /* 1 */
    int pad2 : 24;
    unsigned char fcr; /* 2 */
    int pad3 : 24;
    unsigned char lcr; /* 3 */
    int pad4 : 24;
    unsigned char mcr; /* 4 */
    int pad5 : 24;
    unsigned char lsr; /* 5 */
    int pad6 : 24;
    unsigned char msr; /* 6 */
    int pad7 : 24;
    unsigned char scr; /* 7 */
    int pad8 : 24;
    unsigned char mdr1; /* mode select reset TL16C750*/
} __attribute__ ((packed));

typedef volatile struct NS16550 *NS16550_t;

#include "msgqlib.h"
#include "ringbuffer.h"



// #define SIODRV_USE_MESSAGES // Использовать очереди сообщений в драйвере.


typedef struct {
    struct NS16550 *base; /* Базовый адрес адаптера */
    int IRQ;              /* Вектор прерыван. адаптера */
    int baud;             /* Скорость передачи */
    int nBit;             /* Число битов 5,6,7 или 8 */
    int nStop;            /* Число стоп-битов 0-1, 1-2 */
    int parity;           /* Четность 0-нет 1-есть */
    int pType;            /* Тип четности Чет/Нечет */
    int pFix;             /* Фиксац. четности 0-нет 1-да */
    int rqsize;           /* Размер очереди на прием */
    int tqsize;           /* Размер очереди на передачу */
    int r_timeout;        /* Таймаут на прием */
    int w_timeout;        /* Таймаут на выдачу */
    int opened;           /* Флаг того, что файл уже открыт */
#ifdef SIODRV_USE_MESSAGES
    MSG_Q_ID r_sio_msg;
    MSG_Q_ID w_sio_msg;
#else
    tRingBuffer *r_sio_msg;
    tRingBuffer *w_sio_msg;
#endif
    unsigned int readBufferOverflowCnt;    // Счётчик ошибок чтения вызванных переполнение принимающего буфера.
    int mode; /* Режим выдачи 0-binary 1-text */
    bool waitTxComplete;    // Дожидаться реального окончания отправки по линии передачи при записи данных, иначе дожидаться окончания записи в буфер.
    bool txProceed;         // В данный момент идёт передача байта по линии (исп. в прерывании передачи).
} serialInfo;

typedef serialInfo *SERIAL_INFO;



int SIOinit (SERIAL_INFO Inf);
int sioWrite (SERIAL_INFO dev, const char *buf, int count, int timeout);
int sioRead (SERIAL_INFO dev, char *buf, int count, int timeout);
int sioFlush (SERIAL_INFO dev);
int sioCancelWrite (SERIAL_INFO dev);
int sioWaitFor (SERIAL_INFO dev, char *s, int timeout);
int sioPutC (SERIAL_INFO dev, char C);
int sioGetC (SERIAL_INFO dev);
int sioPutS (SERIAL_INFO dev, char *s);
int sioGetS (SERIAL_INFO dev, char *s, int maxcount, int timeout);
int sioDrv ();

// int sioCheckWakeup (SERIAL_INFO Inf);


STATUS sioConCreate (int base, int irq, int baud);
STATUS __attribute__((deprecated)) sioDevCreate (char *name, int base, int irq, int baud); // Переходить на драйвер uart.h.
STATUS __attribute__((deprecated)) sioDevCreate_Extended (char *name, unsigned int base, int irq, int baud, int nBit, int nStop, int parity, int pFix, int pType, int rqsize, int tqsize); // Переходить на драйвер uart.h.

STATUS setCommState (int fd, SERIAL_INFO sio);
SERIAL_INFO getCommState (int fd);
int commFlush (int fd);
void setCommTextMode (int fd);
void setCommBinMode (int fd);
STATUS setCommTimeouts (int fd, int w_timeout, int r_timeout);
STATUS commReinit (int fd);
STATUS setCommBaudRate (int fd, int baud);

// Функции прямого доступа к регистрам настройки (для uart.h)
void sioWrRegBaud (const SERIAL_INFO Inf);
void sioWrRegNBit (const SERIAL_INFO Inf);
void sioWrRegStopBit (const SERIAL_INFO Inf);
void sioWrRegParity (const SERIAL_INFO Inf);
// Функции инициализации и контроля (для uart.h)
void sioInitBuffers (SERIAL_INFO Inf);
int sioRdBufferCounter (SERIAL_INFO Inf);
int sioWrBufferCounter (SERIAL_INFO Inf);
STATUS sioBufferFlush (SERIAL_INFO Inf);

#ifdef __cplusplus
}
#endif

#endif
