/**
 * @addtogroup groupCore
 * @{
 * @file sleep.h
 * @brief Различные обертки над функционалом приостановки задачи.
 * @}
 */

#ifndef SLEEP_H_
#define SLEEP_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>

//! @details Кол-во микросекунд в секунде.
#define MKSINSEC (1000L*1000L)

//! @details Кол-во миллисекунд в секунде.
#define MSINSEC (1000L)

/**
 * Подсчитать, сколько рабочих тиков ОС занимает ближайшее к nominator/denominator[секунд] время.
 * @param nominator Числитель в формуле.
 * @param denominator Знаменатель в формуле.
 * @return Соответствующее кол-во рабочих тиков ОС.
 */
int nodetick(int nominator, int denominator);

/**
 * Подсчитать, сколько рабочих тиков ОС занимает заданное кол-во 1/60 долей секунды.
 * @param secDiv60 Кол-во 1/60х долей секунды.
 * @return Соответствующее кол-во рабочих тиков ОС.
 */
int tick60(int secDiv60);

/**
 * Подсчитать, сколько рабочих тиков ОС занимает заданное кол-во микросекунд.
 * @param mks Кол-во микросекунд (не более 2х секунд).
 * @return Соответствующее кол-во рабочих тиков ОС.
 */
int tickmks(int mks);

/**
 * Подсчитать, сколько рабочих тиков ОС занимает заданное кол-во миллисекунд.
 * @param ms Кол-во миллисекунд (не более 35 минут).
 * @return Соответствующее кол-во рабочих тиков ОС.
 */
int tickms(int ms);

/**
 * Задержать задачу на ближайшее к nominator/denominator[секунд] время.
 * @details Задержка не блокирующая - в ее время будут исполняться другие задачи.
 * @param nominator Числитель в формуле.
 * @param denominator Знаменатель в формуле.
 * @return Всегда возвращает \ref OK.
 */
STATUS nodesleep(int nominator, int denominator);

/**
 * Задержать задачу на заданное кол-во миллисекунд.
 * @details Задержка не блокирующая - в ее время будут исполняться другие задачи.
 * @param secDiv60 Кол-во 1/60х долей секунды.
 * @return Всегда возвращает \ref OK.
 */
STATUS sleep60(int secDiv60);

/**
 * Задержать задачу на заданное кол-во микросекунд.
 * @details Задержка не блокирующая - в ее время будут исполняться другие задачи.
 * @param mks Кол-во микросекунд (не более 17 секунд).
 * @return Всегда возвращает \ref OK.
 */
STATUS sleepmks(int mks);

/**
 * Задержать задачу на заданное кол-во миллисекунд.
 * @details Задержка не блокирующая - в ее время будут исполняться другие задачи.
 * @param ms Кол-во миллисекунд (не более 17 секунд).
 * @return Всегда возвращает \ref OK.
 */
STATUS sleepms(int ms);


#ifdef __cplusplus
}
#endif
#endif // ifndef SLEEP_H_
