/**
    @file socket.h
    @brief Протокол \b TCP
    @details В файле описаны методы работы с сетевой подсистемой с использованием
    протокола \b TCP/IP.

    <b>Подключение</b>: @verbatim #include <socket.h> @endverbatim
    \ref sectionConfig "config.h": @verbatim #define INCLUDE_NETINET @endverbatim
    \ref sectionMakefile "Makefile": @verbatim LIBRARIES += -l_enet -l_socket @endverbatim

    @see Общее описание в главе \ref pageNetWorkSys.

    @page pageNetWorkSys
    @section sectionNetWorkTcp Протокол TCP/IP, сокеты TCP
    
    В отличии от \b UDP, \b TCP обеспечивает надежную доставку данных. При этом
    \b TCP обеспечивает как гарантию доставки данных, так и гарантию сохранения
    порядка следования сообщений. Для работы по протоколу \b TCP используются
    сокеты.
    
    Для передачи данных через сокеты можно пользоваться стандартными функциями
    ввода / вывода read() и write().

    @see Описание методов работы с протоколом \b TCP/IP в файле socket.h.

 */

#ifndef SOCKET_H_
#define SOCKET_H_

#ifdef __cplusplus
extern "C" {
#endif
#include <multex.h>
#include <ip.h>


#define INVALID_SOCKET (-1)


/**
 *  @name Коммуникационный домен 
 *  @brief Допустим домен \b Internet.
 *  @{
 */
#define AF_INET   2 
#define PF_INET   AF_INET
/// @}


/**
 *  @name Типы сокетов
 *  @{
 */
#define SOCK_RAW       0
#define SOCK_STREAM    0x1000
#define SOCK_DGRAM     0x2000
#define SOCK_TYPE_MASK 0x3000
/// @}


/**
 *  @name Значения параметра flags для send, receive и т.п.
 *  @{
 */
#define MSG_OOB	        0x1     ///< @details Process out-of-band data.
#define MSG_PEEK	    0x2     ///< @details Peek at incoming message.
#define MSG_DONTROUTE   0x4     ///< @details Bypass routing, use direct interface.
#define MSG_EOR	        0x8     ///< @details Data completes record.
#define MSG_EOF	        0x100   ///< @details Data completes transaction.
/// @}

#define INADDR_ANY    {-1}
#define inet_addr(x)  strtoip(x)

typedef unsigned short sa_family_t;

/** Обобщенная структура адреса сокета. */
struct sockaddr {
        sa_family_t sa_family; ///< @details Семейство адресов.
		UCHAR sa_data[14];	   ///< @details Адрес сокета.
		};

/** Структура адреса \b Internet. */
struct in_addr {
        	    UINT s_addr;
			   };


/**
 *  Порт в сети (16 бит).
 */
typedef unsigned short in_port_t;
		

/**
 *  @brief Структура адреса сокета для связи через сеть.
 *  @details Структура адреса сокета для связи через сеть.
 *  @warning В отличие от стандарта параметр \b sin_port
 *  в структуре \ref sockaddr_in требуется задавать в формате
 *  \b LITTLE \b ENDIAN, т.е. не требуется использовать макрос
 *  \b htons при его задании.
 *  
 */
struct sockaddr_in {
        	    sa_family_t  sin_family;    ///< Семейство адресов.
        	    in_port_t 	 sin_port; 		///< Номер порта.
        	    struct  in_addr sin_addr;   ///< IP - адрес.
        	    unsigned char sin_zero[8];	///< Поле выравнивания.
			   };

			    
/**
 *  @brief Создание сокета.
 *  @details Функция создает сокет.
 *  
 *  @param domain Нужно указывать \ref AF_INET.
 *  @param type Нужно указывать \ref SOCK_STREAM.
 *  @param protocol Нужно указывать \b 0 — значение 
 *  по умолчанию для данного соединения.
 *  @return Возвращает дескриптор сокета, 
 *  как стандартного устройства ввода / вывода.
 */
int socket (int domain,int type,int protocol);

/**
 *  @brief Связывание сокета с адресом.
 *  @details Связывает сокет с конкретным адресом.
 *  Когда сокет создается при помощи socket(),
 *  он ассоциируется с некоторым семейством адресов,
 *  но не с конкретным адресом. До того как сокет сможет 
 *  принять входящие соединения, он должен быть связан с адресом.
 *  
 *  @param sockfd Дескриптор сокета.
 *  @param address Указатель на структуру \ref sockaddr,
 *  представляющую адрес, к которому привязываем.
 *  @param add_len Длина структуры адреса.
 *  @return \ref OK при успехе.
 *  @return \ref ERROR при неудаче.
 */
int bind (int sockfd, const struct sockaddr *address, size_t add_len);

/**
 *  @brief Включение приема \b TCP-соединения.
 *  @details Подготавливает привязываемый сокет к принятию входящих соединений.
 *  
 *  @param sockfd Дескриптор сокета.
 *  @param queue_size Число установленных соединений, которые могут быть обработаны единовременно.
 *  @return \ref OK при успехе.
 *  @return \ref ERROR при неудаче.
 */
int listen (int sockfd, int queue_size);

/**
 *  @brief Приём запроса на установку \b TCP-соединения.
 *  @details Процедура подтверждает запрос на установление
 *  соединения от удаленного хоста.
 *  
 *  @param sockfd Дескриптор сокета.
 *  @param address Указатель на структуру \ref sockaddr 
 *  для получения информации об адресе клиента.
 *  @param add_len Указатель на переменную, 
 *  содержащую размер структуры адреса.
 *  @return Функция возвращает дескриптор сокета,
 *  связанного с этим конкретным соединением, кроме того,
 *  в переменную \b add_len записывается фактический размер адреса.
 */
int accept (int sockfd, struct sockaddr *address, size_t *add_len);

/**
 *  @brief Подключение клиента.
 *  @details Функция устанавливает соединение с сервером.
 *  
 *  @param sockfd Дескриптор сокета.
 *  @param address Указатель на структуру адреса.
 *  @param add_len Размер структуры адреса.
 *  @return \ref OK
 *  @return Код ошибки при неудаче.
 */
int connect (int sockfd, const struct sockaddr *address, size_t add_len);


/// @privatesection
// Пересылка данных	(TCP-соединение)
size_t recv (int sockfd, void *buffer, size_t length, int flags);
size_t send (int sockfd, void *buffer, size_t length, int flags);

// Пересылка данных	(UDP-соединение)
size_t recvfrom (int sockfd, void *message, size_t length, int flags,
                 struct sockaddr *send_addr, int *add_len);

// Doesn't actually work!
size_t sendto (int sockfd, const void *message, size_t length, int flags,
               const struct sockaddr *dest_addr, int add_len);

/// @publicsection

/**
 *  @brief Прекращение обмена сокетом.
 *  @details Функция прекращает обмен для указанного сокета.
 *  
 *  @param sockfd Дескриптор сокета.
 *  @param how Что именно прекратить - значение из 
 *  группы \ref ancSocketShutdownHow "значений":
 *  - \ref SD_RECEIVE
 *  - \ref SD_SEND
 *  - \ref SD_BOTH
 *  @return \ref OK при успехе.
 *  @return \ref ERROR при неудаче.
 */
int shutdown (int sockfd, int how);

/**
 *  @anchor ancSocketShutdownHow
 *  @name Значения параметра how функции shutdown().
 *  @{
 */
#define SD_RECEIVE 1 ///< @details Прекратить прием данных сокетом.
#define SD_SEND    2 ///< @details Прекратить выдачу данных сокетом.
#define SD_BOTH    3 ///< @details Прекратить обмен в обе стороны.
/// @}

/**
 *  @brief Задать таймаут сокета.
 *  @details Функция устанавливает таймаут для сокета.
 *  
 *  @param sockfd Дескриптор сокета.
 *  @param timeout Таймаут в тиках таймера.
 *  @return \ref OK при успешном завершении.
 *  @return \ref ERROR при неудаче.
 */
int setsocktimeout (int sockfd, int timeout);

/**
 *  @brief Получить таймаут сокета.
 *  @details Функция возвращает таймаут, установленный для сокета.
 *  
 *  @param sockfd Дескриптор сокета.
 *  @return Функция возвращает таймаут, установленный для данного сокета.
 */
int getsocktimeout (int sockfd);

#ifdef __cplusplus
}
#endif
#endif // ifndef SOCKET_H_
