/**
    @addtogroup groupDrivers
    @{
    @file spi.h
    @brief Интерфейс SPI.
    @details Настройка аппаратного модуля \b SPI.

    <b>Подключение</b>: @verbatim #include <spi.h> @endverbatim
    
    @see Общее описание работы с интерфейсом \b SPI в разделе \ref sectionSpi.
    @}

    @page pageDrivers

    @section sectionSpi SPI
    @see Описание методов работы с интерфейсом \b SPI в файле spi.h.

    Шина \b SPI - последовательный синхронный интерфейс передачи данных в режиме
    полного дуплекса, предназначенный для обеспечения простого
    высокоскоростного сопряжения микроконтроллеров и периферии. \b SPI также
    иногда называют четырёхпроводным (four-wire) интерфейсом. 

    \b SPI является синхронным интерфейсом, в котором любая передача
    синхронизирована с общим тактовым сигналом, генерируемым ведущим
    устройством (процессором). Принимающая (ведомая) периферия синхронизирует
    получение битовой последовательности с тактовым сигналом. К одному
    последовательному периферийному интерфейсу ведущего устройства может
    присоединяться несколько ведомых. Ведущее устройство выбирает ведомое для
    передачи, активируя сигнал \b CS (Chip Select) на ведомой микросхеме.
    Периферия, не выбранная процессором, не принимает участия в передаче по
    \b SPI. В процессорах \em Allwinner может содержаться до четырёх
    аппаратных интерфейса \b SPI и для каждого из них предусмотрено до
    четырёх сигналов \b CS. Число линий \b CS может быть увеличено путём
    программного использования линий ввода/вывода общего назначения \b GPIO.

    Интерфейс \b SPI использует следующие сигналы:
        - \b SCLK -- тактовый сигнал (от ведущего к ведомому);
        - \b MOSI -- данные, передаваемые (от ведущего к ведомому);
        - \b MISO -- принимаемые данные (от ведомого к ведущему);
        - \b CSn  -- выбор абонента (от ведущего к выбираемому ведомому).
    
    В @osName реализован драйвер, позволяющий работать с несколькими
    интерфейсами \b SPI в режиме ведущего устройства (\b MASTER).
    Для подключения одного из имеющихся интерфейсов \b SPI в @osName нужно
    инициализировать нужный модуль с помощью spiInit() и затем вызывать
    функцию spiTransfer() для запуска цикла обмена данными с выбранным
    устройством. Для подключения аппаратных линий \b CS следует использовать
    функцию инициализации spiInitChipSelectLine() для каждой используемой
    линии. При подключении выбранного аппаратного модуля \b SPI будут
    использованы настройки интерфейса по умолчанию. Для изменения настроек
    следует воспользоваться функциями из соответствующей \ref ancSpiSettings
    "группы". Изменять данные настройки можно перед каждым циклом обмена
    данными с устройством. Это может понадобится, если к одному интерфейсу
    подключены устройства с разными параметрами передачи данных.

    Ниже приведён пример инициализации интерфейса \b SPI0 и получение данных от
    подключенного устройства. При инициализации выбирается основной набор
    линий ввода/вывода и подключается линия выбора ведомого устройства \b
    CS0. Тактовая частота \b CLK устанавливается равной 4 МГц. Остальные
    настройки используются по умолчанию. Далее выполняется чтение из
    устройства по адресу \b 0x10 (это частный случай определённый протоколом
    обмена конкретного устройства). Для этого в первый байт массива данных
    кладётся нужный адрес и задаётся длина значащих передаваемых байт равная
    единице. Остальные передаваемые данные будут иметь нулевое значение.
    Такое поведение задаётся аппаратным модулем \b SPI. Общее количество
    обменов задаётся равным трём. После получения первого байта ведомое
    устройство начнёт передачу данных, таким образом прочитанное значение
    после окончания обмена будет лежать в последних двух байтах массива.

    @code
    spiInit (SPI_0, SPI_GPIO_DEFAULT);
    spiSetClkFrequency (SPI_0, 4000000);
    spiInitChipSelectLine (SPI_0, SPI_GPIO_DEFAULT, SPI_CS_0);

    // read from 0x10
    char data[3];
    data[0] = 0x10;
    spiTransfer (SPI_0, SPI_CS_0, data, 1, 3);
    @endcode

 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 22.06.2023
*/

/*
Header file: spi.h
*/
#ifndef _SPI_H_
#define _SPI_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>


/**
 *  @anchor ancSpiDefines
 *  @name Макросы выбора каналов SPI
 *  @{
 */

#define SPI_0   0 ///< @details Индекс для \b SPI0.
#define SPI_1   1 ///< @details Индекс для \b SPI1.
#define SPI_2   2 ///< @details Индекс для \b SPI2.
#define SPI_3   3 ///< @details Индекс для \b SPI3.
/// @}

/**
 *  @anchor ancSpiCsDefines
 *  @name Макросы выбора линий Chip Select
 *  @{
 */
#define SPI_CS_0   0 ///< @details Номер линии Chip Select \b 0.
#define SPI_CS_1   1 ///< @details Номер линии Chip Select \b 1.
/// @}


/**
 *  @anchor ancSpiGpioDefines
 *  @name Макросы выбора набора выходных линий SPI
 *  @{
 */

#define SPI_GPIO_DEFAULT    0 ///< @details Выбор основного набора линий модуля SPI (обычно расположен в младших портах \b GPIO).
#define SPI_GPIO_ADDITIONAL 1 ///< @details Выбор дополнительного набора линий модуля SPI (обычно расположен в старших портах \b GPIO).

/// @}

/**
 *  @anchor ancSpiSettings
 *  @name Настройка параметров передачи данных
 *  @brief Параметры передачи можно изменять как до инициализации модуля, так и после.
 *  Также параметры передачи можно изменять между сеансами обмена с устройствами.
 *  @{
 */

/**
 *  @brief Выбор порядка следования бит.
 *  @details Функция задаёт порядок следования бит при передаче по шине \b SPI.
 *  Параметр можно изменять в паузах между передачами
 *  разным устройствам. Если данный параметр не изменяется в ходе выполнения
 *  программы и не отличается от значения по умолчанию -- данную функцию
 *  вызывать не обязательно.
 *  
 *  @param n Номер модуля \b SPI рекомендуется брать из соответствующей
 *  группы \ref ancSpiDefines "макросов".
 *  @param enable \ref true -- младший бит передаётся первым, 
 *  иначе первым передаётся старший бит (значение по умолчанию).
 *  @return \ref OK в случае успешного выполнения настройки, иначе \ref ERROR.
 */
STATUS spiSetBitOrderLsbFirst (unsigned int n, bool enable);

/**
 *  @brief Задать начальный уровень сигнала \b Chip \b Select.
 *  @details Функция задаёт начальный уровень сигнала \b CS на шине \b SPI.
 *  Настройка относится ко всем имеющимся в выбранном модуле линиям \b CS.
 *  Параметр можно изменять в паузах между передачами
 *  разным устройствам. Если данный параметр не изменяется в ходе выполнения
 *  программы и не отличается от значения по умолчанию -- данную функцию
 *  вызывать не обязательно.
 *  
 *  @param n Номер модуля \b SPI рекомендуется брать из соответствующей
 *  группы \ref ancSpiDefines "макросов".
 *  @param enable \ref true -- высокий начальный уровень сигнала \b CS, 
 *  иначе начальным является низкий уровень (значение по умолчанию).
 *  @return \ref OK в случае успешного выполнения настройки, иначе \ref ERROR.
 */
STATUS spiSetCsInitialHigh (unsigned int n, bool enable);

/**
 *  @brief Задать начальный уровень сигнала \b CLK.
 *  @details Функция задаёт начальный уровень сигнала \b CLK на шине \b SPI.
 *  Параметр можно изменять в паузах между передачами
 *  разным устройствам. Если данный параметр не изменяется в ходе выполнения
 *  программы и не отличается от значения по умолчанию -- данную функцию
 *  вызывать не обязательно.
 *  
 *  @param n Номер модуля \b SPI рекомендуется брать из соответствующей
 *  группы \ref ancSpiDefines "макросов".
 *  @param enable \ref true -- высокий начальный уровень сигнала \b CLK, 
 *  иначе начальным является низкий уровень (значение по умолчанию).
 *  @return \ref OK в случае успешного выполнения настройки, иначе \ref ERROR.
 */
STATUS spiSetClkInitialHigh (unsigned int n, bool enable);

/**
 *  @brief Задать фронт фиксации данных сигнала \b CLK на шине \b SPI.
 *  @details Функция задаёт фронт сигнала \b CLK, по которому будут считываться данные.
 *  Параметр можно изменять в паузах между передачами
 *  разным устройствам. Если данный параметр не изменяется в ходе выполнения
 *  программы и не отличается от значения по умолчанию -- данную функцию
 *  вызывать не обязательно.
 *  
 *  @param n Номер модуля \b SPI рекомендуется брать из соответствующей
 *  группы \ref ancSpiDefines "макросов".
 *  @param enable \ref true -- фиксация данных происходит по заднему фронту сигнала \b CLK, 
 *  иначе факсация по переднему фронту (значение по умолчанию).
 *  @return \ref OK в случае успешного выполнения настройки, иначе \ref ERROR.
 */
STATUS spiSetClkTrailingEdge (unsigned int n, bool enable);

/**
 *  @brief Задать частоту тактового сигнала \b CLK на шине \b SPI.
 *  @details Функция задаёт частоту тактового сигнала \b CLK. 
 *  Реальное значение тактовой частоты будет совпадать или максимально приближаться
 *  к заданному значению.
 *  Параметр можно изменять в паузах между передачами
 *  разным устройствам. Если данный параметр не изменяется в ходе выполнения
 *  программы и не отличается от значения по умолчанию -- данную функцию
 *  вызывать не обязательно.
 *  
 *  @param n Номер модуля \b SPI рекомендуется брать из соответствующей
 *  группы \ref ancSpiDefines "макросов".
 *  @param f Тактовая частота сигнала \b CLK в герцах. Значение по умолчанию -- 1 МГц.
 *  @return \ref OK в случае успешного выполнения настройки, иначе \ref ERROR.
 */
STATUS spiSetClkFrequency (unsigned int n, unsigned int f);


/// @}



/**
 *  @name Инициализация аппаратного модуля SPI
 *  @{
 */

/**
 *  @brief Задействовать аппаратное управление линий \b CS.
 *  @details Функция подключает заданную линию \b Chip \b Select
 *  к аппаратному модулю \b SPI. К одному модулю могут быть подключены
 *  несколько линий \b CS, в этом случае следует вызвать функцию для каждой
 *  используемой линии. Далее управление линией происходит
 *  аппаратно при вызове spiTransfer(). При использовании программного
 *  управления выбором устройств данную функцию вызывать не нужно.
 *  
 *  @param n Номер модуля \b SPI рекомендуется брать из соответствующей
 *  группы \ref ancSpiDefines "макросов".
 *  @param gpion Номер набора портов ввода/вывода. Рекомендуется использовать
 *  значение из группы \ref ancSpiGpioDefines "макросов".
 *  @param csn Номер линии \b Chip \b Select, если такая линия 
 *  задана с помощью spiInitChipSelectLine(). Рекомендуется использовать
 *  значение из группы \ref ancSpiCsDefines "макросов".
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS spiInitChipSelectLine (unsigned int n, unsigned int gpion, unsigned int csn);


/**
 *  @brief Инициализация аппаратного модуля \b SPI.
 *  @details Функция инициализации аппаратного модуля \b SPI 
 *  настраивает запускает аппаратный модуль.
 *  
 *  @param n Номер модуля \b SPI рекомендуется брать из соответствующей
 *  группы \ref ancSpiDefines "макросов".
 *  @param gpion Номер набора портов ввода/вывода. Рекомендуется использовать
 *  значение из группы \ref ancSpiGpioDefines "макросов".
 *  @return \ref OK если настройка прошла успешно, иначе \ref ERROR.
 */
STATUS spiInit (unsigned int n, unsigned int gpion);


/// @}

/**
 *  @name Обмен данными по шине SPI
 *  @{
 */


/**
 *  @brief Запуск обмена по шине данных \b SPI.
 *  @details Функция запускает побайтовый обмен данными по шине \b SPI
 *  и ожидает его окончания. Во время ожидания 
 *  управление передаётся другим задачам.
 *  
 *  
 *  @param n Номер модуля \b SPI рекомендуется брать из соответствующей
 *  группы \ref ancSpiDefines "макросов".
 *  @param csn Номер линии \b Chip \b Select, если такая линия 
 *  задана с помощью spiInitChipSelectLine(). Рекомендуется использовать
 *  значение из группы \ref ancSpiCsDefines "макросов".
 *  Если используется программное управление линиями выбора ведомых устройств, то 
 *  данный параметр будет проигнорирован (в этом случае следует использовать \b csn=0).
 *  @param data Указатель на буфер обмен данными. Перед началом отправки буфер
 *  должен содержать отправляемые данные, если таковые имеются. После окончания
 *  отправки в буфере будут хранится данные полученные от выбранного устройства.
 *  Размер буфера должен быть достаточным для приёма запрашиваемого количества данных.
 *  @param txLength Количество байт для передачи. Если количество передаваемых
 *  байт меньше, чем общее количество обменов по шине, недостающие байты при передаче
 *  будут заполнены значениями по умолчанию (нулями). Количество передаваемых байт
 *  не может превышать общее количество обменов \b totalLength.
 *  @param totalLength Общее количество обменов байтами по шине. Данное значение
 *  должно быть меньше, чем 64 байта.
 *  @return \ref OK если обмен данными прошёл успешно, иначе \ref ERROR.
 */
STATUS spiTransfer (unsigned int n, unsigned int csn, void *data, unsigned int txLength, unsigned int totalLength);

/// @}


/// @privatesection


/**
 *  @brief Инициализация модуля.
 *  @details Считывание информации об используемом
 *  процессоре. Функция используется системой --- пользователю
 *  вызывать не нужно.
 *  
 *  @return \ref OK в случае успешного выполнения, иначе \ref ERROR.
 */
STATUS spiArchInit ();

// ==================================================================
// DEPRECATED SECTION


/* Clock gating */
#include <semlib.h>

/// @deprecated Устаревшая версия драйвера будет удалена.
typedef struct spi_dev {
    int spi_num;
    unsigned int base;
    unsigned int param;
    int irq;
    int result;
    unsigned int clk;
    SEM_ID sem;
    unsigned char *buf;
    unsigned char *wbuf;
    unsigned char *ptr;
    unsigned char *wptr;
    unsigned int wcnt;
    unsigned int wlen;
    unsigned int len;
    unsigned int count;
} spi_dev;

/// @deprecated Устаревшая версия драйвера будет удалена.
spi_dev __attribute__((deprecated)) *spi_init (int spi_num, int clk);
/// @deprecated Устаревшая версия драйвера будет удалена.
int __attribute__((deprecated)) spi_xfer (spi_dev *dev, int chipSelect, char *buf, int wc, int rc);
/// @deprecated Устаревшая версия драйвера будет удалена.
void __attribute__((deprecated)) spi_set_clk (spi_dev *dev, int clk);

/**
 * SPI bus configuration, that should be initialized.
 * Naming convention: SC_[socName]_[busNumber]_[portConfiguration].
 * @deprecated Устаревшая версия драйвера будет удалена.
 */
typedef enum {
    SC_A20_SPI0_PC,
    SC_A20_SPI0_PI,
    SC_A20_SPI1_PA,
    SC_A20_SPI1_PI,
    SC_A20_SPI2_PB,
    SC_A20_SPI2_PC,
    SC_A20_SPI3_PA,

    // SC_V3s_SPI0_PC, //! Same as H3 SPI0.
    //
    // SC_H3_SPI0_PC,  //! Same as V3s SPI0.
    // SC_H3_SPI1_PA,
} eSpiConfiguration;

/**
 * Initialize SPI bus for certain hardware configuration.
 * @deprecated Устаревшая версия драйвера будет удалена.
 * @param spiConfig Configuration type.
 * @param clk SPI frequency.
 * @return Pointer to SPI bus descriptor or NULL in case of error.
 */
spi_dev __attribute__((deprecated)) *spi_init_extended (eSpiConfiguration spiConfig, int clk);

/**
 * SPI bus clock phase and polarity combination.
 * @deprecated Устаревшая версия драйвера будет удалена.
 */
typedef enum {
    SM_RisingSample_FallingSetup = 0,
    SM_RisingSetup_FallingSample = 1,
    SM_FallingSample_RisingSetup = 2,
    SM_FallingSetup_RisingSample = 3,
} eSpiMode;

/**
 * Set clock and polarity mode for SPI.
 * @deprecated Устаревшая версия драйвера будет удалена.
 * @param dev SPI bus descriptor.
 * @param mode Mode type.
 * @return OK if succeded, ERROR otherwise.
 */
STATUS __attribute__((deprecated)) spi_set_mode (spi_dev *dev, eSpiMode mode);

/**
 * @deprecated Устаревшая версия драйвера будет удалена.
 * SPI chip select line logic.
 */
typedef enum {
    SCSM_ActiveHigh = 0,  //!< Chip-select will be activated with high polarity (0 == idle).
    SCSM_ActiveLow = 1,   //!< Chip-select will be activated with low polarity (1 == idle).
} eSpiChipSelectMode;

/**
 * Set polarity mode for SPI's chip select line.
 * @deprecated Устаревшая версия драйвера будет удалена.
 * @param dev SPI bus descriptor.
 * @param mode Chip select mode type.
 * @return OK if succeded, ERROR otherwise.
 */
STATUS __attribute__((deprecated)) spi_set_cs_mode (spi_dev *dev, eSpiChipSelectMode mode);

#ifdef __cplusplus
}
#endif
#endif  // ifndef _SPI_H_
