/**
 * @addtogroup groupStdTypes
 * @{
 * @file stdio.h
 * @brief Стандартные функции ввода-вывода.
 * @details См. стандарт C11 7.21.
 * @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.21</a>.
 * @}
 */

/*
 * @date Created on: unknown.
 * @author Sergey Sokolov
 * @author Dubovich Timofey
 */


#ifndef _STDIO
#define _STDIO

#ifdef __cplusplus
extern "C" {
#endif

#include "stddef.h"     // size_t, NULL
#include "stdarg.h"     // va_list
#include "filesyst.h"   // Backward compatibility
#include "inputstr.h"   // Backward compatibility



/**
 *  @brief Сигнатура корректной структуры FILE.
 */
#define FILE_SIGNATURE  0xF12EF12E

/**
 * @brief Структура файла на блочном устройстве.
 */
typedef struct
{
    int signa;        //!< Сигнатура файла.
    int fd;           //!< Дескриптор файла.
    char unget;       //!< Буфер ungetc.
    int ugf;          //!< Флаг ungetc.
} FILE;

/**
 * @brief Тип, позволяющий однозначно указать позицию в файле.
 */
typedef long int fpos_t;

/**
 * @brief Значения для 3го аргумента функции setvbuf.
 * @{
 */
//! Полная/блочная буферизация.
#define _IOFBF 0
//! Линейная/строчная буферизация.
#define _IOLBF 1
//! Отключенная буферизация.
#define _IONBF 2
/**
 * @}
 */

/**
 * @brief Размер буфера setbuf по-умолчанию.
 */
#define BUFSIZ (8192)

/**
 * @brief Значение, индицирующее ситуацию 'конец файла' для потока.
 */
#define EOF (-1)

/**
 * @brief Минимальное количество файлов, которое гарантированно возможно иметь одновременно открытыми.
 */
#define FOPEN_MAX (16)

/**
 * @brief Максимально допустимая длинна строки, которая разрешена системой для определения имени файла.
 */
#define FILENAME_MAX (256) // TODO

/**
 *@brief Длина имен временных файлов, создаваемых tmpnam.
 */
#define L_tmpnam (128)

/**
 * @brief Максимальное число уникальных имен файлов, которые может создать функция tmpnam.
 */
#define TMP_MAX (10)

/**
 * @brief Значения для третьего аргумента фунции fseek, относительно чего нужно сдвигать позицию чтения/записи в файле.
 * @{
 */
//! @brief Позицию нужно сдвигать относительно начала файла.
#define SEEK_SET 0
//! @brief Позицию нужно сдвигать относительно текущей позиции в файле.
#define SEEK_CUR 1
//! @brief Позицию нужно сдвигать относительно конца файла.
#define SEEK_END 2
/**
 * @}
 */

/**
 * @brief Стандартные потоки.
 * @{
 */
//! @brief Стандартный поток ввода.
extern FILE *stdin;
//! @brief Стандартный поток вывода.
extern FILE *stdout;
//! @brief Стандартный поток вывода сообщений об ошибках.
extern FILE *stderr;
/**
 * @}
 */


//--------------------------------------------------
/**
 * @name Функции операций над файлами
 * @{
 */

/**
 * Удалить файл.
 * @param filename Имя файла.
 * @return 0 при успехе, не-0 иначе.
 */
int remove(const char *filename);

/**
 * Переименовать файл.
 * @param oldName Старое имя файла.
 * @param newName Новое имя файла.
 * @return 0 при успехе, не-0 иначе.
 */
int rename(const char *oldName, const char *newName);

// /**
//  * Создать временный бинарный файл.
//  * @return Указатель на временный файл или NULL, если файл не удалось создать.
//  */
// FILE *tmpfile(void);
//
// /**
//  * Создать уникальное имя файла.
//  * @param s Буфер размера минимум L_tmpnam, в который будет записан результат, или NULL.
//  * @return Указатель на s, если s - не-NULL, или указатель на внутренний буфер, или NULL при ошибке.
//  */
// char* tmpnam(char *s);

//! @}


//--------------------------------------------------
/**
 * @name Функции доступа к файлам
 * @{
 */

/**
 * Закрыть поток, ранее открытый с помощью fopen или freopen.
 * @param stream Поток, который требуется закрыть.
 * @return 0 при успехе, EOF иначе.
 * @details Функция также вызовет принудительную буферизацию данных.
 */
int fclose(FILE *stream);

/**
 * Принудительно буферизировать данные, не закрывая поток.
 * @param stream Поток.
 * @return 0 при успехе, EOF иначе.
 */
int fflush(FILE *stream);

/**
 * Открыть файл и связать его с потоком.
 * @param filename Имя файла.
 * @param mode Строка, описывающая режим открытия (r, r+, w, w+, a, a+).
 * @return Указатель на открытый поток или NULL при ошибке.
 */
FILE *fopen(const char * restrict filename, const char * restrict mode);

/**
 * Связать с потоком открытый файл.
 * @param fd Дескриптор открытого файла.
 * @param mode Строка, описывающая режим открытия (r, r+, w, w+, a, a+). Режим должен быть совместим с режимом открытого файла.
 * @return Указатель на открытый поток или NULL при ошибке.
 */
FILE *fdopen(int fd, const char *mode);

/**
 * Открыть файл и связать его с существующим потоком.
 * @param filename Имя файла.
 * @param mode Строка, описывающая режим открытия (r, r+, w, w+, a, a+).
 * @param stream Существующий поток. Файл, связанный с этим потоком будет закрыт.
 * @return Указатель на открытый поток или NULL при ошибке.
 */
FILE *freopen(const char * restrict filename, const char * restrict mode, FILE * restrict stream);

/**
 * Не используется: Установить буфер для буферизации потока.
 * @param stream Поток.
 * @param buf Указатель на буфер размера минимум BUFSIZ для включения блочной буферизации, или NULL для отключения буферизации.
 * @warning Фактически всегда используется буферизация по-умолчанию (блочная для файлов, отключенная для терминала, по-разному для устройств).
 */
void setbuf(FILE * restrict stream, char * restrict buf);

/**
 * Не используется: Изменить тип буферизации для потока и установить новый буфер.
 * @param stream Поток.
 * @param buf Буфер для буферизации.
 * @param mode Режим буферизации.
 * @param size Размер буфера.
 * @return 0 при успехе, не-0 иначе.
 * @warning Фактически всегда используется буферизация по-умолчанию (блочная для файлов, отключенная для терминала, по-разному для устройств).
 */
int setvbuf(FILE * restrict stream, char * restrict buf, int mode, size_t size);

//! @}


//--------------------------------------------------
/**
 * @name Функции форматированного ввода-вывода
 * @{
 */

/**
 * Вывести текст в stdout в соответствии с форматной строкой.
 * @param format Форматная строка.
 * @param ... Аргументы для форматной строки.
 * @return Кол-во фактически напечатанных символов, не включая нуль-терминатор.
 */
int printf(const char * restrict format, ...)        __attribute__ ((format (printf, 1, 2)));

/**
 * Вывести текст в поток в соответствии с форматной строкой.
 * @param stream Поток вывода.
 * @param format Форматная строка.
 * @param ... Аргументы для форматной строки.
 * @return Кол-во фактически напечатанных символов, не включая нуль-терминатор.
 */
int fprintf(FILE * restrict stream, const char * restrict format, ...)      __attribute__ ((format (printf, 2, 3)));

/**
 * Вывести текст в буфер в соответствии с форматной строкой.
 * @param s Буфер вывода.
 * @param format Форматная строка.
 * @param ... Аргументы для форматной строки.
 * @return Кол-во фактически напечатанных символов, не включая нуль-терминатор.
 */
int sprintf(char * restrict s, const char * restrict format, ...)        __attribute__ ((format (printf, 2, 3)));

/**
 * Вывести текст в буфер в соответствии с форматной строкой.
 * @param s Буфер вывода.
 * @param n Максимальное количество символов, которое можно вывести в буфер вывода (включая нуль-терминатор).
 * @param format Форматная строка.
 * @param ... Аргументы для форматной строки.
 * @return Кол-во фактически напечатанных символов, не включая нуль-терминатор.
 */
int snprintf(char * restrict s, size_t n, const char * restrict format, ...)         __attribute__ ((format (printf, 3, 4)));

/**
 * Вывести текст в stdout в соответствии с форматной строкой и используя va_list вместо переменного количества аргументов.
 * @param format Форматная строка.
 * @param arg Аргументы для форматной строки.
 * @return Кол-во фактически напечатанных символов, не включая нуль-терминатор.
 */
int vprintf(const char * restrict format, va_list arg)       __attribute__ ((format (printf, 1, 0)));

/**
 * Вывести текст в поток в соответствии с форматной строкой и используя va_list вместо переменного количества аргументов.
 * @param stream Поток вывода.
 * @param format Форматная строка.
 * @param arg Аргументы для форматной строки.
 * @return Кол-во фактически напечатанных символов, не включая нуль-терминатор.
 */
int vfprintf(FILE * restrict stream, const char * restrict format, va_list arg)      __attribute__ ((format (printf, 2, 0)));

/**
 * Вывести текст в буфер в соответствии с форматной строкой и используя va_list вместо переменного количества аргументов.
 * @param s Буфер вывода.
 * @param format Форматная строка.
 * @param arg Аргументы для форматной строки.
 * @return Кол-во фактически напечатанных символов, не включая нуль-терминатор.
 */
int vsprintf(char * restrict s, const char * restrict format, va_list arg)       __attribute__ ((format (printf, 2, 0)));

/**
 * Вывести текст в буфер в соответствии с форматной строкой и используя va_list вместо переменного количества аргументов.
 * @param s Буфер вывода.
 * @param n Максимальное количество символов, которое можно вывести в буфер вывода (включая нуль-терминатор).
 * @param format Форматная строка.
 * @param arg Аргументы для форматной строки.
 * @return Кол-во фактически напечатанных символов, не включая нуль-терминатор.
 */
int vsnprintf(char * restrict s, size_t n, const char * restrict format, va_list arg)        __attribute__ ((format (printf, 3, 0)));

/**
 * Считать информацию из stdin в соответствии с форматной строкой.
 * @param format Форматная строка.
 * @param ... Аргументы для форматной строки.
 * @return Кол-во считанных элементов или EOF при ошибке.
 */
int scanf(const char * restrict format, ...)        __attribute__ ((format (scanf, 1, 2)));

/**
 * Считать информацию из потока в соответствии с форматной строкой.
 * @param stream Поток.
 * @param format Форматная строка.
 * @param ... Аргументы для форматной строки.
 * @return Кол-во считанных элементов или EOF при ошибке.
 */
int fscanf(FILE * restrict stream, const char * restrict format, ...)        __attribute__ ((format (scanf, 2, 3)));

/**
 * Считать информацию из текстового буфера в соответствии с форматной строкой.
 * @param s Текстовый буфер.
 * @param format Форматная строка.
 * @param ... Аргументы для форматной строки.
 * @return Кол-во считанных элементов или EOF при ошибке.
 */
int sscanf(const char * restrict s, const char * restrict format, ...)        __attribute__ ((format (scanf, 2, 3)));

/**
 * Считать информацию из stdin в соответствии с форматной строкой и используя va_list вместо переменного количества аргументов.
 * @param format Форматная строка.
 * @param ... Аргументы для форматной строки.
 * @return Кол-во считанных элементов или EOF при ошибке.
 */
int vscanf(const char * restrict format, va_list arg)        __attribute__ ((format (scanf, 1, 0)));

/**
 * Считать информацию из потока в соответствии с форматной строкой и используя va_list вместо переменного количества аргументов.
 * @param stream Поток.
 * @param format Форматная строка.
 * @param ... Аргументы для форматной строки.
 * @return Кол-во считанных элементов или EOF при ошибке.
 */
int vfscanf(FILE * restrict stream, const char * restrict format, va_list arg)        __attribute__ ((format (scanf, 2, 0)));

/**
 * Считать информацию из текстового буфера в соответствии с форматной строкой и используя va_list вместо переменного количества аргументов.
 * @param s Текстовый буфер.
 * @param format Форматная строка.
 * @param ... Аргументы для форматной строки.
 * @return Кол-во считанных элементов или EOF при ошибке.
 */
int vsscanf(const char * restrict s, const char * restrict format, va_list arg)        __attribute__ ((format (scanf, 2, 0)));

//! @}

//--------------------------------------------------
/**
 * @name Функции ввода-вывода символов
 * @{
 */

/**
 * Считать символ из потока.
 * @param stream Поток.
 * @return Символ, приведенный к int, или EOF при достижении конца файла или при ошибке.
 */
int fgetc(FILE *stream);

/**
 * Считать символ из потока.
 * @param stream Поток.
 * @return Символ, приведенный к int, или EOF при достижении конца файла или при ошибке.
 */
int getc(FILE *stream);

/**
 * Считать символ из stdin.
 * @return Символ, приведенный к int, или EOF при достижении конца файла или при ошибке.
 */
int getchar(void);

/**
 * Считать строку из stdin и записать ее в буфер.
 * @param s Буфер для записи, переполнение буфера не проверяется.
 * @return s при успехе, NULL при ошибке.
 */
char* gets(char* s);

/**
 * Считать строку из потока и записать ее в буфер, нуль-терминировать итоговую строку.
 * @param s Буфер для записи.
 * @param n Размер буфера.
 * @param stream Поток.
 * @return s при успехе, NULL при ошибке или достижении конца файла/потока.
 */
char *fgets(char * restrict s, int n, FILE * restrict stream);


/**
 * Вывести символ в поток.
 * @param c Символ, приведенный к unsigned char.
 * @param stream Поток.
 * @return Символ, обратно приведенный к int, или EOF при ошибке.
 */
int fputc(int c, FILE *stream);

/**
 * Вывести символ в поток.
 * @param c Символ, приведенный к unsigned char.
 * @param stream Поток.
 * @return Символ, обратно приведенный к int, или EOF при ошибке.
 */
int putc(int c, FILE *stream);

/**
 * Вывести строку в поток, за исключением нуль-терминатора.
 * @param s Выводимая строка.
 * @param stream Поток.
 * @return Неотрицательное число при успехе, EOF иначе.
 */
int fputs(const char * restrict s, FILE * restrict stream);

/**
 * Вывести символ в stdout.
 * @param c Символ, приведенный к unsigned char.
 * @return Символ, обратно приведенный к int, или EOF при ошибке.
 */
int putchar(int c);

/**
 * Вывести многобайтовый символ в stdout.
 * @param ch Указатель на строку с данными.
 * @param len Количество символов для вывода.
 * @return Количество записанных байт или EOF при ошибке.
 */
int putbytes (char *ch, int len);

/**
 * Вывести строку и завершающий перевод строки в stdout.
 * @param s Выводимая строка.
 * @return Неотрицательное число при успехе, EOF иначе.
 */
int puts(const char *s);

/**
 * Занести символ обратно в поток, сделав его первым к чтению.
 * @param c Символ для занесения.
 * @param stream Поток.
 * @return c при успехе, EOF при ошибке.
 */
int ungetc(int c, FILE *stream);

//! @}

//--------------------------------------------------
/**
 * @name Функции прямого ввода-вывода
 * @{
 */


/**
 * Считать данные из потока.
 * @param ptr Буфер для данных
 * @param size Размер элемента данных.
 * @param nmemb Кол-во элементов данных.
 * @param stream Поток.
 * @return Кол-во успешно считанных элементов (не символов/байт!).
 */
size_t fread(void * restrict ptr, size_t size, size_t nmemb, FILE * restrict stream);

/**
 * Записать данные в поток.
 * @param ptr Буфер для данных
 * @param size Размер элемента данных.
 * @param nmemb Кол-во элементов данных.
 * @param stream Поток.
 * @return Кол-во успешно записанных элементов (не символов/байт!).
 */
size_t fwrite(const void * restrict ptr, size_t size, size_t nmemb, FILE * restrict stream);

//! @}

//--------------------------------------------------
/**
 * @name Функции позиционирования в файлах
 * @{
 */


/**
 * Получить текущую позицию операции ввода-вывода в потоке, сохранив результат в виде типа fpos_t.
 * @param stream Поток.
 * @param pos Буфер для позиции ввода-вывода.
 * @return 0 при успехе, -1 иначе.
 */
int fgetpos(FILE * restrict stream, fpos_t * restrict pos);

/**
 * Установить позицию следующей операции ввода-вывода в потоке, используя результат из функции fgetpos.
 * @param stream Поток.
 * @param pos Позиция ввода-вывода.
 * @return  0 при успехе, -1 иначе.
 */
int fsetpos(FILE *stream, const fpos_t *pos);

/**
 * Установить позицию следующей операции ввода-вывода в потоке.
 * @param stream Поток.
 * @param offset Смещение от начальной позиции.
 * @param whence Начальная позиция: SEEK_SET, SEEK_CUR или SEEK_END.
 * @return 0 при успехе, -1 иначе.
 */
int fseek(FILE *stream, long int offset, int whence);

/**
 * Получить текущую позицию операции ввода-вывода в потоке.
 * @param stream Поток.
 * @return Значение текущего смещения или -1 при ошибке.
 */
long int ftell(FILE *stream);

/**
 * Установить позицию операции ввода-вывода в потоке в начало.
 * @param stream Поток.
 */
void rewind(FILE *stream);

//! @}
//!
//--------------------------------------------------
/**
 * @name Функции обработки ошибок
 * @{
 */

/**
 * Очистить укзатели конца файла и ошибок для потока.
 * @param stream Поток.
 * @note Учет ошибок фактически не ведется.
 */
void clearerr(FILE *stream);

/**
 * Проверить указатель конца файла для потока.
 * @param stream Поток.
 * @return Не-0, если поток указывает на конец файла.
 */
int feof(FILE *stream);

/**
 * Проверить наличие ошибок потока.
 * @param stream Поток.
 * @return Не-0, если у потока присутствуют ошибки.
 * @warning Учет ошибок не ведется, фактически всегда возвращается 0.
 */
int ferror(FILE *stream);

// /**
//  * Вывести в stderr сообщение об ошибке, произошедшей при последнем системном вызове.
//  * @param s Если не-NULL, то строка, которая будет выведена до сообщения об ошибке.
//  */
// void perror(const char *s);

//! @}

//--------------------------------------------------
/**
 * @name Прочие нестандартные функции
 * @{
 */

/**
 * Вывести текст в stderr в соответствии с форматной строкой.
 * @param format Форматная строка.
 * @param ... Аргументы для форматной строки.
 * @return Кол-во фактически напечатанных символов, не включая нуль-терминатор.
 */
int printerr(const char * restrict format, ...)        __attribute__ ((format (printf, 1, 2)));

/**
 * Считать символ из stdin, вернуть ошибку если символ не будет тут же получен.
 * @return Символ, приведенный к int, или EOF при достижении конца файла или при ошибке.
 */
int getch(void);

//! @}
      
#ifdef __cplusplus
}
#endif
      
#endif
