/**
    @addtogroup groupCore
    @{
    @file timer-arm.h
    @brief Управление аппаратными таймерами \b ARM процессоров.
    @details Процессоры \b ARM архитектуры содержат несколько аппаратных
    таймеров. Один из них ВСЕГДА задействован под нужды @osName. Остальные
    могут быть использованы в пользовательском ПО для задания коротких
    (менее 1мс) временных интервалов. Так данный модуль нацелен на создание
    именно коротких временных интервалов, то настройка всех таймеров
    выполняется соответствующим образом --- прочие экзотические способы
    настройки таймеров не используются. Минимальный временной интервал \b 42мкс.
    Большие временные (от 1мс и выше) интервалы рекомендуется создавать,
    используя задержку выполнения текущей задаи taskDelay().
    @}
 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 29.04.2023

*/

#ifndef _TAMER_ARM_H_
#define _TAMER_ARM_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>
#include <intlib.h>


/**
 *  @anchor ancTimerArmOrder
 *  @name Макросы номеров аппаратных таймеров
 *  @brief Синонимы номеров таймеров возможно использовать 
 *  в качестве параметров функций \b timer
 *  для улучшения читаемости кода. Таймер 1 не указан, так
 *  как он задействован под нужны системы.
 *  @{
 */
#define TIMER_0 0 ///< Создание блокирующих задержек udelay() @details Аппаратный таймер 0. @warning Не использовать в пользовательских задачах!
#define TIMER_1 1 ///< Системный таймер @details Аппаратный таймер 1. @warning Не использовать в пользовательских задачах!
#define TIMER_2 2 ///< @details Аппаратный таймер 2.
#define TIMER_3 3 ///< @details Аппаратный таймер 3.
#define TIMER_4 4 ///< @details Аппаратный таймер 4.
#define TIMER_5 5 ///< @details Аппаратный таймер 5.
#define TIMER_SYS (TIMER_1) ///< Синоним для системного таймера.
/// @}



/**
 *  @brief Количество доступных таймеров.
 *  @details Функция возвращает количество доступных
 *  аппаратных таймеров, включая системный.
 *  
 *  @return Количество аппаратных таймеров, либо \ref ERROR,
 *  если инициализация модуля не выполнена.
 */
int taCount ();

/**
 *  @brief Номер системного таймера.
 *  @details Функция возвращает номер аппаратного таймера,
 *  отведённого под нужды операционной системы.
 *  
 *  @return Номер системного таймера, либо \ref ERROR,
 *  если инициализация модуля не выполнена.
 */
int taSystemNumber ();

/**
 *  @brief Задать обработчик прерывания для таймера.
 *  @details Функция задаёт обработчик прерывания для указанного таймера.
 *  
 *  @param timer Номер таймера в пределах от 0 до taCount().
 *  Таймер с номером taSystemNumber() не может быть использован.
 *  @param group Группа прерываний.
 *  @param priority Приоритет прерывания в группе.
 *  @param handler Обработчик прерывания, назначаемый выбранному таймеру.
 *  @return \ref OK, если прерывание задано, иначе
 *  \ref ERROR.
 *  @see Порядок обработки прерываний в зависимости от группы и приоритета
 *  в разделе \ref sectionIntLib.
 */
STATUS taSetInterrupt (
    unsigned int timer, 
    unsigned int group,
    unsigned int priority,
    usr_int_proc handler);

/**
 *  @brief Запуск таймера.
 *  @details Таймер может быть запущен как в непрерывном режиме
 *  так и в режиме одиночного отсчёта. По истечении заданного
 *  интервала времени каждый раз будет формироваться аппаратное 
 *  прерывание. Обработчик прерывания следует задать с помощью
 *  
 *  
 *  @param timer Номер таймера в пределах от 0 до taCount().
 *  Таймер с номером taSystemNumber() не может быть запущен
 *  из пользовательской задачи.
 *  @param period_ns Период работы таймера, либо интервал для
 *  одиночного срабатывания. Задаётся в наносекундах.
 *  @param continuously \ref true --- непрерывный режим работы
 *  таймера, иначе --- одиночное срабатывание.
 *  @return \ref OK, если таймер успешно запущен, иначе
 *  \ref ERROR.
 */
STATUS taStart (
    unsigned int timer,
    unsigned int period_ns,
    bool continuously);

/**
 *  @brief Остановка таймера.
 *  @details Остановка таймера, отключение прерываний.
 *  
 *  @param timer Номер таймера в пределах от 0 до taCount().
 *  Таймер с номером taSystemNumber() не может быть остановлен
 *  из пользовательской задачи.
 *  @return \ref OK, если таймер успешно остановлен, иначе
 *  \ref ERROR.
 */
STATUS taStop (unsigned int timer);


/**
 *  @name Wathcdog timer
 *  @{
 */

/**
 *  @brief Запуск \b watchdog таймера.
 *  @details Функция запускает \b watchdog таймер 
 *  и устанавливает интервал срабатывания.
 *  
 *  @param time Время в секундах до аппаратного перезапуска системы.
 *  Минимальное значение \b 0 -- соответствует интервалу 0,5 секунды.
 */
void wdtStart (unsigned int time);

/**
 *  @brief Перезапуск \b watchdog таймера.
 *  @details Функция запускает \b watchdog таймер.
 *  Вызывать функцию следует с периодом меньшим, чем интервал
 *  указанный при запуске таймера wdtStart(). 
 *  
 */
void wdtRestart ();
/// @}



/// @privatesection

// Системные функции

/**
 *  @brief Чтение конфигурации аппаратной части.
 *  @details Чтение текущей конфигурации аппаратной части,
 *  настройка модуля соответствующим образом.
 *  Вызывается при старте операционной системы
 *  после загрузки конфигурации.
 *  @return \ref OK если в конфигурации найдены
 *  нужные параметры (известный тип процессора),
 *  иначе \ref ERROR.
 */
STATUS taArchInit ();




#ifdef __cplusplus
}
#endif

#endif
