/**
    @addtogroup groupDrivers
    @{
    @file udp.h
    @brief Сетевой протокол UDP
    @details Работа с сетью по протоколу \b UDP.

    <b>Подключение</b>: @verbatim #include <udp.h> @endverbatim

    @see Настройка \ref sectionMakefile "Makefile" и файла конфигурации 
    \ref sectionConfig "config.h" в разделе 
    \ref sectionNetWorkInclude "Подключение сетевой подсистемы".

    Пример работы с сетью по UDP:
    @code
    #include <udp.h>
    
    static char my_ip[] = "10.0.3.40";
    static char target_ip[] = "10.0.3.41";
    static unsigned int port = 1243;

    // Функция приёма пакетов
    void udpReceiveService (IP_PACKET *ipp) {
        if (ipp->s_addr != strtoip (my_ip))
            return;
        void *data = udpGetDataPointer (ipp);
        // ... Некие действия с принятыми данными data

    }

    void startUdp () {
        // Задать адрес устройства
        setPrimaryIpAddress (strtoip (my_ip));
        // Регистрация функции приёма данных
        udpRegisterService (port, udpReceiveService);
        int dataLen = 512;
        char data[dataLen];
        // ... Заполнение отправляемого массива data
        udpSendPacket (data, dataLen, port, port, strtoip (target_ip));
    }
    @endcode


    @}
 */

/*

     Internet Protocol User Datagram protocol
     Author: Sergey V.Sokolov
     05.2003
*/


#ifndef UDP_H_
#define UDP_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>

/**
 *  @name Описание заголовков пакетов.
 *  @{
 */

/**
 *  @brief Заголовок пакета \b UDP.
 *  @details Заголовок пакета \b UDP. 
 *  Порядок следования байт -- старшим байтом вперёд.
 *  Для чтения значений байты следует менять местами.
 */
typedef struct udp_hdr {
    unsigned short udp_s_port;  ///< Порт отправителя.
    unsigned short udp_d_port;  ///< Порт получателя.
    unsigned short udp_len;     ///< Длина пакета \b UDP.
    unsigned short udp_sum;     ///< Контрольная сумма.
} UDP_HDR;

/**
 *  @brief Длина заголовка пакета \b UDP.
 */
#define SIZEOF_UDPHDR 8

/**
 *  @brief Заголовок пакета \b IP.
 *  @details Заголовок пакета \b IP. 
 *  Порядок следования байт -- младшим байтом вперёд.
 */
typedef struct ip_packet {
    unsigned int length;    ///< Длина пакета \b IP.
    unsigned char protocol; ///< 
    unsigned int s_addr;    ///< Адрес отправителя.
    unsigned int d_addr;    ///< Адрес отправителя.
    char *data;     ///< Указатель на заголовок полученных данных \ref UDP_HDR.
} IP_PACKET;


/// @}


/**
 *  @name Настройка параметров устройства
 *  @{
 */

/**
 *  @brief Установить \b IP адрес устройства.
 *  @details Функция позволяет изменить \b IP адрес устройства.
 *  @param ip Первичный \b IP адрес.
 */
void setPrimaryIpAddress (unsigned int ip);

/**
 *  @brief Получить \b IP адрес устройства.
 *  @details Функция позволяет получить текущий \b IP адрес устройства. 
 *  @return ip Первичный \b IP адрес.
 */
unsigned int getPrimaryIpAddress (void);


/**
 *  @brief Перевод строки \b IP адреса в целое число.
 *  @details Перевод строки \b IP адреса в целое число.
 *  
 *  @param IPAddr Указатель на строку, содержащую \b IP адрес.
 *  @return Целое число.
 */
unsigned int strtoip (const char *IPAddr);

/**
 *  @brief Перевод целого числа в строку \b IP адреса.
 *  @details Функция переводит целое число в строку \b IP адреса.
 *  
 *  @param IP Беззнаковое целое.
 *  @param ipstr Строка, в которую будет помещён результат перевода.
 *  Размер предоставляемого массива должен быть не менее 16 байт.
 */
void iptostr(unsigned int IP, char *ipstr);


/// @}


/**
 *  @name Служба приёма пакетов
 *  @{
 */

/**
 *  @brief Функция приёма пакета по \b UDP.
 *  @details Функцию приём следует зарегистрировать с помощью udpRegisterService().
 *  
 */
typedef void (*udpServRout) (IP_PACKET *ipp);

typedef struct udp_service {
    unsigned short port;
    udpServRout service;
    struct udp_service *next;
} UDP_SERVICE;



/**
 *  @brief Зарегистрировать службу приёма пакетов UDP.
 *  @details Подключение функции разбора входящих пакетов 
 *  к заданному порту. Функция будет вызываться в системной \b задаче
 *  разбора принимаемых пакетов.
 *  
 *  @param port Порт, на который подключается функция.
 *  @param service Подключаемая функция приёма пакетов.
 */
void udpRegisterService (unsigned short port, udpServRout service);

/**
 *  @brief Отключить службу приёма пакетов UDP.
 *  @details Отключение всех подключенных функций приёма пакетов от всех портов.
 *  
 */
void udpKillServices (void);

/// @}


/**
 *  @name Отправка пакетов
 *  @{
 */


/**
 *  @brief Отправка пакета UDP.
 *  @details Функция отправляет пакет и дожидается конца отправки.
 *  
 *  @param buf Указатель на передаваемые данные.
 *  @param len Размер передаваемых данных в байтах.
 *  @param sport Номер порта отправителя.
 *  @param dport Номер порта получателя.
 *  @param host Адрес порта, получаемый с помощью strtoip().
 *  @return \ref OK при удачной отправке пакета, иначе \ref ERROR.
 */
STATUS udpSendPacket (const void *buf, unsigned short len, unsigned short sport, unsigned short dport, unsigned int host);


/// @}

/**
 *  @name Обработка полученных данных
 *  @{
 */


/**
 *  @brief Получить указатель на данные \b UDP.
 *  @details Функция возвращает указатель на данные 
 *  \b UDP пакета внутри пакета \b IP.
 *  
 *  @param ipp Пакет \b IP.
 *  @return Указатель на данные, которые следуют 
 *  сразу после заголовка \ref UDP_HDR.
 */
void *udpGetDataPointer (IP_PACKET *ipp);

/**
 *  @brief Получить указатель на заголовок \b UDP.
 *  @details Функция возвращает указатель на заголовок 
 *  \b UDP пакета внутри пакета \b IP.
 *  
 *  @param ipp Пакет \b IP.
 *  @return Указатель на заголовок \b UDP в пакете \b IP.
 */
UDP_HDR *udpGetUdpHeader (IP_PACKET *ipp);


/// @}





/// @privatesection

void udpProtoHandler (int idx, IP_PACKET *p);
void udpEcho (IP_PACKET *p);




#ifdef __cplusplus
}
#endif
#endif  // ifndef UDP_H_
