/**
    @addtogroup groupUSB
    @{
    @file usb.h
    @brief Методы работы с \b USB
    @details Файл содержит описание методов работы с \b USB в ОС @osName.

    <b>Подключение</b>: @verbatim #include <usb.h> @endverbatim
    \ref sectionConfig "config.h": @verbatim #define INCLUDE_USB @endverbatim
    \ref sectionMakefile "Makefile": @verbatim LIBRARIES += -l_usbsunxi @endverbatim

    @see Общее описание работы с \b USB в главе \ref pageUSB.
    @}


*/

/*
 * (C) Copyright 2001
 * Denis Peter, MPL AG Switzerland
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * Note: Part of this code has been derived from linux
 *
 */
#ifndef _USB_H_
#define _USB_H_

#include "usb_defs.h"
#include "usbdescriptors.h"

//--------------------------------------------------------------------
/**
 *  @name Конфигурация для OMAP3
 *  @{
 */
#define ARCH_DMA_MINALIGN	64 
#define __LITTLE_ENDIAN
#define CONFIG_USB_EHCI
/// @}

/*
 * The EHCI spec says that we must align to at least 32 bytes.  However,
 * some platforms require larger alignment.
 */
/**
 *  @name Настройки USB
 *  @{
 */
#if ARCH_DMA_MINALIGN > 32
#define USB_DMA_MINALIGN	ARCH_DMA_MINALIGN
#else
#define USB_DMA_MINALIGN	32
#endif

/* Everything is aribtrary */
#define USB_ALTSETTINGALLOC		4
#define USB_MAXALTSETTING		128	///< Hard limit.

#define USB_MAX_DEVICE			32
#define USB_MAXCONFIG			8
#define USB_MAXINTERFACES		8
#define USB_MAXENDPOINTS		16
#define USB_MAXCHILDREN			8	///< This is arbitrary.
#define USB_MAX_HUB			16

#define USB_CNTL_TIMEOUT 100 ///< 100 ms timeout

/*
 * This is the timeout to allow for submitting an urb in ms. We allow more
 * time for a BULK device to react - some are slow.
 */
#define USB_TIMEOUT_MS(pipe) (usb_pipebulk(pipe) ? 5000 : 1000)
/// @}

/* device request (setup) */
/// @private
struct devrequest {
	unsigned char	requesttype;
	unsigned char	request;
	unsigned short	value;
	unsigned short	index;
	unsigned short	length;
} __attribute__ ((packed));


/* All standard descriptors have these 2 fields in common */
/// @private
struct usb_descriptor_header {
	unsigned char	bLength;
	unsigned char	bDescriptorType;
} __attribute__ ((packed));


/**
    @brief Структура дескриптора интерфейса.
    @details Это вспомогательная структура, которая содержит дескриптор
    интерфейса, число конечных точек для данного интерфейса, число альтернативных
    установок и дескрипторы для всех конечных точек интерфейса.
 */
struct usb_interface {
	struct usb_interface_descriptor desc;   ///< @details Дескриптор интерфейса.
	unsigned char	no_of_ep;               ///< @details Количество конечных точек.
	unsigned char	num_altsetting;         ///< @details Число альт. установок.
	unsigned char	act_altsetting;         ///< @details Актуальная установка.
	struct usb_endpoint_descriptor ep_desc[USB_MAXENDPOINTS]; ///< @details Дескрипторы конечных точек.
} __attribute__ ((packed));

/**
    @brief Структура дескриптора конфигурации.

    @details Это вспомогательная структура, которая содержит дескриптор
    конфигурации, количество интерфейсов для устройства и массив структур
    \ref usb_interface для всех интерфейсов устройства.
 */
struct usb_config {
	struct usb_configuration_descriptor desc;           ///< @details Дескриптор конфигурации.
	unsigned char	no_of_if;                           ///< @details Число интерфейсов.
	struct usb_interface if_desc[USB_MAXINTERFACES];    ///< @details Массив структур \ref usb_interface
                                                        ///  для всех интерфейсов устройства.
} __attribute__ ((packed));

/**
 *  @brief Размер пакета для обмена с USB-устройством.
 *  @details Используется для определения размера пакета
 *  в структуре \ref usb_device.
 *  
 */
enum {
	PACKET_SIZE_8   = 0,    ///< 8 байт.
	PACKET_SIZE_16  = 1,    ///< 16 байт.
	PACKET_SIZE_32  = 2,    ///< 32 байта.
	PACKET_SIZE_64  = 3,    ///< 64 байта.
};


/**
 *  @brief Структура данных о подключении USB-устройства.
 */
struct usb_device {
	int	devnum;         ///< @details Номер устройства на шине USB.
    char *devname;      ///< @details Указатель на строку имени устройства.
	int	speed;			///< @details Скорость подключения (full/low/high).
	char	mf[32];		///< @details Производитель устройства.
	char	prod[32];	///< @details Заводское наименование.
	char	serial[32];	///< @details Серийный номер.

	int maxpacketsize;  ///< @details Максимальный размер пакета. Одно из значений: \ref PACKET_SIZE_8, \ref PACKET_SIZE_16, \ref PACKET_SIZE_32, \ref PACKET_SIZE_64.
	unsigned int toggle[2]; ///< @details По одному биту на каждую конечную точку ([0] = IN, [1] = OUT).
	/* endpoint halts; one bit per endpoint # & direction;
	 * [0] = IN, [1] = OUT
	 */
	unsigned int halted[2]; ///< @details Признак останова конченой точки.
	int epmaxpacketin[16];  ///< @details INput endpoint specific maximums.
	int epmaxpacketout[16]; ///< @details OUTput endpoint specific maximums.

	int configno;
	struct usb_device_descriptor descriptor
		__attribute__((aligned(ARCH_DMA_MINALIGN)));    ///< @details Дескриптор устройства.
	struct usb_config config;   ///< @details Конфигурационный дескриптор.

	int have_langid;    ///< @details Признак является ли /b string_langid еще действительным.
	int string_langid;  ///< @details Идентификатор языка для строк.
	int (*irq_handle)(struct usb_device *dev);
	unsigned long irq_status;
	int irq_act_len;    ///< @details Переданные байты.
	void *privptr;
	/*
	 * Child devices -  if this is a hub device
	 * Each instance needs its own set of data structures.
	 */
	unsigned long status;
	int act_len;    ///< @details Переданные байты.
	int maxchild;   ///< @details Количество портов, если это концентратор.
	int portnr;     
	struct usb_device *parent;
	struct usb_device *children[USB_MAXCHILDREN];
    
    void *hcd;
    struct usb_driver *driver;
    void *irq_q;
};

/*
 * this is how the lowlevel part communicate with the outer world
 */

#if defined(CONFIG_USB_UHCI) || defined(CONFIG_USB_OHCI) ||            \
    defined(CONFIG_USB_EHCI) || defined(CONFIG_USB_OHCI_NEW) ||        \
    defined(CONFIG_USB_SL811HS) || defined(CONFIG_USB_ISP116X_HCD) ||  \
    defined(CONFIG_USB_R8A66597_HCD) || defined(CONFIG_USB_DAVINCI) || \
    defined(CONFIG_USB_OMAP3) || defined(CONFIG_USB_DA8XX) ||          \
    defined(CONFIG_USB_BLACKFIN) || defined(CONFIG_USB_AM35X)

/**
 *  @name Работа с конечными точками
 *  @{
 */

/// @privatesection
int wait_bulk_msg (struct usb_device *dev, int id, int *act_len);
int submit_bulk_msg (struct usb_device *dev, unsigned long pipe,
                     void *buffer, int transfer_len, int timeout);

int submit_control_msg (struct usb_device *dev, unsigned long pipe, void *buffer,
                        int transfer_len, struct devrequest *setup);

void usb_scan_devices (void *hcd);
int usb_get_port_status (struct usb_device *dev, int port, void *data);
int usb_set_port_feature (struct usb_device *dev, int port, int feature);
void usb_disconnect_root (int hcdnum);
int usb_clear_port_feature (struct usb_device *dev, int port, int feature);

/// @publicsection
/**
 *  @brief Подключить обработчик прерывания к конечной точке типа \b interrupt.
 *  @details Функция позволяет подключать обработчик прерывания 
 *  к конечной точке типа \b interrupt.
 *  Чтобы подключить обработчик на прерывание от конечной точки, 
 *  нужно перед этой командой записать следующую строку:
 *  @code
 *  dev->irq_handle = usb_my_irq;
 *  @endcode
 *  Где \b usb_my_irq – функция вида:
 *  @code
 *  int usb_my_irq(struct usb_device *dev);
 *  @endcode
 *  Для многократного входа в прерывание эта функция должна вернуть
 *  ненулевое значение. Так, например, для драйвера указателя типа <em>мышь</em>
 *  необходимо записать следующие строки:
 *  @code
 *  // --- Установка обработчика ---
 *  // Обработчик прерывания мыши
 *  dev->irq_handle = usb_mouse_irq;
 * 
 *  // Сформировать pipe
 *  unsigned int pipe = usb_rcvintpipe(dev, ep->bEndpointAddress);
 * 
 *  // Определить размер пакета
 *  int maxp = usb_maxpacket(dev, pipe);
 * 
 *  // Подключить обработчик
 *  usb_submit_int_msg(dev, pipe, data, maxp, ep->bInterval);
 *  @endcode
 *  
 *  @param dev Устройство USB.
 *  @param pipe Интегральный параметр \ref sectionUsbPipe "pipe".
 *  @param buffer Указатель на буфер.
 *  @param transfer_len Длина в байтах
 *  @param interval Период обменов в микрофреймах (~1мс).
 *  @return \ref OK при успешном завершении, или код ошибки.
 */
int submit_int_msg(struct usb_device *dev, unsigned long pipe, void *buffer,
			int transfer_len, int interval);

/**
 *  @brief Передача информации к конечной точке типа \b bulk и от неё.
 *  @details Эта функция позволяет передавать информацию на конечную точку
 *  и принимать информацию от конечной точки типа \b bulk.
 *  
 *  @param dev Устройство USB.
 *  @param pipe Интегральный параметр \ref sectionUsbPipe "pipe".
 *  @param data Указатель на буфер данных.
 *  @param len Длина в байтах.
 *  @param actual_length Фактическая длина данных.
 *  @param timeout Таймаут в миллисекундах.
 *  @return \ref OK при успешном завершении, или код ошибки.
 */
int usb_bulk_msg (struct usb_device *dev, unsigned int pipe,
                  void *data, int len, int *actual_length, int timeout);


/// @}


/* Defines */
/**
 *  @name Настройка идентификации устройства.
 *  @{
 */
#define USB_UHCI_VEND_ID	0x8086  ///< @details Определение идентификатора поставщика для устройства.
#define USB_UHCI_DEV_ID		0x7112  ///< @details Определение индекса строкового дескриптора.
/// @}

#else
#error USB Lowlevel not defined
#endif

#ifdef CONFIG_USB_STORAGE

#define USB_MAX_STOR_DEV 5
block_dev_desc_t *usb_stor_get_dev(int index);
int usb_stor_scan(int mode);
int usb_stor_info(void);

#endif

#ifdef CONFIG_USB_HOST_ETHER

#define USB_MAX_ETH_DEV 5
int usb_host_eth_scan(int mode);

#endif

#include <hcd/hcd.h>

// Функции работы с USB на уровне системы.
/// @privatesection

int drv_usb_kbd_init (void *hcd);
int drv_usb_mouse_init (void *hcd);
int drv_usb_storage_init (void *hcd);

/* routines */
int usb_init (void); /* initialize the USB Controller */
int usb_stop (void); /* stop the USB Controller */

int usb_set_protocol (struct usb_device *dev, int ifnum, int protocol);
int usb_set_idle (struct usb_device *dev, int ifnum, int duration,
                  int report_id);
struct usb_device *usb_get_dev_index (struct ehci_hcd *h, int index);
int usb_control_msg (struct usb_device *dev, unsigned int pipe,
                     unsigned char request, unsigned char requesttype,
                     unsigned short value, unsigned short index,
                     void *data, unsigned short size, int timeout);
int usb_submit_int_msg (struct usb_device *dev, unsigned long pipe,
                        void *buffer, int transfer_len, int interval);
int usb_disable_asynch (int disable);
int usb_maxpacket (struct usb_device *dev, unsigned long pipe);
int usb_get_configuration_no (struct usb_device *dev, unsigned char *buffer,
                              int cfgno);
int usb_get_report (struct usb_device *dev, int ifnum, unsigned char type,
                    unsigned char id, void *buf, int size);
int usb_get_class_descriptor (struct usb_device *dev, int ifnum,
                              unsigned char type, unsigned char id, void *buf,
                              int size);
int usb_clear_halt (struct usb_device *dev, int pipe);
int usb_string (struct usb_device *dev, int index, char *buf, size_t size);
int usb_get_hub_descriptor (struct usb_device *dev, void *data, int size);

/// @publicsection
/**
 *  @name Функции работы с управляющей конечной точкой
 *  @brief Для работы с управляющей конечной точкой используются <b>control message</b>.
 *  @{
 */

/**
 *  @brief Задать интерфейс устройства.
 *  @details Функция устанавливает требуемый интерфейс для устройства по номеру.
 *  
 *  @param dev Устройство USB.
 *  @param interface Интерфейс, который будет обновлён.
 *  @param alternate Выбранная настройка.
 *  @return \ref OK при успешном завершении, или код ошибки.
 */
int usb_set_interface (struct usb_device *dev, int interface, int alternate);

/**
 *  @brief Задать конфигурацию устройства.
 *  @details Функция устанавливает заданную конфигурацию устройства.
 *  
 *  @param dev Устройство USB.
 *  @param configuration Выбранный номер конфигурации.
 *  @return \ref OK при успешном завершении, или код ошибки.
 */
int usb_set_configuration(struct usb_device *dev, int configuration);

/**
 *  @brief Задать протокол устройства.
 *  @details Функция устанавливает заданный протокол работы устройства.
 *  
 *  @param dev Устройство USB.
 *  @param ifnum Номер \b Тре.
 *  @param protocol Требуемый протокол.
 *  @return \ref OK при успешном завершении, или код ошибки.
 */
int usb_set_protocol (struct usb_device *dev, int ifnum, int protocol);

/**
 *  @brief Отправить обобщенное управляющее сообщение.
 *  @details Функция отправляет обобщенное управляющее сообщение устройству.
 *  
 *  @param dev Устройство USB.
 *  @param pipe См. \ref sectionUsbPipe.
 *  @param request Код запроса.
 *  @param requesttype Тип запроса.
 *  @param value Значение.
 *  @param index Индекс.
 *  @param data Указатель на данные.
 *  @param size Размер данных.
 *  @param timeout Таймаут запроса.
 *  @return \ref OK при успешном завершении, или код ошибки.
 */
int usb_control_msg (struct usb_device *dev, unsigned int pipe,
                     unsigned char request, unsigned char requesttype,
                     unsigned short value, unsigned short index,
                     void *data, unsigned short size, int timeout);

/// @}


/**
 *  @name Макросы сдвига переменных
 *  @brief Конвертация переменных <b>big endian -> little endian</b>.
 *  Некоторые процессоры, например \b ARM920T, 
 *  исходно работают со значениями \b little \b endian.
 *  @{
 */
#define __swap_16(x) \
	({ unsigned short x_ = (unsigned short)x; \
	 (unsigned short)( \
		((x_ & 0x00FFU) << 8) | ((x_ & 0xFF00U) >> 8)); \
	})
#define __swap_32(x) \
	({ unsigned long x_ = (unsigned long)x; \
	 (unsigned long)( \
		((x_ & 0x000000FFUL) << 24) | \
		((x_ & 0x0000FF00UL) <<	 8) | \
		((x_ & 0x00FF0000UL) >>	 8) | \
		((x_ & 0xFF000000UL) >> 24)); \
	})

#ifdef __LITTLE_ENDIAN
# define swap_16(x) (x)
# define swap_32(x) (x)
#else
# define swap_16(x) __swap_16(x)
# define swap_32(x) __swap_32(x)
#endif
/// @}

/*
 * Calling this entity a "pipe" is glorifying it. A USB pipe
 * is something embarrassingly simple: it basically consists
 * of the following information:
 *  - device number (7 bits)
 *  - endpoint number (4 bits)
 *  - current Data0/1 state (1 bit)
 *  - direction (1 bit)
 *  - speed (2 bits)
 *  - max packet size (2 bits: 8, 16, 32 or 64)
 *  - pipe type (2 bits: control, interrupt, bulk, isochronous)
 *
 * That's 18 bits. Really. Nothing more. And the USB people have
 * documented these eighteen bits as some kind of glorious
 * virtual data structure.
 *
 * Let's not fall in that trap. We'll just encode it as a simple
 * unsigned int. The encoding is:
 *
 *  - max size:		bits 0-1	(00 = 8, 01 = 16, 10 = 32, 11 = 64)
 *  - direction:	bit 7		(0 = Host-to-Device [Out],
 *					(1 = Device-to-Host [In])
 *  - device:		bits 8-14
 *  - endpoint:		bits 15-18
 *  - Data0/1:		bit 19
 *  - speed:		bit 26		(0 = Full, 1 = Low Speed, 2 = High)
 *  - pipe type:	bits 30-31	(00 = isochronous, 01 = interrupt,
 *					 10 = control, 11 = bulk)
 *
 * Why? Because it's arbitrary, and whatever encoding we select is really
 * up to us. This one happens to share a lot of bit positions with the UHCI
 * specification, so that much of the uhci driver can just mask the bits
 * appropriately.
 */
/* Create various pipes... */

/**
 *  @anchor ancUsbPipeDefs
 *  @name Создание интегрального параметра pipe
 *  @brief Подробнее см в разделе \ref sectionUsbPipe.
 *  @{
 */
#define create_pipe(dev,endpoint) \
		(((dev)->devnum << 8) | ((endpoint) << 15) | \
		((dev)->speed << 26) | (dev)->maxpacketsize)
#define default_pipe(dev) ((dev)->speed << 26)

#define usb_sndctrlpipe(dev, endpoint)	((PIPE_CONTROL << 30) | \
					 create_pipe(dev, endpoint))
#define usb_rcvctrlpipe(dev, endpoint)	((PIPE_CONTROL << 30) | \
					 create_pipe(dev, endpoint) | \
					 USB_DIR_IN)
#define usb_sndisocpipe(dev, endpoint)	((PIPE_ISOCHRONOUS << 30) | \
					 create_pipe(dev, endpoint))
#define usb_rcvisocpipe(dev, endpoint)	((PIPE_ISOCHRONOUS << 30) | \
					 create_pipe(dev, endpoint) | \
					 USB_DIR_IN)
#define usb_sndbulkpipe(dev, endpoint)	((PIPE_BULK << 30) | \
					 create_pipe(dev, endpoint))
#define usb_rcvbulkpipe(dev, endpoint)	((PIPE_BULK << 30) | \
					 create_pipe(dev, endpoint) | \
					 USB_DIR_IN)
#define usb_sndintpipe(dev, endpoint)	((PIPE_INTERRUPT << 30) | \
					 create_pipe(dev, endpoint))
#define usb_rcvintpipe(dev, endpoint)	((PIPE_INTERRUPT << 30) | \
					 create_pipe(dev, endpoint) | \
					 USB_DIR_IN)
#define usb_snddefctrl(dev)		((PIPE_CONTROL << 30) | \
					 default_pipe(dev))
#define usb_rcvdefctrl(dev)		((PIPE_CONTROL << 30) | \
					 default_pipe(dev) | \
					 USB_DIR_IN)
/// @}


/**
 *  @anchor ancUsbPipeWorkDefs
 *  @name Работа с интегральным параметром pipe
 *  @brief Подробнее см в разделе \ref sectionUsbPipe.
 *  @{
 */

/* The D0/D1 toggle bits */
#define usb_gettoggle(dev, ep, out) (((dev)->toggle[out] >> ep) & 1)
#define usb_dotoggle(dev, ep, out)  ((dev)->toggle[out] ^= (1 << ep))
#define usb_settoggle(dev, ep, out, bit) ((dev)->toggle[out] = \
						((dev)->toggle[out] & \
						 ~(1 << ep)) | ((bit) << ep))

/* Endpoint halt control/status */
#define usb_endpoint_out(ep_dir)	(((ep_dir >> 7) & 1) ^ 1)
#define usb_endpoint_halt(dev, ep, out) ((dev)->halted[out] |= (1 << (ep)))
#define usb_endpoint_running(dev, ep, out) ((dev)->halted[out] &= ~(1 << (ep)))
#define usb_endpoint_halted(dev, ep, out) ((dev)->halted[out] & (1 << (ep)))

#define usb_packetid(pipe)	(((pipe) & USB_DIR_IN) ? USB_PID_IN : \
				 USB_PID_OUT)

#define usb_pipeout(pipe)	((((pipe) >> 7) & 1) ^ 1)
#define usb_pipein(pipe)	(((pipe) >> 7) & 1)
#define usb_pipedevice(pipe)	(((pipe) >> 8) & 0x7f)
#define usb_pipe_endpdev(pipe)	(((pipe) >> 8) & 0x7ff)
#define usb_pipeendpoint(pipe)	(((pipe) >> 15) & 0xf)
#define usb_pipedata(pipe)	(((pipe) >> 19) & 1)
#define usb_pipespeed(pipe)	(((pipe) >> 26) & 3)
#define usb_pipeslow(pipe)	(usb_pipespeed(pipe) == USB_SPEED_LOW)
#define usb_pipetype(pipe)	(((pipe) >> 30) & 3)
#define usb_pipeisoc(pipe)	(usb_pipetype((pipe)) == PIPE_ISOCHRONOUS)
#define usb_pipeint(pipe)	(usb_pipetype((pipe)) == PIPE_INTERRUPT)
#define usb_pipecontrol(pipe)	(usb_pipetype((pipe)) == PIPE_CONTROL)
#define usb_pipebulk(pipe)	(usb_pipetype((pipe)) == PIPE_BULK)
/// @}


/*
 * Hub Stuff
 */
/// @private
struct usb_port_status {
	unsigned short wPortStatus;
	unsigned short wPortChange;
} __attribute__ ((packed));

/// @private
struct usb_hub_status {
	unsigned short wHubStatus;
	unsigned short wHubChange;
} __attribute__ ((packed));


/* Hub descriptor */
/// @private
struct usb_hub_descriptor {
	unsigned char  bLength;
	unsigned char  bDescriptorType;
	unsigned char  bNbrPorts;
	unsigned short wHubCharacteristics;
	unsigned char  bPwrOn2PwrGood;
	unsigned char  bHubContrCurrent;
	unsigned char  DeviceRemovable[(USB_MAXCHILDREN+1+7)/8];
	unsigned char  PortPowerCtrlMask[(USB_MAXCHILDREN+1+7)/8];
	/* DeviceRemovable and PortPwrCtrlMask want to be variable-length
	   bitmaps that hold max 255 entries. (bit0 is ignored) */
} __attribute__ ((packed));


/// @private
struct usb_hub_device {
	struct usb_device *pusb_dev;
	struct usb_hub_descriptor desc;
};

/*
 *  Функции USB-Hub
 */
/// @privatesection
int usb_hub_probe(struct usb_device *dev);
void usb_hub_reset(void);
int hub_port_reset(struct usb_device *dev, int port,
			  unsigned short *portstat);

struct usb_device *usb_alloc_new_device(void *hcd);
int usb_new_device(struct usb_device *dev);

extern int USB_CONNECTION_STATE;

#include "usb_driver.h"

/// @publicsection
/**
 *  @name Проверка подключения устройства.
 *  @{
 */

#define UCS_SOMETHING_IS_CONNECTED (0x01)   ///< @details Подключено какое-то устройство, отключение не отслеживается.
#define UCS_FLASHDRIVE_IS_CONNECTED (0x02)  ///< @details Подключена флешка, отключение не отслеживается.
#define UCS_KEYBOARD_IS_CONNECTED (0x04)    ///< @details Подключена клавиатура.

/**
 *  @brief Проверка подключения устройства.
 *  @details Функция проверки подключения устройства.
 *  Определение подключения осуществляется проверкой, 
 *  взведен ли соответствующий бит из набора \ref UCS_SOMETHING_IS_CONNECTED,
 *  \ref UCS_FLASHDRIVE_IS_CONNECTED, \ref UCS_KEYBOARD_IS_CONNECTED.
 */
int usb_connection_status();
/// @}

/**
 * @brief Get volume letter of last mounted flash drive.
 * @return Volume letter of last mounted flash drive.
 * @private
 */
char usb_GetFlashDriveLetter(void);

#endif /*_USB_H_ */
