/**
    @addtogroup groupStdTypes
    @{
    @file vector.h
    @brief Работа с массивами данных типа \b Вектор.
    @details Вектор --- динамический массив, 
    обеспечивающий быстрое добавление новых элементов 
    в конец и меняющий свой размер при необходимости.
    @}
 */

/*
 * @date Created on: 12 февр. 2020 г.
 * @author Dubovich Timofey
*/

#ifndef __VECTOR_H_
#define __VECTOR_H_

/**
 * Структура вектора - динамически расширяющегося массива.
 */
typedef struct
{
    void* Data;                     //!< @details Указатель на блок памяти с данными.
    size_t ElementSize;             //!< @details Размер элемента.
    size_t CurrentElementsAmount;   //!< @details Текущее кол-во элементов в векторе.
    size_t MaxElementsAmount;       //!< @details Максимальное кол-во элементов в векторе (может быть автоматически увеличено при добавлении элементов).
}sVector;

/**
 * Создать вектор - динамически расширяющийся массив.
 * @param elementSize Размер одного элемента в векторе.
 * @param elementsAmount Примерное кол-во элементов в векторе, которое нужно будет туда добавить.
 * @return Указатель на структуру вектора или NULL при ошибке.
 */
sVector* vector_Create(size_t elementSize, size_t elementsAmount);

/**
 * Добавить новый элемент в вектор.
 * @param vPtr Указатель на структуру вектора.
 * @param element Адрес элемента, который должен быть добавлен в вектор.
 * @return OK при успехе, ERROR иначе.
 * @note При надобности и невозможности расширения массива будет возвращен ERROR. Тем не менее, массивом можно будет продолжать пользоваться.
 */
STATUS vector_Add(sVector* vPtr, const void* element);

/**
 * Add new element to the vector. Initialize it with zeros.
 * @param vPtr Pointer to vector.
 * @return OK if succeeded, ERROR otherwise.
 */
STATUS vector_AddEmpty(sVector* vPtr);

/**
 * Удалить последний добавленный элемент из вектора (размер при этом НЕ будет пересчитан в меньшую сторону).
 * @param vPtr Указатель на структуру вектора.
 * @return OK при успехе, ERROR иначе.
 */
STATUS vector_RemoveLast(sVector* vPtr);

/**
 * Delete element form vector (memory will not be reallocated!).
 * @param vPtr Pointer to vector.
 * @param elementIndex Index of target element.
 * @return OK if succeeded, ERROR otherwise.
 */
STATUS vector_Remove(sVector* vPtr, size_t elementIndex);

/**
 * Получить последний элемент в векторе.
 * @param vPtr Указатель на элемент или NULL.
 */
void* vector_GetLast(sVector* vPtr);

/**
 * Получить первый элемент в векторе.
 * @param vPtr Указатель на элемент или NULL.
 */
void* vector_GetFirst(sVector* vPtr);

/**
 * Получить элемент из вектора.
 * @param vPtr Указатель на структуру вектора.
 * @param elementIndex Индекс элемента, который нужно извлечь.
 * @return Указатель на элемент или NULL, при ошибках.
 * @note Если будет запрошен 10 элемент из массива на 5 элементов, то функция вернет NULL.
 * @details Если в векторе хранится тип Type, то при отправляться туда должны &TypeVar, а полученные указатели кастоваться в Type*.
 */
void* vector_Get(sVector* vPtr, size_t elementIndex);

/**
 * Освободить вектор и хранящиеся в нем данные.
 * @param vPtr Указатель на структуру вектора.
 */
void vector_Free(sVector* vPtr);

/**
 * Скопировать содержимое одного вектора в другой.
 * @param dstVPtr Вектор, в который проводится копирование.
 * @param srcVPtr Копируемый вектор.
 * @return OK при успехе, ERROR иначе.
 * @details Содержимое dstVPtr будет утеряно!
 */
STATUS vector_Copy(sVector* dstVPtr, const sVector* srcVPtr);

/**
 * Создать копию вектора. Копию нужно будет освободить отдельно от исходного вектора.
 * @param vPtr Указатель на исходный вектор.
 * @return Указатель на структуру вектора или NULL при ошибке.
 */
sVector* vector_Duplicate(const sVector* vPtr);

#endif /* __VECTOR_H_ */
